# E.S.O. - VLT project/ESO Archive
# $Id 
#
# AstroCat.tcl - user interface class for viewing catalog info
#
# See man page AstroCat(1) for a complete description.
#
# who         when       what
# --------   ---------   ----------------------------------------------
# A.Brighton 14 Dec 95   created


itk::usual AstroCat {}


# create a class for the application

class AstroCat {
    inherit util::TopLevelWidget


    # constructor

    constructor {args} {
	eval itk_initialize $args
    }

    
    # destructor - delete C++ based objects so that the temp image
    # files are deleted

    destructor {
	catch {$w_.cat delete}
	catch {$w_.wcs delete}
	catch {unset instances_($instance_idx_)}
	if {"$current_instance_" == "$w_"} {
	    set current_instance_ {}
	}
    }

    
    # called after options have been evaluated

    method init {} {
	global ::env
	wm title $w_ "Catalog Interfaces"
        wm geometry $w_ +10+10

	if {$itk_option(-standalone)} {
	    rtd::setXdefaults
	    cat::setXdefaults
	}

	# if no -image option was specified, look for a running skycat and use tk send
	if {"$itk_option(-image)" == ""} {
	    find_skycat
	} 

	# open log file used to keep track of URLs (for debugging)
	set dir $env(HOME)/.skycat
	# XXX tmp: also remove old log files from previous version
	catch {exec rm -f ${dir}_logo ${dir}log $dir}
	if {! [file isdirectory $dir]} {
	    catch {mkdir $dir}
	}
	catch {
	    set logfile_ [open $dir/log w]
	    fcntl $logfile_ LINEBUF 1
	}
	
	# create a pipe to get feedback during http transfers
	lassign [pipe] rfd_ wfd_
	fileevent $rfd_ readable [code $this feedback]
	
	# do window layout
	if {[layout_dialog]} {
	    destroy $w_
	    return
	}

	# add a short help window
	make_short_help

	# create an object for running interruptable batch queries
	Batch $w_.batch \
	    -command [code $this query_done] \
	    -debug $itk_option(-debug)

	# these are the supported plot symbols
	foreach i "circle square plus cross triangle diamond" {
	    set symbols_($i) 1
	}
	
	# create object for managing world coords
	wcs $w_.wcs
	
	# position from previous window, if pos.
	if {"$current_instance_" != ""} {
	    wm geometry $w_ +[winfo x $current_instance_]+[winfo y $current_instance_]
	}
	set current_instance_ $w_

	# set default values from image
	set_default_values

	# for local catalogs, start search automatically
	if {"$serv_type_" == "local"} {
	    catch {search}
	}

	# check for TCS catalog
	if {! $itk_option(-tcs) && [file exists $itk_option(-catalog)] \
		&& [is_tcs_catalog $itk_option(-catalog)]} {
	    global ::$w_.tcs
	    set $w_.tcs 1
	    set_tcs_columns
	}

	set initialized_ 1
    }


    # add the menu bar

    method add_menubar {} {
	TopLevelWidget::add_menubar
	set m [add_menubutton File]
	set file_menu_ $m
	
	add_menuitem $m command "Open" \
	    {Open a local catalog in tab table format} \
	    -command [code $this local_catalog]
	
	$m add separator

	add_menuitem $m command "Save as..." \
	    {Save listed objects to a local catalog file} \
	    -command [code $this save_as]

	add_menuitem $m command "Add to..." \
	    {Add listed objects to a local catalog file} \
	    -command [code $this add_to]

	add_menuitem $m command "Add selected..." \
	    {Add selected rows to a local catalog file} \
	    -command [code $this add_selected]

	$m add separator

	add_menuitem $m command "Print..." \
	    {Print the listing to a printer or file} \
	    -command [code $this print]

	add_menuitem $m command "Clear" \
	    {Clear the catalog listing} \
	    -command [code $this clear]

	$m add separator

	add_menuitem $m command "Close" \
	    {Close this window} \
	    -command [code $this close]
	
	if {$itk_option(-standalone)} {
	    add_menuitem $m command "Exit" \
		{Exit the application} \
		-command [code $this quit]
	}
	
	# Edit menu
	set m [add_menubutton Edit]
	set edit_menu_ $m

	add_menuitem $m command "Remove selected" \
	    {Remove selected rows from the local catalog} \
	    -command [code $this remove_selected]

	add_menuitem $m command "Enter new object..." \
	    {Enter the data for a new object for the local catalog} \
	    -command [code $this enter_new_object]

	add_menuitem $m command "Edit selected object..." \
	    {Edit the data for the selected object in the local catalog} \
	    -command [code $this edit_selected_object]


	# Options menu
	set m [add_menubutton Options]
	add_menuitem $m cascade "Name Server" \
	    {Select the name server used to resolve astronomical object names} \
	    -menu [set ns_menu [menu $itk_component(options).m.ns]]

	get_name_servers $ns_menu

	add_menuitem $m checkbutton "TCS Columns" \
	    {Display the standard columns used by the ESO TCS (Telescope Control Software)} \
	    -command [code $this set_tcs_columns] \
	    -variable $w_.tcs -onvalue 1 -offvalue 0

	add_menuitem $m command "Sort..." \
	    {Set options for sorting the query results} \
	    -command [code $this sort]

	add_menuitem $m command "Select Columns..." \
	    {Set options for displaying columns of the query results} \
	    -command [code $this select_columns]


	# Catalogs menu
	set m [add_menubutton Catalogs]
	fill_catalog_menu $m catalog

	# Archives menu
	set m [add_menubutton Archives]
	fill_catalog_menu $m archive

	# ImageServers menu
	set m [add_menubutton ImageServers]
	fill_catalog_menu $m imagesvr
    }
    
    
    # close this window

    method close {} {
	if {$itk_option(-standalone)} {
	    wm iconify  $w_
	} else {
	    wm withdraw $w_
	}
    }

    
    # quit the application (for standalone version)

    method quit {} {
	if {$itk_option(-standalone)} {
	    clear
	    destroy $w_
	    exit
	} else {
	    wm withdraw $w_
	}
    }

    # pop up a dialog to sort the list

    method sort {} {
	$table_ sort_dialog
    }

    # pop up a dialog to select table columns to display

    method select_columns {} {
	$table_ layout_dialog
    }

    # this method is called when the sort options have been selected. 
    # sort_cols is a list of column names to sort by
    # sort_idx is the same list, but with column indexes to sort by
    # sort_order is one of "increasing", "decreasing"

    method table_sort {sort_cols sort_idx sort_order} {
	busy {
	    set sort_cols_ $sort_cols
	    set sort_order_ $sort_order
	    # XXX this might be slow for slow servers...
	    search
	}
    }

    # add the name server catalogs to the given menu

    method get_name_servers {m} {
	if {[catch {set list [$w_.cat info namesvr]} msg]} {
	    error_dialog $msg $w_
	    return
	}

	foreach namesvr_ $list {
	    $m add radiobutton \
		-label $namesvr_ \
		-command [code $this set_namesvr $namesvr_] \
		-variable $m
	}

	# set default name server
	global ::$m
	set $m $namesvr_
    }


    # toggle the TCS option (use the tcscat or the astrocat Tcl command)

    method set_tcs_columns {} {
	global ::$w_.tcs
	set itk_option(-tcs) [set $w_.tcs]
	catch {$w_.cat delete}

	if {$itk_option(-tcs)} {
	    tcscat $w_.cat
	} else {
	    astrocat $w_.cat
	}
	$w_.cat feedback $wfd_
	catch {$w_.cat open $itk_option(-catalog)}
	set reset_columns_ 1
	search
    }


    # return true if the given file contains a TCS catalog

    proc is_tcs_catalog {filename} {
	if {[catch {set f [open $filename]}]} {
	    return 0
	}
	set title [gets $f]
	::close $f
	if {[string match "Tcs*" $title]} {
	    return 1
	}
	return 0
    }

    
    # clear the table listing
    
    method clear {} {
	catch {$table_ clear}

	# remove any catalog symbols (since table is also empty now)
	catch {$canvas_ delete [$w_.cat shortname $itk_option(-catalog)]}
    }

    
    # pop up a dialog to print the table listing
    
    method print {} {
	$table_ print_dialog
    }

       
    # open a local catalog file (tab table)
    
    method local_catalog {} {
	set file [filename_dialog [pwd] * $w_]
	if {"$file" != ""} {
	    if {[file isfile $file]} {
		if {[catch {$w_.cat check $file} msg]} {
		    error_dialog $msg $w_
		} else {
		    select_catalog $file local [is_tcs_catalog $file]
		}
	    } else {
		error_dialog "There is no file named '$file'" $w_
	    }
	}
    }

    
    # Fill up a menu of known data servers to choose from.
    # The serv_type argument should be one of: "catalog", "archive".

    method fill_catalog_menu {m serv_type} {
	if {[catch {set catalog_list [lsort [$w_.cat info $serv_type]]} msg]} {
	    error_dialog $msg $w_
	    return
	}
	
	if {[llength $catalog_list]} {
	    foreach i $catalog_list {
		add_menuitem $m command $i \
		    "Open $serv_type: \"$i\"" \
		    -command [code $this select_catalog $i $serv_type]
	    }
	}
    }


    # save the current data to a local catalog
    
    method save_as {} {
	set file [filename_dialog [pwd] * $w_]
	if {"$file" != ""} {
	    if {[file isfile $file]} {
		if {![confirm_dialog "File: `[file tail $file]' exists \
                                      - Do you want to overwrite it ?" $w_]} {
		    return
		}
		if {[file isdir $file]} {
		    error_dialog "File: `[file tail $file]' is a directory" $w_
		    return
		}
	    }
	    save_to_file $file [$table_ cget -info] [$table_ cget -headings]
	}
    }


    # add the rows in the current listing to a local catalog file
    
    method add_to {} {
	set info [$table_ cget -info]
	set headings [$table_ cget -headings]
	if {[llength $info] == 0} {
	    error_dialog "There are now rows to save" $w_
	    return;
	}
	add_rows $info $headings
    }

    
    # add the currently selected rows to a local catalog file

    method add_selected {} {
	set info [$table_ get_selected]
	set headings [$table_ cget -headings]
	if {[llength $info] == 0} {
	    error_dialog "No rows are selected" $w_
	    return;
	}
	add_rows $info $headings
    }

    
    # add the given info rows (result of a query) to a local catalog file with 
    # the given headings. The user selects the name of the catalog file.

    method add_rows {info headings} {
	if {[llength $headings] == 0} {
	    error_dialog "There is no data to save" $w_
	    return;
	}
	set file [filename_dialog [pwd] * $w_]
	if {"$file" != ""} {
	    if {! [file isfile $file]} {
		if {[confirm_dialog "File: `[file tail $file]' does not exists \
                                    - Do you want to create it ?" $w_]} {
		    save_to_file $file $info $headings
		}
	    } else {
		if {[file isdir $file]} {
		    error_dialog "File: `[file tail $file]' is a directory" $w_
		    return
		}
		save_to_file $file $info $headings 1
	    }
	}
    }

    
    # remove the currently selected rows from a local catalog file

    method remove_selected {} {
	set info [$table_ get_selected]
	set headings [$table_ cget -headings]
	set file $itk_option(-catalog)

	if {[llength $info] == 0} {
	    error_dialog "No rows are selected" $w_
	    return;
	}

	if {! [confirm_dialog "Remove object with id [lindex [lindex $info 0] 0]" $w_]} {
	    return
	}

	set equinox [$equinox_ get]
	if {[catch {$w_.cat remove $file $info $equinox $headings} msg]} {
	    error_dialog $msg $w_
	    return
	}
	
	# update the display
	search
    }

    
    # save the given info (the result of query) to the given catalog file,
    # using the given column headings.
    # If iflag is 1, insert rows in the existing file.

    method save_to_file {file info headings {iflag 0}} {
	set equinox [$equinox_ get]
	if {[catch {$w_.cat save $file $iflag $info $equinox $headings} msg]} {
	    error_dialog "error saving rows to file: $msg" $w_
	    return 1
	}
	return 0
    }
    

    # pop up a dialog to enter the data for a new object for a local catalog

    method enter_new_object {} {
	catch {destroy $w_.ef}
	EnterObject $w_.ef \
	    -title {Please enter the data for a new object below:} \
	    -image $itk_option(-image) \
	    -labels [$table_ cget -headings] \
	    -command [code $this enter_object]
    }

    
    # check that the given row contains valid data for a catalog
    # and return 0 if OK

    method check_row {data} {
	if {[catch {$w_.cat checkrow $data} msg]} {
	    error_dialog $msg
	    return 1
	}
	return 0
    }

    
    # this method is called with the data for a new object to add to a local
    # catalog
    
    method enter_object {info} {
	if {[check_row $info]} {
	    return
	}
	set id [lindex $info 0]
	
	if {! [confirm_dialog "Enter new object with id $id ?" $w_]} {
	    return
	}

	set headings [$table_ cget -headings]
	set file $itk_option(-catalog)
	save_to_file $file [list $info] $headings 1
    }

    
    # pop up a window so that the user can edit the selected object(s)
    
    method edit_selected_object {} {
	catch {destroy $w_.ef}
	set values [lindex [$table_ get_selected] 0]

	if {[llength $values] == 0} {
	    error_dialog "No rows are selected" $w_
	    return;
	}

	EnterObject $w_.ef \
	    -title {Please edit the data for the object below:} \
	    -image $itk_option(-image) \
	    -labels [$table_ cget -headings] \
	    -values $values \
	    -command [code $this replace_object $values]
    }

    
    # this method is called with the new data to replace the data
    # for the selected object/row in the local catalog
    
    method replace_object {old_data new_data} {
	if {[check_row $new_data]} {
	    return
	}
	
	if {"$old_data" == "$new_data"} {
	    info_dialog "No changes were made" $w_
	    return
	}

	set headings [$table_ cget -headings]
	set file $itk_option(-catalog)
	set id [lindex $new_data 0]

	if {! [confirm_dialog "Update object with id $id ?" $w_]} {
	    return
	}

	# remove old data
	set equinox [$equinox_ get]
	if {[catch {$w_.cat remove $file [list $old_data] $equinox $headings} msg]} {
	    error_dialog $msg $w_
	    return
	}

	# replace with new data
	if {[save_to_file $file [list $new_data] $headings 1] != 0} {
	    return
	}

	# change command to replace new object if changed again
	$w_.ef config -command [code $this replace_object $new_data]

	# update the display
	search
    }

    
    # do the dialog window layout
    
    method layout_dialog {} {
	# create astrocat object (or tcscat if TCS catalog) and setup feedback
	global tcs ::$w_.tcs
	if {[set $w_.tcs $itk_option(-tcs)]} {
	    tcscat $w_.cat
	} else {
	    astrocat $w_.cat
	}
	$w_.cat feedback $wfd_

	add_menubar

	pack [frame $w_.main] \
	    -side top -fill both -expand 1
	pack [frame $w_.main.top -relief groove -borderwidth 2] \
	    -side top -fill x

	pack \
	    [label $w_.main.top.head -text "Search Options"] \
	    -side top

	pack [set f [frame $w_.main.top.f -relief groove -borderwidth 2]] \
	    -side top -fill x -pady 1m

	blt::table $f
	blt::table $f \
	    [set name_ [LabelEntry $f.name \
		 -text "Object Name:" \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    0,0 -fill x -pady 1m \
	    [set equinox_ [LabelEntry $f.equinox \
		 -text "Equinox:" \
		 -value "J2000" \
		 -autoselect 1 \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    0,1 -fill x -pady 1m \
	    [set ra_ [LabelEntry $f.ra \
		 -text "a:" \
		 -autoselect 1 \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-wcsfont)]] \
	    1,0 -fill x -pady 1m \
	    [set dec_ [LabelEntry $f.dec \
		 -text "d:" \
		 -autoselect 1 \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-wcsfont)]] \
	    1,1 -fill x -pady 1m \
	    [set rad1_ [LabelEntry $f.rad1 \
		 -text "Min Radius:" \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    2,0 -fill x -pady 1m \
	    [set rad2_ [LabelEntry $f.rad2 \
		 -text "Max Radius (max 13.5):" \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    2,1 -fill x -pady 1m \
	    [set mag1_ [LabelEntry $f.mag1 \
		 -text "Brightest (min 10):" \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    3,0 -fill x -pady 1m \
	    [set mag2_ [LabelEntry $f.mag2 \
		 -text "Faintest (max 14):" \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    3,1 -fill x -pady 1m \
	    [set maxnum_ [LabelEntry $f.maxnum \
		 -text "Max Objects:" \
		 -command [code $this search] \
		 -labelwidth $itk_option(-labelwidth) \
		 -anchor $itk_option(-anchor) \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    4,0 -fill x -pady 1m \
	    [set setfrom_ [button $f.setfrom \
		 -text "Set From Image" \
		 -command [code $this set_from_image]]] \
	    4,1 -anchor e -pady 1m \
	    [set copyright_ [LabelValue $f.copyright \
		 -anchor w -relief flat \
		 -labelwidth 0 -valuewidth 0 \
		 -valuefont $itk_option(-labelfont)]] \
	    5,0 -anchor e -fill x -pady 1m -columnspan 2

	blt::table configure $f C1 -padx 2m

	# table to display results as text
	itk_component add table {
	    set table_ [TableList $w_.main.table \
			    -title "Search Results" \
			    -hscroll 1 \
			    -sortcommand [code $this table_sort] \
			    -selectmode extended \
			    -exportselection 0]
	} {	    
	}
	
	pack $itk_component(table) -side top -fill both -expand 1

	bind $table_.listbox <ButtonRelease-1> [code $this select_table_row]
	bind $table_.listbox <Double-ButtonPress-1> [code $this label_selected_object]
	$table_ set_options {MORE PREVIEW more preview} Show 0
	
	# dialog buttons
	pack [frame $w_.buttons -borderwidth 2 -relief groove] \
	    -side top -fill x
	pack \
	    [button $w_.search \
		 -text "Search" \
		 -command [code $this search]] \
	    [button $w_.plot \
		 -text "Plot" \
		 -command [code $this plot_again]] \
	    [button $w_.more \
		 -text "More Info" \
		 -command [code $this more]] \
	    [button $w_.preview \
		 -text "Preview" \
		 -command [code $this preview]] \
	    [button $w_.stop \
		 -text "Stop" \
		 -state disabled \
		 -command [code $this interrupt]] \
	    [button $w_.close \
		 -text "Close" \
		 -command [code $this close]] \
	    -side left -expand 1 -padx 2m -pady 2m -in $w_.buttons

        # Add frame for tcs commands

        # pack [set tcsFrame [frame $w_.tcsFrame]] -side top -fill both
        # add_tcs_controls $tcsFrame
	set tcs(starSelMode) 0


	# add a progress bar at the botton
	pack [ProgressBar $w_.progress] \
	    -side bottom -fill x

	# set the catalog (default if not set)
	if {"$itk_option(-catalog)" == ""} {
	    if {[catch {set catalog_list [$w_.cat info $itk_option(-catalogtype)]} msg]} {
		error_dialog $msg $w_
		return
	    }
	    set itk_option(-catalog) [lindex $catalog_list [expr [llength $catalog_list]-1]]
	}
	set_catalog $itk_option(-catalog)
	return 0
    }

    # add TCS controls
    
    method add_tcs_controls { top } {
       tcacamAddTcsControl $top $itk_option(-wcsfont)
    }

    # add a short help window and set the help texts
    
    method make_short_help {} {
	TopLevelWidget::make_short_help

	add_short_help $name_ {object name: resolved via name server, if given}
	add_short_help $equinox_ {World Coordinates equinox, default J2000}
	add_short_help $ra_ {World Coordinates: right ascension (RA)}
	add_short_help $dec_ {World Coordinates: declination (DEC)}
	add_short_help $rad1_ {Radius: minimum radius in arcmin for circular search}
	add_short_help $rad2_ {Radius: maximum radius in arcmin for circular search}
	add_short_help $mag1_ {Magnitude: brightest magnitude for search}
	add_short_help $mag2_ {Magnitude: faintest magnitude for search}
	add_short_help $maxnum_ {Max number of stars/objects to display}
	add_short_help $setfrom_ {Set default values from the current image}

	add_short_help $table_ {Table: results of query: {bitmap b1} (double click) = label object}
	add_short_help $w_.search {{bitmap b1} = start catalog search}
	add_short_help $w_.plot {{bitmap b1} = Plot the listed objects again in the image}
	add_short_help $w_.preview {{bitmap b1} = view the preview image or plot data for the selected object}
	add_short_help $w_.more {{bitmap b1} = display more information for the selected object}
	add_short_help $w_.stop {{bitmap b1} = interrupt the current catalog operation}
	#add_short_help $w_.close {{bitmap b1} = close this window}

	add_short_help $w_.progress {Progress bar: displays status of work in progress}
    }


    # look for a running skycat application and use tk send to communicate with it
    
    method find_skycat {} {
	set n 0
	set skycat_interp_ {}
	set skycat_inst_ {}
	set image_ {}
	set canvas_ {}
	set itk_option(-image) {}

	foreach i [winfo interps] {
	    if {[string match "Skycat*" $i]} {
		if {[catch {set instances [send $i get_skycat_images]} msg]} {
		    error_dialog "Tk send error: interp: $i: $msg\n\
                                  (Maybe Tk send was not enabled when compiling Tk?)" $w_
		    continue
		}
		set skycat_interp_ $i
		foreach inst $instances {
		    incr n
		    lappend interp($i) [set skycat_inst_ $inst]
		}
	    }
	}

	if {$n == 0} {
	    # no skycat running now
	    return
	}
	if {$n > 1} {
	    # XXX popup dialog to choose skycat
	}

	# use a local method with tk send instead of a local image...
	eval [format {
	    proc %s {args} {return [send {%s} %s $args]}
	} [set image ::send_to_skycat_image] $skycat_interp_ $skycat_inst_]

	configure -image $image

	# access internal image object
	set i $image_
	eval [format {
	    proc %s {args} {return [send {%s} %s $args]}
	} [set image_ ::send_to_skycat_image_] $skycat_interp_ $i]

	# access skycat canvas
	set c $canvas_
	eval [format {
	    proc %s {args} {return [send {%s} %s $args]}
	} [set canvas_ ::send_to_skycat_canvas_] $skycat_interp_ $c]

	# use this for bindings on canvas
	set cmd_prefix_ "send {[winfo name .]}"
    }

    
    # check that the remote skycat application is still running and if not,
    # try to find a new one

    method check_skycat {} {
	if {"$skycat_interp_" != ""} {
	    if {[catch {send $skycat_interp_ set _x_ {}}]} {
		find_skycat
	    }
	}
    }
    
        
    # called when a catalog ($name) is selected from the menu.
    # serv_type is one of: catalog, archive, local, imagesvr,...
    # tcs_flag can be set to true, if you know it is a TCS catalog.

    method select_catalog {name {serv_type "catalog"} {tcs_flag 0}} {
	check_skycat

	if {"$skycat_interp_" == ""} {
	    set im $itk_option(-image)
 	} else {
 	    set im {}
 	}

	if {"$serv_type" == "imagesvr"} {
	     cat::AstroImage::new_imagesvr $name $im $itk_option(-debug)
	} else {
	    # reuse this window ? (if empty...)
	    if {[$table_ info_rows] == 0} {
		reuse_catalog $name $im
	    } else {
		new_catalog $name $im $itk_option(-debug) $tcs_flag
	    }
	}

	# guidecam_block begin
	set column_layout_ok 0
	# guidecam_block end
    }

    # guidecam_block begin
    method getListCount {} {
	if {"$table_" != ""} {
	    return [$table_ total_rows]
	}
	return 0
    }

    proc hideCatalogs {} {
	if [info exists instances_] {
	    set catList [array get instances_]
	    foreach {name value} $catList {
		$value close
	    }
	}
    }
    
    # guidecam_block end

    # set the name server to use to resolve object names
    
    method set_namesvr {name} {
	set namesvr_ $name
    }


    # interrupt the current search 

    method interrupt {} {
	$w_.batch interrupt
	catch {file delete $itk_option(-tmpfile)}
	$table_ config -title "Search Results"
	set_state normal
    }

    
    # set/reset widget states while busy 

    method set_state {state} {
	set state_ $state
	if {"$state" == "normal"} {
	    catch {blt::busy release $w_.main}
	    catch {focus -lastfor $w_.main}
	    $w_.search config -state normal
	    $w_.plot config -state normal
	    $w_.stop config -state disabled
	} else {
	    catch {focus .}
	    catch {blt::busy hold $w_.main}
	    $w_.search config -state disabled
	    $w_.plot config -state disabled
	    $w_.more config -state disabled
	    $w_.preview config -state disabled
	    $w_.stop config -state normal
	}
	update idletasks
	$w_.progress reset
    }

    
    # this method is called by the fileevent handler during the data transfer 
    # from the HTTP C++ class to give the user feedback about how much there is 
    # left to copy, etc...
    # Read a line from the feedback pipe, which should contain text to be displayed
    # in the progress widget.
    #
    # When we are in busy (disabled, waiting) mode, we also interpret some special
    # messages, such as: "total length: n bytes" to update the progress bar.

    method feedback {} {
	set text [gets $rfd_]
	if {"$state_" != "normal"} {
	    if {[scan $text {total length: %d bytes} n] == 1} {
		$w_.progress config -to $n
	    } elseif {[scan $text {read %d bytes} n] == 1} {
		$w_.progress config -value $n
	    } elseif {[scan $text {url: %s} url] == 1} {
	    	catch {
		    puts $logfile_ "[fmtclock [getclock]]\n$itk_option(-catalog)\n$url\n\n"
		    flush $logfile_
		}
	    }
	}
	$w_.progress config -text $text
	update idletasks
    }


    # reset table dialogs if needed

    method reset_table {} {
	foreach w "$w_.tblsort $w_.tblcfg" {
	    if {[winfo exists $w]} {
		$w reset
	    }
	}
	$table_ set_options {MORE PREVIEW more preview} Show 0
    }
    

    # This method is called when the background query is done.
    # The arguments are the error status (0 is OK) and the result
    # of evaluating the Tcl commands. The result is a list of 3 items:
    # the query result, the query position and a flag "more" indicating
    # if there were more rows available.

    method query_done {status result} {
	if {$status} {
	    set_state normal
	    $table_ config -title "Search Results"
    	    # error messages starting with "***" are only displayed in progress win
	    if {"[string range $result 0 2]" != "***"} {
		error_dialog $result $w_
	    }
	    after 0 [list $w_.progress config -text $result]
	} else {
	    busy {

		lassign $result info_ headings_ symbol_ pos more

		# update table
		$table_ config -headings $headings_
		if {$reset_columns_} {
		    set reset_columns_ 0
		    reset_table
		}
		$table_ config -info $info_

		if {$more} {
		    set more "+"
		} else {
		    set more ""
		}
		$table_ config -title "Search Results ([$table_ total_rows]$more)"

		# note column indexes in array (use upper case to simplify search)
		catch {unset col_}
		set n -1
		foreach i $headings_ {
		    set col_([string toupper $i]) [incr n]
		}
		
		# if mag is not supported, disable it
		# if {[info exists col_(MAG)]} {
		#     $mag1_ config -state normal
		#     $mag2_ config -state normal
		# } else {
		#     $mag1_ config -state disabled
		#     $mag2_ config -state disabled
		# }

		if {[llength $pos] >= 2} {
		    lassign $pos ra dec equinox
		    $ra_ config -value $ra
		    $dec_ config -value $dec
		    $name_ config -value ""
		}

		# plot stars
		plot
	    }

	    set_state normal
	}
	
	# guidecam_block begin
	if {$column_layout_ok == 0} {
	    $table_ layout_dialog
	    update
	    set found 0
	    set childrenList [winfo children $w_]
	    foreach child $childrenList {
		if {[string first "tblcfg" $child] != -1} {
		    set found 1
		    break;
		}
	    }
	    if {$found} {
		set dblList $child.dlist
		set colList [$dblList.left get_contents] 
		if [lcontain $colList "ra"] {lappend subList ra}
		if [lcontain $colList "dec"] {lappend subList dec}
		if [lcontain $colList "mag"] {lappend subList mag}
		$dblList.left set_contents $subList 
		set column_layout_ok 1
		$child ok
	    }
	}
	# guidecam_block end
    }

    
    # start the catalog search based on the current search options
    # and display the results in the table.

    method search {args} {
	set cmd "$w_.cat query"

	#check limits
	if { [ $mag2_ get ] > 14 } {
	    $mag2_ config -value 14
	}
	if { [ $mag1_ get ] < 10 } {
	    $mag1_ config -value 10
	}
	if { [ $rad2_ get ] > 13.5 } {
	    $rad2_ config -value 13.5
	}

	set ra [$ra_ get]
	set dec [$dec_ get]
	set name [$name_ get]
	set equinox [$equinox_ get]
	set rad1 [$rad1_ get]
	set rad2 [$rad2_ get]
	set mag1 [$mag1_ get]
	set mag2 [$mag2_ get]

	if {"$equinox" != ""} {
	    lappend cmd "-equinox" $equinox
	}
	if {"$name" != ""} {
	    lappend cmd "-nameserver" $namesvr_ "-name" $name
	} elseif {"$ra" != "" && "$dec" != ""} {
	    lappend cmd "-pos" [list $ra $dec]
	} else {
	    #warning_dialog "Please specify either an object name or a position in WCS" $w_
	}

	if {"$rad1" != "" || "$rad2" != ""} {
	    lappend cmd "-radius" "$rad1 $rad2"
	}
	if {"$mag1" != "" || "$mag2" != ""} {
	    lappend cmd "-mag" "$mag1 $mag2"
	}

	set maxnum [$maxnum_ get]
	if {"$maxnum" != ""} {
	    lappend cmd "-nrows" $maxnum
	}
	
	if {"$sort_cols_" != ""} {
	    lappend cmd "-sort" $sort_cols_ "-sortorder" $sort_order_
	}
	
	# start the query in the background
	$table_ config -title "Searching..."
	clear

	set_state disabled
	$w_.progress config -text "Attempting to contact catalog server..."
	$w_.progress look_busy

	$w_.batch config -command [code $this query_done]
	$w_.batch bg_eval [code [format {
	    %s.cat open {%s}; list [%s] [%s.cat headings] \
		[%s.cat symbol] [%s.cat querypos] [%s.cat more]
	} $w_ $itk_option(-catalog) $cmd $w_ $w_ $w_ $w_ $w_]]
    }
    
    
    # Set the default values for the form entries:
    # set the default position (RA, DEC) to the image center, 
    # set the min radius to 0 and the max radius to the distance 
    # from the center to the origin.

    method set_default_values {} {
	set_from_image
	$mag1_ config -value 11.0
	$mag2_ config -value 13.5
	$maxnum_ config -value 60
    }


    # set the default values for position and radius from the image

    method set_from_image {} {
	check_skycat
	if {"$image_" == ""} {
	    return
	}

	$name_ config -value ""
	$rad1_ config -value 0.0
	set center  [$image_ wcscenter]
	# only set if there is an image loaded
	if {[llength $center] >= 2 && ! [$image_ isclear]} {
	    lassign $center ra dec equinox
	    # set equinox [$image_ wcsequinox]
	    $ra_ config -value $ra
	    $dec_ config -value $dec
	    $equinox_ config -value $equinox
	    set r [format "%.2f" [$image_ wcsradius]]

	    # guidecam_block begin
	    # Lines commented out
	    #if {$r} {
		#$rad2_ config -value $r
	    #} else {
		$rad2_ config -value 7.5
	    #}
	    # guidecam_block end 

	} else {
	    $ra_ config -value ""
	    $dec_ config -value ""
	    $rad2_ config -value 7.5
	}
    }

    
    # insert the Id for the object selected in the Table in the canvas
    # near the object

    method label_selected_object {} {
        global ::tcs
	check_skycat
	if {"$canvas_" == ""} {
	    return
	}

	set id [lindex [lindex [$table_ get_selected] 0] 0]
	if {"$id" == ""} { return }

        # Extract ra,dec of the selected star in the catalog

        if {$tcs(starSelMode) == 0} {
#           puts "DBG> table row = [$table_ get_selected]"
	   set tmp [string trim [$table_ get_selected] "\{\}"]
           guidecamGetStarParams  [split [$table_ get_selected] " "]
        }

	$canvas_ delete label$id
	if {[llength [set box [$canvas_ bbox cat$id]]]} {
	    lassign $box x0 y0 x1 y1
	    set cid [$canvas_ create text [expr ($x1+$x0)/2.0] [expr ($y1+$y0)/2.0] \
			 -text $id \
			 -anchor sw \
			 -fill white \
			 -font -*-courier-medium-r-*-*-*-120-*-*-*-*-*-* \
			 -tags [list objects cat$id label$id [$w_.cat shortname $itk_option(-catalog)]]]

	    # check if using tk send
	    if {"$skycat_interp_" != ""} {
		send $skycat_interp_ [$itk_option(-image) component draw] add_object_bindings $cid
		send $skycat_interp_ ct_add_bindings [$itk_option(-image) get_canvas] $cid
	    } else {
		[$itk_option(-image) component draw] add_object_bindings $cid
		ct_add_bindings $canvas_ $cid
	    }

	    $w_.progress config -text "labeled object '$id' in image"
	} else {
	    $w_.progress config -text "object '$id' is not visible"
	}
    }


    # this method is called whenever a table row is selected
    # note the selected values and enable/disable some buttons

    method select_table_row {} {
	if {! [info exists col_(PREVIEW)] && ! [info exists col_(MORE)]} {
	    return
	}
	set row [lindex [$table_ get_selected] 0]
	if {[llength $row] == 0} {
	    return
	}
	set preview [get_col PREVIEW $row]
	set more [get_col MORE $row]
	set object_name_ [lindex $row 0]

	# note: HST returns P=http:....
	if {! [regsub {.*(http:.*)} $preview {\1} preview_url_]} {
	    set preview_url_ ""
	    $w_.preview config -state disabled
	} else {
	    $w_.preview config -state normal
	}

	# note: HST returns M=http:....
	if {! [regsub {.*(http:.*)} $more {\1} more_url_]} {
	    set more_url_ ""
	    $w_.more config -state disabled
	} else {
	    $w_.more config -state normal
	}   
    }


    # assuming there is a field called "preview" that is the URL of an
    # image, get the image or table data for the selected line over HTTP 
    # and display it
    # 
    # The preview data may be an image or other data, such as a tab table to
    # display as a graph (like the "cuts" (RtdImageSpectrum) graph).

    method preview {} {
	check_skycat
	if {"$image_" == ""} {
	    return
	}

	if {[$table_ num_selected] == 0} {
	    warning_dialog "Please select an object from the list first" $w_
	    return
	}
	if {! [info exists col_(PREVIEW)]} {
	    warning_dialog "Previewing is not supported for this catalog" $w_
	    return
	}
	if {"$preview_url_" == ""} {
	    warning_dialog "No preview for this object" $w_
	    return
	}
	
	# XXX temp
	#if {"[exec uname -n]" == "piano"} {
	#    set preview_url_ "http://piano:8080/cgi-bin/getpreview"
	#}
	
	# do it in the background, since it may take a while
	set_state disabled
	$w_.progress config -text "Attempting to contact server..."
	$w_.progress look_busy

	$w_.batch config -command [code $this preview_done]
	$w_.batch bg_eval [code \
	    "$w_.cat open {$itk_option(-catalog)}; \
             [list $w_.cat getpreview -url $preview_url_ -tmpfile $itk_option(-tmpfile)]"]
    }
    

    # This method is called when we have received the preview data
    # The "status" argument is the status of the background http get operation (0 if ok).
    # The result is a list of {filename Content-type}, where filename contains the data
    # and Content-type indicates the type of the preview data. Decompression is already
    # taken care of, so we only expect to get a FITS image here or a tab table to plot.

    method preview_done {status result} {
	if {$status} {
	    error_dialog $result $w_
	} else {
	    if {[llength $result] != 2} {
		error_dialog "error getting preview data (result = $result)"
	    } else {
		lassign $result filename type
		if {"$type" == "image/x-fits"} {
		    # load the image and remove the temp file
		    $itk_option(-image) config -file $filename
		    catch {file delete $filename}
		} elseif {"$type" == "text/tab-separated-values" \
			      || "$type" == "text/x-starbase" \
			      || "$type" == "text/plain"} {
		    PreviewPlot $w_.pplot[incr count_] \
			-file $filename \
			-name $object_name_ \
			-shorthelpwin $this \
			-transient 1
		    catch {file delete $filename}
		} else {
		    error_dialog "unrecognized Content-type for preview data: $type"
		    catch {file delete $filename}
		}
	    }
	}
 	set_state normal
	
	# restore preview button state...
	select_table_row
    }

    
    # use the "more" URL to display more info about an object using netscape
    # or mosaic

    method more {} {
	if {[$table_ num_selected] == 0} {
	    warning_dialog "Please select an object from the list first" $w_
	    return
	}
	if {"$more_url_" == ""} {
	    warning_dialog "No more information is available for this object" $w_
	    return
	}
	# info_dialog "more url = $more_url_" $this
	
	# start netscape
	$w_.progress config -text "sending URL to netscape..."
	update idletasks
	$itk_option(-image) send_to_netscape $more_url_
    }


    # generate a dummy blank image for the purpose of plotting catalog
    # objects on it.  Return 0 if OK, otherwise 1.

    method gen_blank_image {} {
	set ra [$ra_ get]
	set dec [$dec_ get]
	if {"$ra" == "" || "$dec" == ""} {
	    # can't create a blank image with no center
	    # use coords from first row, if any
	    lassign [lindex $info_ 0] id ra dec
	    if {"$ra" == "" || "$dec" == ""} {
		return 1
	    }
	    $ra_ config -value $ra
	    $dec_ config -value $dec
	}
	if {[catch {lassign [$w_.wcs hmstod $ra $dec] ra_deg dec_deg} msg]} {
	    warning_dialog "$msg (ra = $ra, dec = $dec)"
	    return 1
	}

	# generate dummy image (XXX: don't need radius)
	$image_ clear \
	    -reuse 1 \
	    -ra $ra_deg \
	    -dec $dec_deg \
	    -radius [$rad2_ get] \
	    -width [winfo width $canvas_] \
	    -height [winfo height $canvas_]
	
	return 0
    }


    # plot the listed objects again (they may have been removed after an
    # image was loaded)
    
    method plot_again {} {
	busy {plot}
    }

        
    # plot the guide stars found in the previous search 
    # in the image window
    #
    # The symbol to use is taken from the config file.
    # "none" means don't plot. For now just accept oval
    # or rect. Add more symbols later.

    method plot {} {
	check_skycat
	if {"$image_" == ""} {
	    return
	}
	
	# if no image loaded, generate dummy image
	if {"[$image_ cget -file]" == "" && "[$image_ object]" == ""} {
	    if {[gen_blank_image] != 0} {
		return
	    }

	    # enable the main panel, since we now have an image
	    catch {[[$itk_option(-image)] component info] configure -state normal}
	}

	# if any objects are selected, deselect them first
	$canvas_ delete grip
	
	update idletasks

	set equinox [$equinox_ get]

	foreach col_info [split $symbol_ ":"] {

#          puts "DBG 1 > [llength $col_info] col_info = $col_info"

           set newinfo [string trim $col_info "\{\} "]

           # Check correctness of the entry in config. file

           set tmp [split $newinfo " "]

	   if {[llength $tmp] != 3} {
		error_dialog "invalid symbol entry in config file: '$col_info' ([llength $col_info])" $w_
	return
	    }

           set col     [ctoken newinfo " "]
           set symbol  [ctoken newinfo " "]
           set expr    [ctoken newinfo " \{\}"]

 #          puts "col : $col  symbol : $symbol expr : $expr"

	    set uc_col [string toupper $col]

	    if {! [info exists col_($uc_col)]} {
		error_dialog "invalid plot column: '$col'" $w_
		continue
	    }
            set symbol [string trim $symbol "\{\} "]
	    if {! [info exists symbols_($symbol)]} {
		error_dialog "invalid plot symbol: '$symbol'" $w_
		continue
	    }

	    set w [$image_ dispwidth]
	    set h [$image_ dispheight]
	    lassign [$image_ scale] xs ys
	    if {$xs < 0} {
		set xs [expr 1.0/-$xs]
	    }

	    set n -1

	    foreach i $info_ {
		incr n
		lassign $i id ra dec
		if {[catch {lassign [$image_ convert coords $ra $dec "wcs $equinox" {} {} canvas] x y} msg]} {
		    #puts "error converting world coordinates: $i $ra $dec"
		    continue
		}
		if {! ($x >= 0 && $y >= 0 && $x < $w && $y < $h)} {
		    #puts "$x,$y +$w,$h: $ra $dec out of range"
		    continue
		}
		set $col [get_col $uc_col $i]
		if {"[set $col]" == ""} {
		    #puts "$col empty"
		    continue
		}

                
		if {[catch {set size [max [expr "(($expr)*2*$xs)+1"] 5]} msg]} {
		    error_dialog "error plotting column $col: $msg" $w_
		    return
		}
		$canvas_ delete cat$id
		
		set x0 [expr $x-$size]
		set y0 [expr $y-$size]
		set x1 [expr $x+$size]
		set y1 [expr $y+$size]
		set tags [list objects cat$id [$w_.cat shortname $itk_option(-catalog)]]

		# draw the symbol
		eval [list draw_$symbol $canvas_ $x $y $x0 $y0 $x1 $y1 black white $tags]

		#guidecam_block begin 
		$canvas_ bind "cat$id" <ButtonRelease-1> "$cmd_prefix_ $this object_selected $n ; $cmd_prefix_ $this picked_wcs_object $ra $dec ; guidecamGetSelected $table_ "
		#$canvas_ bind "cat$id" <Any-Enter> "$cmd_prefix_ $table_ select_row $n"
		#$canvas_ bind "cat$id" <ButtonRelease-1> "puts \"astrocat depressed\""
		$canvas_ bind "cat$id" <Any-Enter> "$cmd_prefix_ $this object_selected $n"
		#guidecam_block end
	    }
	}
    }
    
    #guidecam_block begin
    method object_selected {tableRow} {
	global objRa objDec 
	
	$table_ select_row $tableRow
	set entry [$table_ get_selected]
	#Cut the first and 2 last curly brackets
	set entry [crange $entry 1 [expr [clength $entry] - 2]]
	set objRa [lindex $entry 1]
	set objDec [lindex $entry 2]
    }

    proc get_current_instance {} {return $current_instance_}
    #guidecam_block end

    # This method is called when the user clicks on a graphic symbol for a star.
    # The user might be selecting this star, so call the RtdImage method to do that
    
    method picked_wcs_object {ra dec} {

	if {"$ra" != "" && "$dec" != ""} {
	    if {[catch {$image_ convert coords $ra $dec "wcs [$image_ wcsequinox]" \
			    x y image} msg]} {
		puts $msg
	    }

	   catch {$itk_option(-image) picked_wcs_object $x $y $ra $dec}
	}
	#puts "AstroCat::picked_wcs_object ra $ra dec $dec"
    }

    
    # return the value for the given column name in the given row

    method get_col {name row} {
	if {[info exists col_($name)]} {
	    return [lindex $row $col_($name)]
	} 
    }

   
    # This method is called when the user selects a catalog from the menu
    # Here we open the new catalog and setup the table for listing...
    #
    # Note: we use one window per catalog, so you can have multiple catalogs

    method set_catalog {name} {
        global tcl_version

	if {[catch {$w_.cat open $name} msg]} {
	    error_dialog $msg $w_
	    return
	}
	config -catalog $name
	$copyright_ configure -value [$w_.cat copyright]
	wm title $w_ $name
	wm iconname $w_ [$w_.cat shortname $name]
	$table_ config -title "Search Results" -headings {} -info {}

	$w_.preview config -state disabled
	$w_.more config -state disabled

	# keep aray of instances(name,image)
	set image $itk_option(-image)

        if {$tcl_version >= 8.0} {
	    if {![winfo exists [namespace tail $image]]} {
	        set image {}
	    }
        } else {
	    if {![winfo exists [info namespace tail $image]]} {
	        set image {}
	     }
        }

	set instance_idx_ "$name,$image"
	set instances_($instance_idx_) $w_ 

	# if this is a local catalog, enable some menu items
	set serv_type_ [$w_.cat servtype $name]
	if {"$serv_type_" == "local"} {
	    set state normal
	} else {
	    set state disabled
	}
	$edit_menu_ entryconfig "Remove selected" -state $state
	$edit_menu_ entryconfig "Enter new object..." -state $state
	$edit_menu_ entryconfig "Edit selected object..." -state $state
    }


    # member proc (like a static member function) to create an instance of this
    # class for the named catalog (or reuse the existing one for the catalog)
    #
    # The args are: 
    #
    # name  -    long name of catalog from config file, or {} for default catalog
    #            (first catalog in config file)
    # image -    name of skycat itcl image widget, or empty if running standalone
    # debug -    flag: if true, run queries in foreground
    # tcs_flag - may be set to true if the catalog is TCS format
    # type -     if name is empty and type is "archive", the default catalog will
    #            be an archive. Otherwise type should be "catalog".

    #guidecam_block begin
    proc new_catalog {name image debug {tcs_flag 0} {type "catalog"}} {
	set i "$name,$image"
        if {[info exists instances_($i)] && [winfo exists $instances_($i)]} {
            utilRaiseWindow $instances_($i)
            return $instances_($i)
        }
        set instances_($i) \
            [AstroCat .ac[incr n_instances_] \
                 -image $image \
                 -debug $debug \
                 -catalog $name \
                 -catalogtype $type \
                 -tcs $tcs_flag \
                 -transient 0 \
                 -center 0]
	return $instances_($i)
    }
    #guidecam_block end
    
    # This is the same as new_catalog, except the default for name is the
    # first archive type catalog in the config file

    proc new_archive {name image debug {tcs_flag 0}} {
	new_catalog $name $image $debug $tcs_flag "archive"
    }

    
    # reuse this window for a new catalog, unless the catalog is already open
    # in a different window.
    # The args are: 
    #
    # name  - long name of catalog from config file
    # image - name of skycat itcl image widget

    method reuse_catalog {name image} {
	set idx "$name,$image"
	if {[info exists instances_($idx)] && [winfo exists $instances_($idx)]} {
	    utilRaiseWindow $instances_($idx)
	    return
	}
	foreach i [array names instances_] {
	    if {"$instances_($i)" == "$w_" && "$i" != "$idx"} {
		unset instances_($i)
	    }
	}
	set instance_idx_ $idx
	set instances_($idx) $w_
	set reset_columns_ 1
	set_catalog $name

	# for local catalogs, start search automatically
	if {"$serv_type_" == "local"} {
	    if {[is_tcs_catalog $name]} {
		set_tcs_columns
	    } else {
		catch {search}
	    }
	}
    }
   

    # return a list of the instances of this class

    proc instances {} {
	set list {}
	if {[info exists instances_]} {
	    foreach i [array names instances_] {
		if {[winfo exists $instances_($i)]} {
		    lappend list $instances_($i)
		}
	    }
	}
	return $list
    }
    

    # return a list of top level windows that this class might create

    proc list_windows {} {
	set list [AstroCat::instances]
	# $w.tblsort may be created by the TableList class
	foreach w $list {
	    lappend list $w.tblsort $w.preview
	}
	return $list
    }

   
    # -- options --

    # name of catalog
    itk_option define -catalog catalog Catalog ""

    # type of catalog (catalog or archive)
    itk_option define -catalogtype catalogType CatalogType "catalog"

    # image handle (itcl class)
    itk_option define -image image Image {} {
	if {"$itk_option(-image)" != ""} {
	    set canvas_ [$itk_option(-image) get_canvas]
	    set image_ [$itk_option(-image) get_image]

            # Add binding to select preset star

            bind selectTcsStar <1> getRaDecFromRtd 
            bindtags $canvas_ [linsert [bindtags $canvas_] end selectTcsStar]
	}
    }

    # fonts used
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*
    itk_option define -valuefont valueFont ValueFont -adobe-courier-medium-r-*-*-*-120-*-*-*-*-*-*
    itk_option define -wcsfont wcsFont WcsFont -*-symbol-*-*-*-*-14-*-*-*-*-*-*-*

    # set the width for  displaying labels
    itk_option define -labelwidth labelWidth LabelWidth 15

    # set the anchor for labels
    itk_option define -anchor anchor Anchor e

    # flag: if true, run queries in foreground for better debugging
    itk_option define -debug debug Debug 0

    # temp image file to use for preview
    itk_option define -tmpfile tmpfile Tmpfile {} {
	if {"$itk_option(-tmpfile)" == ""} {
	    set itk_option(-tmpfile) "/tmp/preview[pid].[incr count_]"
	}
    }

    # if -tcs is 1, use fixed format TCS catalog listings
    itk_option define -tcs tcs Tcs {0} 

    # -- protected members --

    #  canvas window containing main image
    protected variable canvas_ {}

    # internal rtdimage widget for main image
    protected variable image_ {}

    # tcl interp used to communicate with skycat, if remote
    protected variable skycat_interp_ {}

    # remote skycat instance name, if using send
    protected variable skycat_inst_ {}
 
    # send prefix, used for bindings on remote canvas, if using send
    protected variable cmd_prefix_ {}

    # table and headings
    protected variable table_ {}
    protected variable headings_ {}

    # table contents
    protected variable info_ {}

    # array(uppercase col name) of col index from catalog headings
    protected variable col_

    # symbol description from config file (skycat.cfg)
    protected variable symbol_ {}

    # array containing supported symbol names
    protected variable symbols_

    # preview and more-info URLs for selected object
    protected variable preview_url_ {}
    protected variable more_url_ {}

    # option fields
    protected variable name_
    protected variable equinox_
    protected variable ra_
    protected variable dec_
    protected variable rad1_
    protected variable rad2_
    protected variable mag1_
    protected variable mag2_
    protected variable maxnum_
    protected variable setfrom_

    # copyright field
    protected variable copyright_

    # pipe to write/read feedback during image transfer
    protected variable rfd_
    protected variable wfd_

    # name server to use to resolve object names
    protected variable namesvr_ {}

    # current state: normal, disabled (i.e.: waiting)
    protected variable state_ {normal}

    # log file handle (used to log URLs)
    protected variable logfile_ stdout

    # flag: set at end of constructor
    protected variable initialized_ 0
    
    # index of this object in the instances_ array
    protected variable instance_idx_ {}

    # name of file menu widet
    protected variable file_menu_

    # name of edit menu widet
    protected variable edit_menu_

    # serv_type of current catalog (local, catalog, archive)
    protected variable serv_type_ {}

    # list of col names to sort results by (empty means don't sort)
    protected variable sort_cols_ {}

    # set direction of sort: may be one of (increasing, decreasing) 
    protected variable sort_order_ {increasing}

    # flag: set when the column headings are changed between TCS and normal
    protected variable reset_columns_ {0}

    # currently selected object (Id field in row)
    protected variable object_name_ {}

    # guidecam_block begin
    protected variable column_layout_ok 0
    # guidecam_block end
  
    # -- common variables (common to all instances of this class) --
    
    # array mapping catalog name to widget/class name
    common instances_

    # instance count
    common n_instances_ 0

    # current instance name
    common current_instance_ {}

    # count used for filename generation
    common count_ 0
}
