# E.S.O. - VLT project/ESO Archive
# @(#) $Id: AstroImage.tcl,v 1.25.1.6 2005/12/09 18:38:44 vltsccm Exp $
#
# AstroImage.tcl - user interface class for accessing image servers
#                     such as DSS (Digitized Sky Survey) 
#
# See man page AstroImage(1) for a complete description.
#
# who         when       what
# --------   ---------   ----------------------------------------------
# A.Brighton 11 Oct 95   created
# mcomin      31/03/98   Add patch to get iswcs sub-command running

itk::usual AstroImage {}

# AstroImage is a user interface class for accessing image servers
# such as DSS (Digitized Sky Survey).

class cat::AstroImage {
    inherit util::TopLevelWidget

    # constructor

    constructor {args} {
	global ::env
	eval itk_initialize $args

	if {"$itk_option(-image)" == ""} {
	    error_dialog "Can't create AstroImage widget: no skycat image widget was specified"
	    destroy $w_
	    return
	}
	
	wm title $w_ "Image Servers"
	
	# set default temp image file
	if {"$itk_option(-tmpfile)" == ""} {
	    config -tmpfile "/tmp/im[pid].[incr count_]"
	}
	
	# open log file used to keep track of URLs (for debugging)
	set dir $env(HOME)/.skycat
	# XXX tmp: also remove old log files from previous version
	catch {exec rm -f ${dir}_logo ${dir}log $dir}
	if {! [file isdirectory $dir]} {
	    catch {mkdir $dir}
	}
	catch {
	    set logfile_ [open $dir/log a+]
	    fcntl $logfile_ LINEBUF 1
	}

	# create a pipe to get feedback during image transfers
	lassign [pipe] rfd_ wfd_
	fileevent $rfd_ readable [code $this feedback]
	
	# do window layout
	layout_dialog

 	# add a short help window
	make_short_help

	# create an object for running interruptable batch image requests
	Batch $w_.batch \
	    -command [code $this request_done] \
	    -debug $itk_option(-debug)

	set initialized_ 1
	config -image $itk_option(-image)
    }


    # destructor - delete C++ based objects so that the temp image
    # files are deleted

    destructor {
	catch {$w_.im delete}
	catch {file delete $itk_option(-tmpfile)}
	catch {close $rfd_}
	catch {close $wfd_}
    }


    # add the menu bar

    method add_menubar {} {
	TopLevelWidget::add_menubar

	set m [add_menubutton File]

	add_menuitem $m command "Close" \
	    {Close this window} \
	    -command [code $this quit]
	
	set m [add_menubutton Options]
	add_menuitem $m cascade "Name Server" \
	    {Select the name server used to resolve astronomical object names} \
	    -menu [set ns_menu [menu $itk_component(options).m.ns]]

	get_name_servers $ns_menu

	# ImageServers menu
	set m [add_menubutton ImageServers]
	fill_imagesvr_menu $m imagesvr
    }


    # Fill up a menu of known data servers to choose from.
    # The serv_type argument should be one of: "catalog", "archive", "imagesvr".

    method fill_imagesvr_menu {m {serv_type "imagesvr"}} {
	if {[catch {set imagesvr_list [lsort [$w_.im info $serv_type]]} msg]} {
	    error_dialog $msg $w_
	    return
	}
	
	if {[llength $imagesvr_list]} {
	    foreach i $imagesvr_list {
		add_menuitem $m command $i \
		    "Open $serv_type: \"$i\"" \
		    -command [code $this select_imagesvr $i]
	    }
	}
    }

    # called when an imagesvr ($name) is selected from the menu.

    method select_imagesvr {name} {
	cat::AstroImage::new_imagesvr $name $itk_option(-image) $itk_option(-debug)
    }


    # quit and close the window

    method quit {} {
	if {$itk_option(-standalone)} {
	    destroy $w_
	} else {
	    wm withdraw $w_
	}
    }



    # add the name server catalogs to the given menu

    method get_name_servers {m} {
	if {[catch {set list [$w_.im info namesvr]} msg]} {
	    error_dialog $msg $w_
	    return
	}

	foreach namesvr_ $list {
	    $m add radiobutton \
		-label $namesvr_ \
		-command [code $this set_namesvr $namesvr_] \
		-variable $m
	}

	# set default name server
	global ::$m
	set $m $namesvr_
    }
    

    # do the dialog window layout
    
    method layout_dialog {} {
	# create astroimage object and setup feedback
	astroimage $w_.im
	$w_.im feedback $wfd_

	add_menubar

	pack [frame $w_.main] \
	    -side top -fill both
	pack [frame $w_.main.top -relief groove -borderwidth 2] \
	    -side top -fill x

	pack [set f [frame $w_.main.top.f -relief groove -borderwidth 2]] \
	    -side top -fill x -pady 1m
	blt::table $f

	blt::table $f \
	    [set name_ [LabelEntry $f.name \
		 -text "Object Name:" \
		 -command [code $this getimage] \
		 -labelwidth $itk_option(-labelwidth) \
		 -valuewidth $itk_option(-valuewidth) \
		 -anchor e \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    0,0 -fill x -pady 1m \
	    [set equinox_ [LabelEntry $f.equinox \
		 -text "Equinox:" \
		 -autoselect 1 \
		 -value "J2000" \
		 -command [code $this getimage] \
		 -labelwidth $itk_option(-labelwidth) \
		 -valuewidth $itk_option(-valuewidth) \
		 -anchor e \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    0,1 -fill x -pady 1m \
	    [set ra_ [LabelEntry $f.ra \
		 -text "a:" \
		 -autoselect 1 \
		 -command [code $this getimage] \
		 -labelwidth $itk_option(-labelwidth) \
		 -valuewidth $itk_option(-valuewidth) \
		 -anchor e \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-wcsfont)]] \
	    1,0 -fill x -pady 1m \
	    [set dec_ [LabelEntry $f.dec \
		 -text "d:" \
		 -autoselect 1 \
		 -command [code $this getimage] \
		 -labelwidth $itk_option(-labelwidth) \
		 -valuewidth $itk_option(-valuewidth) \
		 -anchor e \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-wcsfont)]] \
	    1,1 -fill x -pady 1m \
	    [set width_ [LabelEntry $f.width \
		 -text "Width in" \
		 -command [code $this getimage] \
		 -labelwidth [expr $itk_option(-labelwidth)-3] \
		 -valuewidth $itk_option(-valuewidth) \
		 -anchor e \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    2,0 -fill x -pady 1m \
	    [set height_ [LabelEntry $f.height \
		 -text "Height in" \
		 -command [code $this getimage] \
		 -labelwidth [expr $itk_option(-labelwidth)-3] \
		 -valuewidth $itk_option(-valuewidth) \
		 -anchor e \
		 -valuefont $itk_option(-valuefont) \
		 -labelfont $itk_option(-labelfont)]] \
	    2,1 -fill x -pady 1m \
	    [set setfrom_ [button $f.setfrom \
		 -text "Set From Image" \
		 -command [code $this set_default_values]]] \
	    3,1 -anchor e -pady 1m \
	    [set copyright_ [LabelValue $f.copyright \
		 -anchor w -relief flat \
		 -labelwidth 0 -valuewidth 0 \
		 -valuefont $itk_option(-labelfont)]] \
	    4,0 -anchor e -fill x -pady 1m -columnspan 2

	# add RA and DEC labels to width and height (different font)
	pack [label $f.width.ra -text "a:" -font $itk_option(-wcsfont)] \
	    -side left -after $f.width.label -padx 1
	pack [label $f.height.dec -text "d:" -font $itk_option(-wcsfont)] \
	    -side left -after $f.height.label -padx 2

	blt::table configure $f C1 -padx 2m

	# dialog buttons
	pack [frame $w_.buttons -borderwidth 2 -relief groove] \
	    -side top -fill x
	pack \
	    [button $w_.getimage \
		 -text "Get Image" \
		 -command [code $this getimage]] \
	    [button $w_.stop \
		 -text "Stop" \
		 -state disabled \
		 -command [code $this interrupt]] \
	    [button $w_.close \
		 -text "Close" \
		 -command "wm withdraw $w_"] \
	    -side left -expand 1 -padx 2m -pady 2m -in $w_.buttons

	# set the image server (default if not set)
	if {"$itk_option(-imagesvr)" == ""} {
	    if {[catch {set imagesvr_list [$w_.im info imagesvr]} msg]} {
		error_dialog $msg $w_
		return
	    }
	    set itk_option(-imagesvr) [lindex $imagesvr_list [expr [llength $imagesvr_list]-1]]
	}
	set_imagesvr $itk_option(-imagesvr)

	# add a progress bar at the botton
	itk_component add progress {
	    ProgressBar $w_.progress
	}
	pack $itk_component(progress) -side bottom -fill x
    }


    # add a short help window and set the help texts
    
    method make_short_help {} {
	TopLevelWidget::make_short_help

	add_short_help name_ {SIMBAD object name: resolved via SIMBAD server, if given}
	add_short_help $equinox_ {World Coordinates equinox, default J2000}
	add_short_help $ra_ {World Coordinates: right ascension (RA)}
	add_short_help $dec_ {World Coordinates: declination (DEC)}
	add_short_help $width_ {Width of image in World Coordinates (arcmin)}
	add_short_help $height_ {Height of image in World Coordinates (arcmin)}
	add_short_help $setfrom_ {Set default values from the image}

	add_short_help $w_.getimage {{bitmap b1} = start image request}
	add_short_help $w_.close {{bitmap b1} = close this window}
	add_short_help $w_.stop {{bitmap b1} = interrupt the current image fetch}
    }


    # set the name server to use to resolve object names
    
    method set_namesvr {name} {
	set namesvr_ $name
    }
    

    # interrupt the current request 

    method interrupt {} {
	$w_.batch interrupt
	catch {file delete $itk_option(-tmpfile)}
	set_state normal
    }

    
    # set/reset widget states while busy 

    method set_state {state} {
	set state_ $state
	if {"$state" == "normal"} {
	    catch {blt::busy release $w_.main}
	    catch {focus -lastfor $w_.main}
	    $w_.getimage config -state normal
	    $w_.stop config -state disabled
	} else {
	    catch {focus .}
	    catch {blt::busy hold $w_.main}
	    $w_.getimage config -state disabled
	    $w_.stop config -state normal
	}
	update idletasks
	$w_.progress reset
    }

   
    # this method is called by the fileevent handler during the image transfer 
    # from the HTTP C++ class to give the user feedback about how much there is 
    # left to copy, etc...
    # Read a line from the feedback pipe, which should contain text to be displayed
    # in the progress widget.

    method feedback {} {
	set text [gets $rfd_]
	if {"$state_" != "normal"} {
	    if {[scan $text {total length: %d bytes} n] == 1} {
		$w_.progress config -to $n
	    } elseif {[scan $text {read %d bytes} n] == 1} {
		$w_.progress config -value $n
	    } elseif {[scan $text {url: %s} url] == 1} {
	    	catch {
		    puts $logfile_ "[fmtclock [getclock]]\n$itk_option(-imagesvr)\n$url\n\n"
		    flush $logfile_;}
	     }
	}
	$w_.progress config -text $text
	update idletasks
    }
    

    # This method is called when the background request is done.
    # The arguments are the error status (0 is OK) and the result,
    # which is a list of {filename center-pos}

    method request_done {status result} {
	if {$status} {
	    set_state normal
	    error_dialog $result $w_
	    after 0 [list $w_.progress config -text $result]
	} else {
	    lassign $result filename pos copyright
	    $w_.progress config -text "Loading Image..."
	    $copyright_ configure -value $copyright
	    update idletasks
	    
	    # load the image
	    $itk_option(-image) clear
	    if {[catch {$itk_option(-image) config -file $filename} msg]} {
		error_dialog $msg
		$itk_option(-image) clear
	    }
	    if {[llength $pos] >= 2} {
		lassign $pos ra dec equinox
		$ra_ config -value $ra
		$dec_ config -value $dec
		$name_ config -value ""
	    }
	    set_state normal
	    # guidecam_block begin
	    global gvar
	    if [info exists gvar(armDisplay)] {$gvar(armDisplay) updateImageData}
	    # guidecam_block end
	}
    }


    # set (open) the current image server - called when user selects a 
    # name from the menu
    
    method set_imagesvr {name} {
	config -imagesvr $name
	wm title $w_ $name
	wm iconname $w_ [$w_.im shortname $name]
	$w_.im open $name
	$copyright_ configure -value [$w_.im copyright]

	set image $itk_option(-image)
	set instance_idx "$name,$image"
	set instances_($instance_idx) $w_ 
    }

   
    # member proc (like a static member function) to create an instance of this
    # class for the named imagesvr (or reuse the existing one for the imagesvr)
    #
    # The args are: 
    #
    #   name  - long name of imagesvr from config file
    #
    #   image - name of skycat itcl image widget
    #
    #   debug - flag: if true, run queries in foreground

    public proc new_imagesvr {name image debug} {
	set i "$name,$image"
	if {[info exists instances_($i)] && [winfo exists $instances_($i)]} {
	    utilRaiseWindow $instances_($i)
	    return
	}

	set instances_($i) \
	    [AstroImage .ai[incr n_instances_] \
		 -image $image -debug $debug -imagesvr $name -transient 0]
    }
    
    
    # return a list of the instances of this class

    public proc instances {} {
	set list {}
	if {[info exists instances_]} {
	    foreach i [array names instances_] {
		if {[winfo exists $instances_($i)]} {
		    lappend list $instances_($i)
		}
	    }
	}
	return $list
    }

    
    # Get the requested image from the image server based on the current options
    # and display the resulting image (args are ignored here)

    method getimage {args} {
	getimage_from_args \
	    [$ra_ get] [$dec_ get] [$name_ get] [$equinox_ get] \
	    [$width_ get] [$height_ get] 0
    }
    
   
    # Get the requested image from the image server based on the given arguments
    # for the world coord position or name and the given width and height
    # if flag is 1, the entries are updated from the arguments.

    method getimage_from_args {ra dec name equinox width height {flag 1}} {
	if {$flag} {
	    $ra_ config -value $ra
	    $dec_ config -value $dec
	    $name_ config -value $name
	    $equinox_ config -value $equinox
	    $width_ config -value $width
	    $height_ config -value $height
	}

	set cmd "$w_.im getimage -tmpfile $itk_option(-tmpfile)"
	if {"$equinox" != ""} {
	    lappend cmd "-equinox" $equinox
	}
	if {"$name" != ""} {
	    lappend cmd "-nameserver" $namesvr_ "-name" $name
	} elseif {"$ra" != "" && "$dec" != ""} {
	    lappend cmd "-pos" [list $ra $dec]
	} else {
	    warning_dialog "Please specify either an object name or a position in WCS" $w_
	    return
	}
	lappend cmd -width $width
	lappend cmd -height $height

	# start the request in the background
	set_state disabled
	$w_.progress config -text "Attempting to contact image server..."
	$w_.progress look_busy

	$w_.batch bg_eval [code [format {
	    list [%s] [%s.im centerpos] 
	} $cmd $w_]]
    }
    

    # Set the default values for the form entries:
    # set the default position (RA, DEC) to the image center, 

    method set_default_values {} {
	$name_ config -value ""
	if {[catch {set center  [$image_ wcscenter]}]} {
	    set center "[expr [$image_ width]/2.] [expr [$image_ height]/2.]"	
	}
	# only set if there is an image loaded
	if {[llength $center] >= 2 && ! [$image_ isclear]} {
	    lassign $center ra dec equinox
	    # set equinox [$image_ wcsequinox]
	    $ra_ config -value $ra
	    $dec_ config -value $dec
	    $equinox_ config -value $equinox
	    $width_ config -value [format "%.2f" [$image_ wcswidth]]
	    $height_ config -value [format "%.2f" [$image_ wcsheight]]
	} else {
	    $ra_ config -value ""
	    $dec_ config -value ""
	    $width_ config -value 10.0
	    $height_ config -value 10.0
	}
    }

    
    # -- options --

    # default image server name 
    itk_option define -imagesvr imagesvr Imagesvr "dss@eso"

    #  image handle (itcl class)
    itk_option define -image image Image {} {
	if {$initialized_} {
	    set canvas_ [$itk_option(-image) get_canvas]
	    set image_ [$itk_option(-image) get_image]
	    set_default_values
	}
    }

    # font to use for labels
    itk_option define -labelfont labelFont LabelFont -Adobe-helvetica-bold-r-normal--12*

    # font to use for values
    itk_option define -valuefont valueFont ValueFont -adobe-courier-medium-r-*-*-*-120-*-*-*-*-*-*

    # font to use for ra,dec labels
    itk_option define -wcsfont wcsFont WcsFont -*-symbol-*-*-*-*-14-*-*-*-*-*-*-*

    # set the width for  displaying labels
    itk_option define -labelwidth labelWidth LabelWidth 14

    # set the width for  displaying labels
    itk_option define -valuewidth valueWidth ValueWidth 14

    # set the anchor for labels
    itk_option define -anchor anchor Anchor e

    # text for  label displayed at top
    itk_option define -title title Title "Image Request Parameters"

    # flag: if true, run queries in foreground for better debugging
    itk_option define -debug debug Debug 0

    # temp image file to use
    itk_option define -tmpfile tmpfile Tmpfile {}
    
    
    # -- protected members --

    #  canvas window containing main image
    protected variable canvas_

    # internal rtdimage widget for main image
    protected variable image_

    # name field widget
    protected variable name_

    # equinox field widget
    protected variable equinox_

    # ra field widget
    protected variable ra_

    # dec field widget
    protected variable dec_

    # width field widget
    protected variable width_

    # height field widget
    protected variable height_

    # "set from image" field widget
    protected variable setfrom_

    # copyright field
    protected variable copyright_

    # pipe to read feedback during image transfer
    protected variable rfd_

    # pipe to write feedback during image transfer
    protected variable wfd_

    # name server to use to resolve object names
    protected variable namesvr_ {}

    # current state: normal, disabled (i.e.: waiting)
    protected variable state_ {normal}

    # log file handle (used to log URLs)
    protected variable logfile_ stdout

    # flag: set at end of constructor
    protected variable initialized_ 0

    # -- common variables (shared by all instances) --
    
    # count used for filename generation
    common count_ 0

    # array mapping catalog name to widget/class name
    common instances_

    # instance count
    common n_instances_ 0

}


