#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panCommandReply.tcl,v 5.19 2003/02/14 14:23:53 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    03/03/98  created
#           09/03/98  Add verbose checkbutton to CFW
#           20/04/98  Fixed bug when deleting the CFW
#

#************************************************************************
#   NAME
#     panDisplayCommand, panDisplayComment, panDisplayReply - Set of 
#             procedures to interact to the "Command Feedback Window"
#   SYNOPSIS
#     panDisplayCommand  <cmd> <text>
#     panDisplayComment  <text>
#     panDisplayReply    <cmd> <cmdId> <procName> <errFlag> <lastReply> 
#                        <replyIdx> <text>
#   DESCRIPTION
#     This is a set of procedures allowing the GUI developer to interact 
#     with the "Command Feedback Window" (CFW) . This is a widget that can 
#     be added at the bottom of a panel by selecting the associated 
#     checkbutton in the "Std. Component" menu of the panel editor.
#
#     Parameter description :
#     <cmd>      - String specifying the command
#     <cmdId>    - Command identifier : what returned by seq_msgSendCommand
#     <procName> - Destination process
#
#     <text> - Free text used to describe the command or the reply message
#              or a comment to the operator
#     <errFlag>  - Indicates if the reply is an error : boolean value 1/0
#     <replyIdx> - Reply sequence number
#
#     panDisplayComment 
#        This procedure allows to display a description of the current action
#        or status. This can be used to ive the user high level information
#        on the ongoing activities.
#
#     panDisplayCommand
#        Let the user know that a given command has been sent/invoked.
#        The user has the possibility to add a comment gving more details
#        of what the current action does.
#
#        This is automatically called by panSendCommand.
#
#     panDisplayReply
#        Displays the reply associated to a given command.
#        
#        This is automatically called by panSendCommand.
#  
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#    
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# This procedure checks if the "Command Feedback Window" exists

proc panCommandReplyExist {} {
   global panWdgReply

   if {[info exists panWdgReply]   == 0 } { return 0}
   if {[winfo exists $panWdgReply] == 0 } { return 0}

   return 1
}

proc panAddToFeedbackWin { what } {
   global panWdgReply
   if {[$panWdgReply getSize] > 500 } { $panWdgReply deleteItem 0 }

   # Add new element to the Command Feedback Window
   $panWdgReply addItem end $what

   # Make last element always visible
   $panWdgReply component uifListBox see end
}


proc panSetCommonInfo { cmd {procName ""}  {cmdId  ""} } {
   global par

   set buf [format "%s %-8s" [fmtclock [getclock] "%T" ] $cmd]

   if {$par(cfwCMD_ID)} {
      append buf [format " %4s" $cmdId]
   } 

   if {$par(cfwPROC_NAME)} {
      append buf [format " %-20s " $procName]
   } 

   return "${buf}>"
}


proc panDisplayReply { cmd cmdId procName errFlag lastReply replyIdx text } {

   if {![panCommandReplyExist]} { return }

   if {$errFlag} {
      set logFormat "ERROR %2s   %s" ; set seqReply " "
   } else {

      if {$lastReply} {
          set seqReply L
      } else {
          set seqReply $replyIdx   
      }
      set logFormat "REPLY/%2s   %s"
   }

   set tmp1  [panSetCommonInfo $cmd $procName $cmdId]
   set tmp2  [format $logFormat $seqReply $text]

   panAddToFeedbackWin "$tmp1 $tmp2"
}


proc panDisplayCommand { cmd {procName ""} {text ""} {cmdId ""} } {
   
   if {![panCommandReplyExist]} { return }

   set tmp2 "INVOKED"
   if {![cequal $text ""]} { append tmp2 "    $text" }

   set tmp1  [panSetCommonInfo $cmd $procName $cmdId]
   panAddToFeedbackWin "$tmp1 $tmp2"
}


proc panDisplayComment { text } {
   
   if {[panCommandReplyExist] == 0 } { return }

   set text [string trim $text " *"]
   set tmp "            *** $text ***"

   panAddToFeedbackWin "[panSetCommonInfo "" "" "" ] $tmp"
}

#-----------------------------------------------------------------------------
# PROCEDURE :
#-----------------------------------------------------------------------------

proc checkCmdReply {} {
   global ckCmdReply par EDITING  panCvYSize cv mainHelp 
   global panWinYSize panWinXSize

   set panReplyArea  $par(ROOT_PANEL).replyArea ;# Name of Cmd. reply area
   set panWdgReply   $panReplyArea.reply
   set panCvYSize    [winfo height $cv]

   panDisableGeoHandle $par(ROOT_PANEL)

   if { $ckCmdReply == 0 } {
      if {[winfo exists $panWdgReply]} {
         set sizeCmdReply [winfo reqheight $panReplyArea]
         destroy $panWdgReply $panReplyArea.f1
         pack forget $panReplyArea

         incr panCvYSize $sizeCmdReply 
         $cv configure -height $panCvYSize
      }
   } else {

      if {[winfo exists $panWdgReply] == 0 } {

          createCmdReply $par(ROOT_PANEL)

          set sizeCmdReply [winfo reqheight $panReplyArea]
          set ysize [expr $panWinYSize + $sizeCmdReply]

          incr panCvYSize -$sizeCmdReply 
          $cv configure -height $panCvYSize

          wm geometry $par(ROOT_PANEL) ${panWinXSize}x${ysize}
      }
   }

   panEnableGeoHandle $par(ROOT_PANEL)
   update
   return
}
 

proc createCmdReply { root } {
   global par panWdgReply

   set font "-Adobe-courier-medium-r-normal--12*-iso8859-1" 

   if {[info exists par(sizeCmdReply)]} {
      set h $par(sizeCmdReply)
   } else {
      set h 2
   }

   set geo 10x${h}

   set panReplyArea  $root.replyArea ; # Name of Cmd. reply area
   set panWdgReply   $panReplyArea.reply  
   set f1 [frame $panReplyArea.f1]
  
   label   $f1.lbl -text "Command Feedback Window" \
           -font "-Adobe-helvetica-bold-r-normal--14*-iso8859-1"

   UifMenuButton  $f1.opt -label Options -width 10
   [$f1.opt component uifMenuButton] configure -font -Adobe-courier-medium-r-normal--12*-iso8859-1 

   $f1.opt addCheckButton "Command Id"  -1 "" par(cfwCMD_ID)
   $f1.opt addCheckButton "Proc. Name"  -1 "" par(cfwPROC_NAME)

   $f1.opt addSeparator

   $f1.opt addRadioButton "Small Font" -1 "panSetFontCWF $h" \
          par(fontCWF) -Adobe-courier-medium-r-normal--12*-iso8859-1 

   $f1.opt addRadioButton "Normal Font" -1 "panSetFontCWF $h" \
          par(fontCWF) -Adobe-courier-medium-r-normal--14*-iso8859-1

   $f1.opt addRadioButton "Big Font" -1 "panSetFontCWF [incr h -1]"\
          par(fontCWF) -Adobe-courier-medium-r-normal--18*-iso8859-1

   $f1.opt addSeparator 
   $f1.opt addSeparator 
   $f1.opt addCommand "Clear" -1 "$panWdgReply deleteItem 0 end"


   UifListBox $panWdgReply -geometry $geo -font -Adobe-courier-medium-r-normal--14*-iso8859-1

   # Initialize radio buttons for the font selection
   set par(fontCWF) [$panWdgReply cget -font]

   pack $panReplyArea  -after $root.top -fill x
   pack $f1.lbl  $f1.opt -padx 5 -side left -pady 3
   pack $f1 -anchor nw  -padx 5 
   pack $panWdgReply   -anchor nw -fill x -padx 3 -pady 3
         
   update idletasks
}

proc panSetFontCWF { h } {
   global par panWdgReply 
   [$panWdgReply component uifListBox] configure -height $h
   $panWdgReply configure -font $par(fontCWF) 
}



#
# ___oOo___
