#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panDbWrite.tcl,v 5.19 2003/02/14 14:23:46 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  04/06/95  created
# mcomin  02/08/95  Check if panDbData exists
# mcomin  24/10/95  Add recursive procedure to find list of input widgets
# mcomin  11/01/96  Add man page.
#
#************************************************************************
#   NAME
#     panDbWrite, panDbWriteWidget, panDbWriteClass - Store a value to a database element
#
#   SYNOPSIS
#
#     panDbWrite       <symbAddress> <value>
#     panDbWriteWidget <canvas> ?list of widgets?
#     panDbWriteClass  <class_widget_path> ?list of widgets?
#
#   DESCRIPTION
# 
#     panDbWrite         
#     This procedure takes the database <symbAddress> and convert it into 
#     its absolute address, according to the Current Working Point.
#     In this way relative addresses are properly written according
#     to the current 'cwp' value.
#                 
#     panDbWriteWidgets  
#     Loops on the widget list and stores in the database the values 
#     of those associated to a database variable.
#
#     The procedure requires as first parameter the name of the parent
#     window containing the group of widgets to take into account. 
#
#     The keyword "all" is used to address all the input widgets of the panel
#
#     panDbWriteClass    
#     Loops on the widgets contained in a uifClass-widget and stores
#     all values associated to a database input.
#
#     The keyword "all" is used to address all the input widgets of the panel
#
#     In case of failure an error is logged and also printed on the screen.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#     none
#
#   CAUTIONS
#
#   EXAMPLES
#
#   If a want to write into the database element :PARAMS:SCALARS.scalar_int8,
#   there are the following possibilities :
#  
#   - Using the absolute address :
#     panDbWrite @wte16:PARAMS:SCALARS.scalar_int8 1
#   
#   - Using a relative address and cwp :
#     seq_dbSetCwp @wte16:PARAMS
#     panDbWrite @wte16<relative>SCALARS.scalar_int8 1
#   
#   - Using the alias address :
#     panDbWrite @wte16<alias>SCALARS.scalar_int8 1
#
#   Other examples.
#
#   panDbWriteWidgets $cv all
#   panDbWriteWidgets $cv xxx-wdg23 xxx-wdg7
#
#   set myClass $cv.wdgClassName
#   panDbWriteClass $myClass all
#   panDbWriteClass $myClass wdgClassName-wdg23 wdgClassName-wdg7
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------------
# PROCEDURE : panDbWrite - 
#------------------------------------------------------------------------------

proc panDbWrite { symbAddr value } {
   global par panDEBUG

   if { $par(CCS) == 0 } { return }
   
   set symbAddr [panDbGetFullAddress "" $symbAddr]

#   puts "DBG > panDbWrite :  $symbAddr $value   -  info = $attrInfo"
         
   # If attrInfo is missing it is called in the routine itself
          
   if { [catch {seq_dbWriteSymbolic $symbAddr $value} ret] } {
      panLogDbOperation  Write $symbAddr $ret $value
      seq_errDisplay
   }
}



proc panGetInputList { cv } {
   global i_type db_type i_dbVar panDbData panWidget

   set myList {}

   foreach item [panWdgReferenceList $cv all uifType] {
      if {[info exists panDbData($item)] == 0 }  {continue}
      if {[info exists panWidget($item)] == 0 }  {continue}
      if {[getWidget   $item $i_dbVar ] == 0}    {continue}      
      if {![cequal [getDbConfig $item $db_type] in]}  {continue}

      lappend myList $item
   }
##   puts "DBG> Input Widget List = $myList"
   return $myList
}


#------------------------------------------------------------------------------
# PROCEDURE : panDbWriteWidget - 
#------------------------------------------------------------------------------

proc panDbWriteWidget { cv args } {
   global panWidget gvar panDbSymbAddr par EDITING panInputList panDEBUG
   global i_dbVar db_var db_onval db_offval db_min db_max db_env db_type

#   puts "DBG > dbWriteWidget cv = $cv   args = $args \n"   

   if {[winfo exists $cv] == 0} {
      puts "panDbWriteWidget : Write Operation Not Executed"
      puts "Reason           : Canvas ($cv) does not exists - check parameters"
      return
   }

   if {$par(CCS) == 0 } { return }

   if {[cequal [lindex $args 0] all]} {
        set widgetList [panGetInputList $cv] 
   } else {
        set widgetList $args
   }

#   puts "DBG > dbWriteWidget $widgetList \n"   

   foreach name $widgetList {

      if { [getWidget $name $i_dbVar] == 0}        { continue }
      if { [getDbConfig $name $db_type] != "in" }  { continue }  
    
      set symbAddr $panDbSymbAddr($name)
      set myAddr   [getDbConfig $name $db_var]
      set value    $gvar($myAddr)

#      puts "DBG > dbWrite $symbAddr = $value \n"
                   
      if { [catch {seq_dbWriteSymbolic $symbAddr $value} ret] } {
         panLogDbOperation  Write $symbAddr $ret $value
      }
      
   }; #end of foreach
}


#------------------------------------------------------------------------------
# PROCEDURE : panDbWriteClass - 
#------------------------------------------------------------------------------

proc panDbWriteClass { className args } {
   global panWidget gvar panDbSymbAddr par EDITING panDEBUG
   global i_dbVar db_var db_type  

   set cv $className.cv
#   puts "DBG > dbWriteClass cv = $cv   args = $args \n" 

   if {[winfo exists $cv] == 0} {
      puts "panDbWriteWidget : Write Operation Not Executed"
      puts "Reason           : Canvas ($cv) does not exists - check parameters"
      return
   }

   if {$par(CCS) == 0 } { return }

   if {[cequal [lindex $args 0] all]} {
        set widgetList [panClassWidgetNames $className uifType]
   } else {
        set widgetList $args
   }

#   puts "DBG > dbWriteClass $widgetList \n"   

   foreach name $widgetList {

      if { [getWidget $name $i_dbVar] == 0}        { continue }
      if { [getDbConfig $name $db_type] != "in" }  { continue }  
    
      set symbAddr $panDbSymbAddr($name)
      set myAddr   [getDbConfig $name $db_var]
      set value    $gvar($myAddr)

#      puts "DBG > dbWrite $symbAddr = $value \n"
                   
      if { [catch {seq_dbWriteSymbolic $symbAddr $value} ret] } {
         panLogDbOperation  Write $symbAddr $ret $value
      }
      
   }; #end of foreach
}

#
# ___oOo___
