#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panMenuOperation.tcl,v 5.19 2003/02/14 14:23:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin   19/11/97   created
#          12/12/97   add proc panSetMbState
#
#************************************************************************
#   NAME
#     panMenuOperations - Set of procedures to interact with the menu bar.
#   SYNOPSIS
#     panEnableMenuEntry  <mbLabel> <wdgLabel>
#     panDisableMenuEntry <mbLabel> <wdgLabel>
#     panQueryMenuEntry   <mbLabel> <wdgLabel> 
#
#     panGetMbPath        <mbLabel>
#     panSetMbState       <mbLabel> <state>
#   DESCRIPTION
#
#     These procedures allow the user to get the description of a given menu
#     entry and to change its state.
#
#     The following conventions are used :
#     <mbLabel>  - Identifies the label of a menuButton. 
#                  This is one element of the menu bar.
#     <wdgLabel> - Label of a menu entry.
#     <state>    - New state of the button.
#                  Valid velues are :  disabled or normal.
#
#     panGetMbPath <mbLabel>
#        Returns the full widget name of a menubutton with label <mbLabel>
#        If returns "" if the menubutton does not exits.
#
#     panSetMbState :
#        This procedure set the state of a menuButton
#        It returns 0 in case of failure, 1 otherwise
#
#     panEnableMenuEntry, panDisableMenuEntry :
#        This procedure enables/disables a menu entry with label <wdgLabel>, 
#        under the menubutton <mbLabel>.
#
#        It returns 0 in case of failure, 1 otherwise
#
#     panQueryMenuEntry <mbLabel> <wdgLabel> 
#        This procedure searches for a menu entry with  label <wdgLabel>, 
#        under the menubutton <mbLabel> and returns a list with the following
#        elements :
#          - Full name of the menu containing the entry
#          - Type of the entry : cascade, command, checkbutton, radiobutton
#          - Position in the menu.
#
#        It returns ""  - if the search was not successful
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#     if {[set ret [panQueryMenuEntry MenuButton1 cmd1]] == ""} {
#        Process failure condition ....
#     }
#    
#     if {![panEnableMenuEntry MenuButton1 cmd1]} {
#        Process failure condition ....
#     }
#
#     if {![panSetMbState MenuButton1 disabled]} {
#        Process failure condition ....
#     }
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panSetMbState { wdgLabel state } {
   if {[set w [panGetMbPath wdgLabel]] != ""} {
      $w configure -state $state
      return 1
   } else {
      return 0
   }
}

proc panEnableMenuEntry { mbName label } {
   set menuInfo [panQueryMenuEntry $mbName $label]
   if {! [cequal $menuInfo ""] } {
      lassign $menuInfo menuPath menuType idx
      $menuPath entryconfig $idx -state normal
      return 1
   } else {
      return ""
   }
}

proc panDisableMenuEntry { mbName label } {
   set menuInfo [panQueryMenuEntry $mbName $label]
   if {! [cequal $menuInfo ""]} {
      lassign $menuInfo menuPath menuType idx
      $menuPath entryconfig $idx -state disabled
      return 1
   } else {
      return ""
   }
}

# Returns the full path name of a menubutton or "" if not found.
# (the ones at the top of the menubar)

proc panGetMbPath { wdgLabel } {
   global menu  m_label m_name cv
 
   set root  [lindex [split $cv "."] 1]
   set menuBar .${root}.topMenu.menu

   foreach i [array names menu] {
       if {[lindex $menu($i) $m_label] == "$wdgLabel"} {
          set wdgName [lindex $menu($i) $m_name]
          return "$menuBar.$wdgName"
       }
   }
   return ""
}


# Search a menu entry with the <label> in the menu tree under a given 
# menuButton : it return a list of tree elements :
# <entry path> <type> <index>

proc panQueryMenuEntry { mbLabel label } {
   if {[set mbPath [panGetMbPath $mbLabel]] != ""} {
      return [panVisitMenuTree $mbPath $label]
   } else {
      return ""
   }
}

proc panVisitMenuTree {topMenu label} {
   set menuInfo ""

   switch [winfo class $topMenu] {
       Menubutton {
            set topMenu [$topMenu cget -menu]
            set maxId   [$topMenu index end] 
       }
       Menu    { set maxId  [$topMenu index end] }
       default { set maxId 0 }
   }

   # Empty menu 
   if { $maxId == 0 } { return $menuInfo }

   # Check if the label exists under this menu

   if {[set idx [tkMenuFindName $topMenu $label]] != ""} {
      return [list $topMenu [$topMenu type $idx] $idx]
   }

   # Continue the search
   set i 0

   while {$i <= $maxId && [cequal $menuInfo ""]} {
       if {[cequal [$topMenu type $i] cascade ]} {
          set tmpMenu [$topMenu entrycget $i -menu]
          set menuInfo [panVisitMenuTree $tmpMenu $label]
       }
       incr i
   }

   return $menuInfo
}

#----------------------------------------------------------------------
#     Obsolete procs ....
#----------------------------------------------------------------------
# Given the menu entry label, it returns the entry index.
# The index is used to configure the menu entry.
# It return "" if the label is not found.
# - mbName        Name of the top menu button
# - subMenuPath   Path to the cascade menu

proc panGetMenuPath { mbName subMenuPath } {

   if {[set mbPath [panGetMbPath $mbName]] == "" } { return "" }

   set menuName [$mbPath cget -menu]
   if {$subMenuPath != ""} {
      append menuName .$subMenuPath
   }

   return $menuName
}


proc panGetMenuEntryId { mbName subMenuPath label } {
   if {[set menuPath [panGetMenuPath $mbName $subMenuPath]] == "" } {
      return ""
   }

   set idx [tkMenuFindName $menuPath $label]
   return $idx
}


proc panDisableMenu { mbName subMenuPath label } {
   if {[set menuPath [panGetMenuPath $mbName $subMenuPath]] == "" } {
      return ""
   }
   
   if {[set idx [tkMenuFindName $menuPath $label]] != ""} {
      $menuPath entryconfig $idx -state disabled
   }
}


proc panEnableMenu { mbName subMenuPath label } {
   if {[set menuPath [panGetMenuPath $mbName $subMenuPath]] == "" } {
      return ""
   }
   
   if {[set idx [tkMenuFindName $menuPath $label]] != ""} {
      $menuPath entryconfig $idx -state normal
   }
}


#
# ___oOo___
