#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panSelectFile.tcl,v 5.19 2003/02/14 14:23:45 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    11/05/95  created
# M.COMIN   01/06/95  Fixed bug associated to change of directory
# ------------------
# mcomin    12/04/96  Panel can be re-sized
# mcomin    22/04/97  Panel is not destroyed but only iconized.
#                     It points to the last selected directory
#           24/04/97  Add options
#

#************************************************************************
#   NAME
#
#   panSelectFile  -  Calls a fileSelectionBox to select a file
#
#   SYNOPSIS
#
#   panSelectFile <dir> <filter> ?options?
#
#   DESCRIPTION
#
#   This procedure invokes a File Selection Box to browse the file system.
#    
#      dir      Full path name of the directory where to start.
#      filter   Defines a pattern to select a file from in the <dir> directory.
#      options  Is a list of possible options to control the behaviour 
#               of the file selction box :
#                         
#        -top <widgetName>  Defines the toplevel widget name.
#                           Used when several File Selection Boxes are
#                           active at the same time.
#
#        -mode <w/r>        Default value is "r". When "w" is specified
#                           the procedure issues a dialog to ask the
#                           user to confirm the selection of such a file.
#                           The procedure checks that the file exists and
#                           if the user has proper write permission to
#                           to access the file.
#
#        -title <string>     Title of the File Selection Box 
#   
#        -subDir <path>     Relative directory path. This is automatically 
#                           appended to the MODROOT,INTROOT,VLTROOT,VLTDATA
#
#        -iconify           The File Selection Boxes is iconified after the 
#                           selection otherwise it is destroyed.
#
#         -cd               Forces to take always into account the directory 
#                           specified by <dir>
#  
#   The fileSelectionBox provides the following interactive commands :
#   
#      OK      Select the file and close the dialog.
#      FILTER  Update the list of files according to the current filter
#      CANCEL  Quit the dialog          
#      HELP    Not used
#   
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   The return value depends on which button the user clicked :
#
#     OK     The full path name of the selected file.
#     CANCEL An empty string
#
#   CAUTIONS
#
#   EXAMPLES
#
#   global env
#   
#   set myDir   $env(HOME)/CCS/panel/src
#   set filter  "*.tcl"
#   set filePath [panSelectFile $myDir $filter]
#
#   if { [cequal $filePath ""] } {
#      No files selected
#      return
#   }
#
#
#   SEE ALSO
#
#   UifFileSelectionBox
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panSelectFile { dir filter args } {
   global par

   # Set default values for options
   set par(HOME) [pwd]
   set top       ".panSelFile"; # Default value
   set mode      "r"
   set title     "File Browser"
   set iconify   0
   set subDir    ""

   # Extract options from 'args' parameters

   set len [llength $args]

   for {set idx 0} {$idx < $len} {incr idx} {
       switch -- [lindex $args $idx] {
	  "-top"     { set top    [lindex $args [expr $idx +1]] }
          "-mode"    { set mode   [lindex $args [expr $idx +1]] }
          "-title"   { set title  [lindex $args [expr $idx +1]] }
          "-subDir"  { set subDir [lindex $args [expr $idx +1]] }
          "-iconify" { set iconify 1 }
       }
   }

   if {! [winfo exists $top]} {

#     puts "UifFileBrowser $top -iconify $iconify -mask $filter -title $title -subDir $subDir -newDir $dir"

      UifFileBrowser $top -iconify $iconify -mask $filter \
                     -title $title -subDir $subDir -newDir $dir
   } else {

      $top configure -mask $filter -iconify $iconify -subDir $subDir

      if {[lsearch $args "-cd"] != -1 } {
         $top configure -subDir $subDir -newDir $dir
      }

      $top filter $filter
      wm deiconify $top
   }

   set tmp [$top getFile]
   lassign $tmp filePath fileInfo

   if {[cequal $filePath ""]} { return $filePath }

   # Add additional checks if the file may be opened in "w" mode.

   if {[cequal $mode w]} {
      set msg1 "" ; set msg2 ""
      set msg "File '[file tail $filePath]' "

      if {[lindex $fileInfo 2] == 0} {
         set msg1 "\ndoes NOT have WRITE PERMISSIONS"
      }
      
      if {[lindex $fileInfo 3]} {
         set msg2 "\nMAY BE OVERWRITTEN"
      }
 
      if {$msg1 != "" || $msg2 != ""} {
	 append msg ${msg1}${msg2}
         if {[tk_dialog .d "Dialog" $msg question 1 Continue Cancel]} {
            set filePath "" 
         }
      }
   }

   cd $par(HOME)
   return $filePath
}

#
# ___oOo___
