#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panSendCommand.tcl,v 5.19 2003/02/14 14:23:43 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   28/03/95  created 
# M.COMIN   14/06/95  Fixed bug with braces and quotes for LCU appl.
# M.COMIN   16/09/95  Add reply widget area
# M.COMIN   02/10/95  Full implementation of cmd reply widget + date
# M.COMIN   21/11/95  Bug fixing during re-sizing of the window
#                     Replay widget does not appear if canvas too small.
# M.COMIN   14/12/95  Add procedure createCmdReply()
# M.COMIN   15/12/95  Add proc panDisplayReply ... reply is added also in case
#                     of error
# -------------------
# mcomin    08/01/96  In panDisplayReply (added 'exists' check on panWdgReply)
# mcomin    17/01/96  Fixed probable bug passing param. timeout
#
# mcomin    27/02/96  Porting to TK4.0
#           16/09/96  Add use of parameter 'flag'
#           15/09/97  Show each reply of the message
#           20/03/98  Take away 'after' before seq_errDisplay
#
#************************************************************************
#   NAME
#     panSendCommand - Send Command to processes and waits for replies
#   SYNOPSIS
#     panSendCommand <env> <proc> <cmd> <flags> <timeout>  ?args?
#   DESCRIPTION
#
#   This TCL procedure allows the user to send commands to a process.
#
#     <env>       CCS Environment name
#     <proc>      Name of the process where to send the message to.
#     <cmd>       String defining the command
#     <flags>     Used to define the checks to be performed before sending
#                 the message, and how to process reply messages
#                 Valid keywords are :
#
#                 NO_CHECK, CHECK_ALL  To check the syntax of the message 
#                                      when it is sent.
#                 -quiet         Replies are not displayed
#                 -noErrDisplay  Error dialog is not displayed.
#                 -all           Return all replies in one go separated 
#                                by a newline
#                 -last          Wait for the last reply and returns only that.
#
#                 If flags is "" the option CHECK_ALL is taken by default
#
#     <timeout>   Timeout (in mSec) during the receive.
#     ?args?      Optional list of parameters to send along with the
#                 command.  This list of parameters can also contain widget
#                 names : in this case the name of the widget is replaced
#                 with the value associated to the widget.
#
#   The current implementation allows to send 'synchronous command', 
#   therefore the procedure returns only when all replies have been received
#   or at the first error reply.
#
#   The user is informed in case of error reply. The error dialog appears
#   on the screen with the error description and requires an acknowledge before
#   continuing.
#   FILES   
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   SUCCESS  Return the string with the reply information.
#   FAILURE  Return the string "FAILURE"
#
#   CAUTIONS
#     It is advisable to use the 'position independent' syntax to send
#     commands to the LCUs, because the arguments are automatically 
#     concatenated with ","
#
#   EXAMPLES
#
#     This code to be used to handle the command's reply.
#   
#     set myReply [panSendCommand ... ]
#
#     if {$myReply == "FAILURE"} {
#        process failure condition
#     } else {
#        process reply information
#     }
#
#     Hereafter there are examples on how to send commands to a process.
#
#     This example uses the following conventions :
#
#     - wdg1, wdg2  are the names of two widgets
#     - var1, var2  are the names of the variables associated to the widgets
#     - par1, par2  are the argument names of the parameters accepetd by the
#                   command.
#
#     COMMAND with a SINGLE PARAMETER
#
#     panSendCommand $env $proc CMD "" 1000 wdg1
#     panSendCommand $env $proc CMD "" 1000 $gvar(var1)
#       
#     Several arguments can be bundle together a a single parameter
#
#     panSendCommand $env $proc CMD "" 1000 \"dummyVar $gvar(var1) wdg2 \"
#
#         NOTE : it is important to place \" at the beginning and 
#                at the end of the parameter list when widget names
#                appear in the parameter list otherwise there will be no
#                substitution with the corresponding value.
#
#     COMMAND with SEVERAL PARAMETERS
#
#     This is 'position dependent' syntax
#
#     panSendCommand $env $proc CMD "" 1000 wdg1 wdg2
#     panSendCommand $env $proc CMD "" 1000 $gvar(var1) $gvar(var2)
#
#     Parameters must be separated by "," in order to have rage checking work
#     but references to widget names must be 
#
#     panSendCommand $env $proc CMD "" 1000 wdg1 , wdg2
#     panSendCommand $env $proc CMD "" 1000 $gvar(var1) , $gvar(var2)
#       
#     'position independent' syntax : arguments can be also specified in
#     pairs -<name> <value> . In this case it is not important the order 
#     within the parameter list.
#
#     panSendCommand $env $proc CMD "" 1000 -par1 wdg1 -par2 wdg2
#     panSendCommand $env $proc CMD "" 1000 -par2 wdg2 -par1 $gvar(var2)
#
#     COMMANDS to LCUs 
#
#     These commands require the arguments to be separated by ",".
#     panSendCommand $env $proc CMD "" 1000 "$gvar(var1),$gvar(var2)"
#
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------------
# PROCEDURE : panSendCommand - 
#------------------------------------------------------------------------------

proc panSendCommand { envName procName cmd {flags {}} {timeout {5000}} args } {
   global gvar panWidget i_var panDEBUG

   set quietFlag   0
   set noErrDialog 0
   set cmdCheck  -check

   if {[set idx [lsearch $flags CHECK_ALL]] != -1} {
      set flags [lreplace $flags $idx $idx] ; 
   } 

   if {[set idx [lsearch $flags NO_CHECK]] != -1} {
      set flags [lreplace $flags $idx $idx] ; set cmdCheck -nocheck
   }

   if {[set idx [lsearch $flags "-quiet" ]] != -1} {
      set flags [lreplace $flags $idx $idx] ; incr quietFlag
   }

   if {[set idx [lsearch $flags "-noErrDisplay" ]] != -1} {
      set flags [lreplace $flags $idx $idx] ; incr noErrDialog
   }

   set parList  {}
   foreach item $args {

      if { ![info exists panWidget($item)]} { 
         append parList "$item "; continue 
      } 

      set varName [getWidget $item $i_var]

      # Get current value of the variable associated to the widget
      if {[info exists gvar($varName) ]} {append parList "$gvar($varName)  "}
        
   }; #end of foreach 

   if {[info exists panDEBUG] && [cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > panSendCommand $envName $procName $cmd"
      puts "             flags = \"$flags\"  parameters = $parList"
   }

   #
   # Send Command
   #

   if {[catch {seq_msgSendCommand $cmdCheck $envName $procName $cmd $parList} ret] } {

      if { $noErrDialog == 0 } { seq_errDisplay }
      if { $quietFlag == 0 } {
         panDisplayCommand $cmd $procName "Failure sending the command"
      }

      return "FAILURE"
   } else {
      set cmdId $ret
      if { $quietFlag == 0 } {
         panDisplayCommand $cmd $procName "" $cmdId
      }
   }
 
   #
   # Wait for Replies
   #
  
   set lastReply 0
   set idx  0

   while { $lastReply == 0 } { 
      
      set ret "" 
      set errFlag 0
      incr idx
 
      if {[catch {eval seq_msgRecvReply $flags $cmdId errFlag lastReply -${timeout} } ret]} {
         panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret

         if { $noErrDialog == 0 } seq_errDisplay
         return "FAILURE"
      }

      # Add current reply, because it's not the last one

      if {$lastReply == 0 && $quietFlag == 0} { 
         panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret
      }

      if { $errFlag != 0 } {
         if { $quietFlag == 0 } {
            panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret
         }

         if { $noErrDialog == 0 } { seq_errDisplay }
          
         return "FAILURE"
        }
      
   }
   if { $quietFlag == 0 } {
      panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret
   }
   return $ret
}




#------------------------------------------------------------------------------
# PROCEDURE : panMsgSendCommand - 
#------------------------------------------------------------------------------

proc panMsgSendCommand { envName procName cmd sendOptions replyOptions args } {
   global gvar panWidget i_var panDEBUG

   if {[info exists panDEBUG] && [cequal $panDEBUG dbgDEV]} {
      puts "DEV. DEBUG > panSendCommand $envName $procName $cmd \"$flags\""
      puts "             args passed to the proc. = $args"
   }

   set parList  {}
   foreach item $args {

      if { ![info exists panWidget($item)]} { 
         append parList "$item "; continue 
      } 

      set varName [getWidget $item $i_var]

      # Get current value of the variable associated to the widget
      if {[info exists gvar($varName) ]} {append parList "$gvar($varName)  "}
        
   }; #end of foreach 

   if {[info exists panDEBUG] && [cequal $panDEBUG dbgDEV]} { 
       puts "DEV. DEBUG > sent parameters : $parList"
   }

   #
   # Send Command
   #

   if {[catch {eval seq_msgSendCommand $sendOptions $envName $procName $cmd $parList} ret] } {
      seq_errDisplay
      panDisplayCommand $cmd $procName "Failure sending the command"
      if {[cequal $panDEBUG dbgDEV]} { puts "DEV. DEBUG > Send Command Failed" }
      return "FAILURE"
   } else {
      set cmdId $ret 
      panDisplayCommand $cmd $procName "" $cmdId
   }
 
   #
   # Wait for Replies
   #
  
   set lastReply 0
   set idx  0

   while { $lastReply == 0 } { 
      
      set ret "" 
      set errFlag 0
      incr idx
 
      if {[catch {eval seq_msgRecvReply $replyOptions errFlag lastReply -${timeout} } ret]} {
         panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret
         seq_errDisplay
         return "FAILURE"
      }

      # Add current reply, because it's not the last one
      if {$lastReply == 0} { 
          panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret
      }

      if { $errFlag != 0 } {
         panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret
         seq_errDisplay
         return "FAILURE"
      }
   }
 
   panDisplayReply $cmd $cmdId $procName $errFlag $lastReply $idx $ret
   return $ret
}

