#*************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqDebug.tcl,v 2.77 2003/03/18 19:44:17 vltsccm Exp $" 
#
# who      when     what
# -------- -------- ------------------
# eallaert 20/02/96 created for cmm
#
# seqDebug.tcl --
#
# A couple of Tcl-debug procedures

#***************************************************************************
#   NAME 
#   dputs, breakpoint, showInfo - A couple of Tcl-debug procedures
#  
#   SYNOPSIS 
#   source seqDebug.tcl
#
#   dputs ?<args>?
#   breakpoint ?<level>?
#   showInfo ?<level>?
#
#   DESCRIPTION
#   dputs: prints out the <args> to stderr, but only if the global Tcl variable
#          "Debug" is set to the name of the procedure we want to debug, or to
#          "toplevel" if the toplevel is to be debugged. If no arguments are 
#          passed, a call to this procedure is equivalent to setting a 
#          breakpoint (provided "Debug" is set to the name of the procedure to
#          debug).
#  
#   breakpoint: set a breakpoint.  Four commands are interpreted directly by
#               the breakpoint procedure itself:
#	           + to move up the call stack
#	           - to move down the call stack
#	           ? to print information about the current stack frame
#                  C return from a breakpoint
#               All other commands are evaluated in the current stack frame
#               The optional argument <level> indicates how many more levels to
#               go up, in case several of these debug-procedures are nested 
#               (e.g. when dputs calls breakpoint)
#
#   showInfo: print useful information about the stack level (stack level, 
#             procedure name and arguments) to stderr.
#
#   Based on: "PracTcl Programming Tips", Linux Journal, October 1995
#   by Stephen Uhler
#
#   RETURN VALUES
#   as any Tcl procedure
#
#   CAUTION
#   The breakpoint proc seems to have some problems with namespaces, i.e
#   when in a child namespace it can complain that perfectly fine procedures
#   are not procedures. It looks like a bug in the "uplevel" command.
#------------------------------------------------------------------------ 

proc dputs {args} {
    global Debug
    if {![info exists Debug]} return

    set current [expr {[info level] -1}]
    set caller toplevel
    catch {set caller [lindex [info level $current] 0]}
    if {[string match $Debug $caller]} {
	if {[string match $args ""]} {
	    breakpoint 1
	} else {
	    puts stderr "$caller: $args"
	}
    }
}

proc breakpoint {{level 0}} {
    set max [expr {[info level] - 1 - $level}]
    set current $max
    showInfo $current
    while {1} {
  	puts -nonewline stderr "#$current: "
        gets stdin line
        while {![info complete $line]} {
	    puts -nonewline stderr "? "
            append line \n[gets stdin]
	}
	switch -- $line {
	    + {
		if {$current < $max} {
		    showInfo [incr current]
		}
	    }
	    - {
		if {$current > 0} {
		    showInfo [incr current -1]
		}
	    }
	    C {puts stderr "Resuming execution"; return}
	    ? {showInfo $current}
	    default {
		catch {uplevel #$current $line} result
		puts stderr $result
	    }
	}
    }
}

proc showInfo {current} {
    if {$current > 0} {
	set info [info level $current]
	set proc [lindex $info 0]
	puts stderr "$current: Procedure $proc {[info args $proc]}"
	set index 0
	foreach arg [info args $proc] {
	    puts stderr "\t$arg = [lindex $info [incr index]]"
	}
    } else {
	puts stderr "Top level"
    }
}

