#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: seqErrDisplay.tcl,v 2.77 2003/03/18 19:44:19 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    1996-09-19  created
# eallaert  1996-09-23  integrated into the sequencer
# eallaert  1996-12-17  bug fix - test of existence of .seqErrDisplay
# eallaert  1998-01-14  bug fix - see SPR 970507
# eallaert  1998-06-04  check for "wrong # args" versus "wrong option"
#

#************************************************************************
#   NAME
#   seq_errDisplay - Display the current error stack
#
#   SYNOPSIS
#   seq_errDisplay ?-internal|-reply? ?-suspend?
#
#   DESCRIPTION
#   This procedure shows the top error of the stack which was logged last.
#   It pops up a dialog containing the error description and the following
#   buttons :
#
#   OK         - Used to acknowledge that the error occurred 
#                and make the tcl script continue the execution
#
#   ERROR HELP - Shows the help file associated to the error.
#                
#   SHOW STACK - Displays the whole stack in a different window
#   
#   If the option "-suspend" is given, this command will not return control to
#   its caller until the error is acknowledged by the user.
#   For the -internal and -reply options, see e.g. the seq_errGetStackSize
#   manpage.
#
#   FILES
#   This procedure relies on the availability of the following Tcl-libraries:
#   uif, uifClass
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   The error stack which is shown is the one of the last CCS error, which was
#   either produced internally or received as a reply.  In particular if the
#   "-suspend" option is not given, this means that a later call to 
#   seq_errDisplay or any of its related procedures will reflect the "current"
#   last error.
#   Subsequent calls to seq_errDisplay, without destroying windows in
#   between, will simply update the information displayed in these error stack
#   windows.  This may not be the desired effect for panels with classes 
#   that can try simultaneously to display errors via the seq_errDisplay
#   command.
#
#   EXAMPLES
#
#   SEE ALSO
#
#
#------------------------------------------------------------------------
#

proc seq_errDisplay {args} {
    ::internal::seq_initDbg seq_errDisplay $args
    set suspend 0
    set option {}
    while {![lempty $args]} {
	set arg [lvarpop args]
	switch -glob -- $arg {
	    -suspend	{set suspend 1}
	    -internal	{set option -internal}
	    -reply	{set option -reply}
	    default	{set args [linsert $args 0 $arg]; break}
	}
    }
    # There should be no arguments left
    if {[llength $args] > 0} {
	if {[string equal -length 1 $arg -]} {
	    set msg "wrong option: \"$arg\" should be \"-internal\",\
		    \"-reply\" or \"-suspend\""
	} else {
	    set msg {wrong # args: should be "seq_errDisplay ?-internal|-reply? ?-suspend?"}
	}
	::internal::seq_returnDbg seq_errDisplay ERR $msg
        error $msg
    }

    namespace eval ::internal "variable option; set option [list $option]"
    namespace eval ::internal {
	variable option 
	global myProc myText myMemo mySev errDialogSTACK

        # This conditon should never happen ... so just in case
	if {[set size [eval seq_errGetStackSize $option]] == 0} {
	    set tmp "Error Structure is EMPTY"
	    tk_dialog .dialog "Dialog Box" $tmp info 0 OK
	    ::internal::seq_returnDbg seq_errDisplay OK {}
	    return
        } 

        set tmp [eval seq_errGetFromStack $option $size]

        set token [ctoken tmp "\{\}"] ;# Reject token
        set token [ctoken tmp "\{\}"] ;# Reject token

        # Token containing severity, proc.name ..
        set token  [ctoken tmp "\{\}"]

        set myProc [lindex $token 1]

        switch -- [lindex $token 4] {
	    "W" { set mySev "( Warning )" }
	    "F" { set mySev "( Fatal )"   }
	    "S" { set mySev "( Severe )"  }
	}

        set myMemo   [ctoken tmp " \{\}"]

        set token  [ctoken tmp "\{\}"]
        set myText [string trim $token " :"]

	# Only try to create the window if it is not yet displayed
	if {![winfo exists .seqErrDisplay]} {
	    set errDialogSTACK [::internal::errDialogPanel .seqErrDisplay]
	    $errDialogSTACK configure -command " \
		if {!\[winfo exists .seqErrStack\]} { \
		    internal::errStackPanel .seqErrStack \
		}; \
		internal::errGetProcStack $option; \
		tkwait window .seqErrStack; \
		catch {destroy .seqErrDisplay} \
	    "
	} else {
	    # if seq_errDisplay is called a 2nd time (e.g. via an "after"),
	    # .seqErrDisplay is already existing, but the dialogStack window
	    # isn't; errGetProcStack relies on the existence of that window
	    if {[winfo exists .seqErrStack]} {
		errGetProcStack $option
	    }
	}
    }
    if {$suspend} {tkwait window .seqErrDisplay}
    ::internal::seq_returnDbg seq_errDisplay OK {}
}

#
# ___oOo___
