#*************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqLib.tcl,v 2.77 2003/03/18 19:44:21 vltsccm Exp $" 
#
# who      when       what
# -------- ---------- ------------------
# eallaert 1997-07-16 created
# eallaert 1998-03-09 see SPRs 970547, 970548, 970549
# eallaert 1998-06-02 see SPR 980297; delete cmdId if timeout
# eallaert 2002-10-24 take MODROOT into account (SPR 20020574)
#
# seqLibs.tcl --
#
# Execute a number of commands to initialize the Sequencer

#***************************************************************************
#   NAME 
#   seq_relToAbsPath, seq_findFile, seq_waitTillActive, seq_waitTillDead,
#   seq_isoTime, seq_fitsDate, seq_timeOfDay, seq_isoTimeToClock - 
#       a collection of Sequencer procedures
#  
#   SYNOPSIS
#   seq_relToAbsPath <pathName>
#
#   seq_findFile <relPath>
#
#   seq_waitTillActive <env> <proc> <timeout> \
#                     ?<period> ?<runstring> ?<preferExec>??? 
#
#   seq_waitTillDead <env> <proc> <timeout> ?<period>? ?<exitCmd>?
#
#   seq_isoTime
#
#   seq_fitsDate
#
#   seq_timeOfDay
#
#   seq_isoTimeString2Clock <timeString>
#
#   DESCRIPTION
#   seq_relToAbsPath will return the absolute pathname corresponding to a
#      relative <pathName>; works for files as well as directories
#
#   seq_findFile will return the absolute path of <relPath>, after looking
#      for it in the following order:
#          1. the current working directory, i.e. ./<relPath>
#          2. ../<relPath>  
#          3. $MODROOT/<relPath>
#          4. $INTROOT/<relPath>
#          5. $VLTROOT/<relPath>
#      If <relPath> is not found, an error is returned ('<relPath> not found')
#
#   seq_waitTillActive will wait untill a certain process is active, i.e. until
#      it can receive CCS messages; this is tested by sending periodically PING
#      commands with the CCS message system
#      <env>    : environment where the process will run
#      <proc>   : name under which the process will register itself with CCS
#      <timeout>: how long to wait (in ms) before the seq_waitTillActive
#                 returns an error
#      <period> : period (in ms) for sending PING commands; default: 100 ms.
#                 This determines the resolution of <timeout>
#      <runstring>: If not empty, seq_waitTillActive will first of all schedule
#                 the process in the environment <env>.  If this <env> is the
#                 local environment, "exec" can be used for this scheduling - 
#                 see <preferExec> - thereby setting stdout of this new process
#                 the same as the script's stdout; otherwise, the msgSchedule(1)
#                 utility is used (and there will be no stdout messages).  In 
#                 both cases, the <runstring> is passed to the scheduling 
#                 utility.
#      <preferExec>: a boolean variable; if set to 1 and <env> is the local 
#                 environment, use exec(n) for the scheduling instead of 
#                 msgSchedule(1) (see above). Default: use msgSchedule(1).
#      There are no errors logged for the failed attempts to send a PING 
#      command, nor for the failing of seq_waitTillActive itself.
#      If the scheduling is done with exec(n), the Unix process-id will be
#      returned if successful.
#
#   seq_waitTillDead will wait untill a certain process is inactive, i.e. until
#      it no longer responds to CCS PING messages
#      <env>    : environment where the process is running
#      <proc>   : name under which the process is registered with CCS
#      <timeout>: how long to wait (in ms) before the seq_waitTillDead
#                 returns an error; remark that this timer starts *after* the
#                 reply to <exitCmd> came in.
#      <period> : period (in ms) for sending PING commands; default: 100 ms.
#                 This determins the resolution of <timeout>
#      <exitCmd>: If not empty, seq_waitTilDeadlActive will first of all send
#                 the command <exitCmd> to <proc>, using seq_msgSendCommand,
#                 before starting to test if it is still active. If this
#                 parameter is given, seq_waitTillDead wait for a reply on this
#                 command and wait for an additional 2 seconds before polling
#                 with the PING command.
#                 
#      If the <exitCmd> argument is not given, the script calling this command
#      should make sure (e.g. using a delay) that the process had a reasonable
#      chance to terminate, before calling seq_waitTillDead. Otherwise the PING
#      commands may still get on the queue of the terminating process while it
#      is exiting and therefore it would not reply. Although seq_waitTillDead
#      would finally timeout after <timeout> ms, and consider the process for
#      that reason dead, this "delay" in coming to that conclusion could 
#      surprise the user.
#
#      There are no errors logged for the failed attempt to send a PING command,
#      nor for the failing of seq_waitTillDead itself. If seq_waitTillDead
#      cannot communicate with <env>/<proc>, it is assumed that the process
#      is dead already (or that <env> is no longer active), and a OK-return 
#      will be taken.
#
#   seq_isoTime will return the current UTC time in ISO 8601 format (i.e. in the
#      format 'yyyy-mm-ddThh:mm:ss').
#
#   seq_fitsDate will return the current UTC date in the format 'yyyy-mm-dd'
#
#   seq_timeOfDay will return the current UTC time-of-the-day in the format
#      'hh:mm:ss'.
#
#   seq_isoTimeToClock will convert the UTC time <timeString> (ISO-format) to 
#      the internal clock value (integer, timezone adjusted) corresponding to
#      this UTC time.  This is useful if one wants to make elapsed time 
#      calculations starting from ISO-format time-strings, or to convert an 
#      ISO <timeString> to another non-ISO format (e.g. "clock format 
#      [seq_isoTimeToClock <timeString>]" will return the time in the format 
#      as the standard Unix "date" command).
#
#   RETURN VALUES
#   as any Tcl procedure; OK returns are always empty, except for 
#   seq_waitTillActive when "exec" is used; in this case, the Unix process-id
#   of the scheduled process is returned.
#
#   In case of an error, the following self-explanatory error messages can be 
#   returned:
#   seq_findFile:
#       '<relPath> not found in standard search path''
#   seq_waitTillActive:
#       'process <proc> (env <env>) not active after <timeout> ms'
#   seq_waitTillDead: 
#       'process <proc> (env <env>) still active after <timeout> ms'
#
#   CAUTIONS
#   If seq_waitTillDead cannot communicate with <env>/<proc>, it is assumed 
#   that the process is dead already, and a OK-return will be made.  This
#   behaviour is desired in most cases.  Beware however that this OK-return 
#   will also occur when e.g. <env> has a wrong value.
#
#   On top of that, timeouts for replies on accepted PING commands (both for
#   seq_waitTillActive and seq_waitTillDead) result in the automatic deletion
#   of the corresponding command-handle.
#
#   All time-related procedures require that the TimeZone is set to the proper
#   value, as time-values are converted to/from UTC based on this TZ info.
#
#   SEE ALSO
#   clock(n), msgSchedule(1)
#
#-------------------------------------------------------------------------- 
proc seq_relToAbsPath {path} {
    if {[file isfile $path]} {
	set dir [file dirname $path]
	set file /[file tail $path]
    } elseif {[file isdirectory $path]} {
	set dir $path
	set file {}
    } else {
	error "file $path does not exist"
    }
    set cwd [pwd]
    cd $dir
    set path [pwd]$file
    cd $cwd
    return $path
}
#-------------------------------------------------------------------------- 
proc seq_findFile {name} {
    global MODROOT INTROOT VLTROOT
    if {[file exists $name]} {
	switch -- [string index $name 0] / {
	    return [seq_relToAbsPath $name]
	} default {
	    return [seq_relToAbsPath [file join . $name]]
	}
    } elseif {[file exists [set f [file join .. $name]]]} {
	return [seq_relToAbsPath $f]
    } elseif {[file exists [set f [file join $MODROOT $name]]]} {
	return [seq_relToAbsPath $f]
    } elseif {[file exists [set f [file join $INTROOT $name]]]} {
	return [seq_relToAbsPath $f]
    } elseif {[file exists [set f [file join $VLTROOT $name]]]} {
	return [seq_relToAbsPath $f]
    } else {
	error "$name not found in standard search path (pwd|..|MODROOT|INTROOT|VLTROOT)"
    }
}
#--------------------------------------------------------------------------
if {[info exists seq_ccsType] && 
    ([string equal $seq_ccsType full] || [string equal $seq_ccsType light])} {
proc seq_waitTillActive {env proc timeout {period {}} {runstring {}} {exec 0}} {
    global seq_errLogging seq_ccsEnvName
    
    set pId 0
    set save $seq_errLogging
    set seq_errLogging off
    set remain $timeout
    switch -- {} $runstring {} default {
	if {[string equal $env $seq_ccsEnvName] && $exec} {
	    # Use "exec" in this case - it allows preservation of stdout
	    set pId [eval exec $runstring &]
	} else {
	    exec msgSchedule $env $runstring
	}
    }
    switch -- {} $period {set period 100}
    while {[catch {after $period; seq_msgSendCommand $env $proc PING} cmdId]} {
	if {[incr remain -$period] < 0} {
	    set seq_errLogging $save
	    error "process $proc (env $env) not active after $timeout ms"
	}
    }
    set seq_errLogging $save
    
    # The PING command got onto the message stack of proc, now check if it
    # replies properly (maybe proc will die before sending replies - SPR 980297)
    if {[catch {seq_msgRecvReply -all -$remain $cmdId en lr} msg]} {
	seq_deleteHandle $cmdId
	error "process $proc (env $env) accepted PING\
		but did not reply properly within $timeout ms ($msg)"
    }
    return $pId
}
}
#---------------------------------------------------------------------------
if {[info exists seq_ccsType] && 
    ([string equal $seq_ccsType full] || [string equal $seq_ccsType light])} {
proc seq_waitTillDead {env proc timeout {period {}} {exitCmd {}} } {
    global seq_errLogging
    
    set maxSleep 2000
    switch -- {} $period {set period 100}
    set save $seq_errLogging
    set seq_errLogging off
    switch -- {} $exitCmd {
	set remain $timeout
    } default {
	
	if {[catch {eval seq_msgSendCommand $env $proc $exitCmd} exitCmdId]} {
	    # An error at this point is interpreted as process dead already
	    set seq_errLogging $save
	    return
	}
	seq_msgRecvReply -all $exitCmdId en lr
	# We could have the situation where commands can still get into
	# the queue of the terminating process, but it will no longer reply - 
	# see SPR 970273. This is "avoided" by doing a sleep first, for
	# $maxSleep ms, before sending the PING commands
	set sleep [expr {($timeout < $maxSleep) ? $timeout : $maxSleep}]
	after $sleep
	set remain [expr {$timeout -$sleep}]
    }
    while {![catch {seq_msgSendCommand $env $proc PING} cmdId]} {
	if {[catch {seq_msgRecvReply -all -$remain $cmdId en lr} msg]} {
	    # Must be timeout-error; means PING got on msg-q of proc just 
	    # before exit.
	    set seq_errLogging $save
	    seq_deleteHandle $cmdId
	    if {$timeout >= $maxSleep} {
		return
	    } else {
		error "process $proc (env $env) did not reply to PING \
			within $timeout ms; increase timeout to > $maxSleep"
	    }
	} else {
	    if {[incr remain -$period] < 0} {
		set seq_errLogging $save
		error "process $proc (env $env) still active after $timeout ms"
	    }
	    after $period
	}
    }
    set seq_errLogging $save
    return 
}
}
#---------------------------------------------------------------------------
proc seq_isoTime {} {
    clock format [clock seconds] -format %Y-%m-%dT%H:%M:%S -gmt 1
}
#---------------------------------------------------------------------------
proc seq_fitsDate {} {
    clock format [clock seconds] -format %Y-%m-%d -gmt 1
}
#---------------------------------------------------------------------------
proc seq_timeOfDay {} {
    clock format [clock seconds] -format %H:%M:%S -gmt 1
}
#---------------------------------------------------------------------------
# From Tcl 8.3.3 on "clock scan" should directly recognize ISO format,
# but the clock value returned seems to be off by 7 hours ....
proc seq_isoTimeToClock {string} {
    scan $string "%4d-%2d-%2dT%8s" y m d t
    clock scan "${m}/${d}/${y} $t" -gmt 1
}
#---------------------------------------------------------------------------
