#***************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqRedirect.tcl,v 2.77 2003/03/18 19:44:23 vltsccm Exp $" 
# 
# who      when       what
# -------- ---------- ------------------
# eallaert 1999-11-19 created
#
# seqredirect.tcl --
# Redirect fileId to file

#***************************************************************************
#   NAME 
#   seq_redirect - redirect fileId to a disk file 
#  
#   SYNOPSIS 
#   seq_redirect <fileId> ?<fileName> <checkPeriod> <maxSize>?
#
#   DESCRIPTION
#   When all 4 arguments of this command are specified, seq_redirect 
#   will redirect <fileId> to <fileName>. This disk file is monitored each
#   <checkPeriod> seconds and whenever its size exceeds <maxSize> bytes only
#   the last part (10% of <maxSize>) will be kept.
#
#   If only the first argument is given, this logging to a file for <fileId>
#   is stopped. 
#
#   This command is particularly useful to redirect e.g. stderr to a log-file,
#   as debugging info is always written to stderr (see seq_debug array)   
#  
#   ENVIRONMENT 
#   no environment nor Tcl variables are read nor set 
# 
#   RETURN VALUES
#   
#   returns OK, with an empty result string if all is OK. 
#
#   returns ERROR with corresponding message in the result string if a file
#   access error occurred, or if attempting to stop redirection for a fileId
#   which is not being redirected.
#   
#   EXAMPLES 
#   (For clarity's sake, the example is for an interactive session with 
#   input typed by the user coming after the "seqWish>" prompt, and all
#   replies on a new line) 
#
#   ..... 
#   seqWish>seq_redirect stderr /tmp/myLogFile 3600 1000000
#   seqWish>
#   ..... 
#   seqWish>seq_redirect stderr
#   seqWish>
#
#   SEE ALSO 
#   "Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
# 
#------------------------------------------------------------------------ 
#
# Define first procedures used internally;
namespace eval ::internal {
    # The next proc re-directs origFileId to a file; use line buffering 
    # (i.e. flush # after each newline)
    proc redirectLogFile {origFileId} {
	variable redirect

	set fileName $redirect($origFileId,fileName)
	set redirect($origFileId,logFileId) [open $fileName a+]
	fconfigure $redirect($origFileId,logFileId) -buffering line
	dup $redirect($origFileId,logFileId) $origFileId
    }
    # The proc that will be called periodically to check the logfile size
    proc redirectCheckFileSize {origFileId} {
	variable redirect

	set size [fstat $redirect($origFileId,logFileId) size]
	set maxSize $redirect($origFileId,maxSize)
	if {$size > $maxSize} {
	    close $redirect($origFileId,logFileId)
	    # get the whole previous contents
	    set contents [read_file $redirect($origFileId,fileName)]
	    # write out only last 10%
	    write_file $redirect($origFileId,fileName) \
		    [string range $contents [expr {$size - $maxSize / 10}] end]
	    redirectLogFile $origFileId
	}
	after [expr {$redirect($origFileId,checkPeriod) * 1000}] \
		::internal::redirectCheckFileSize $origFileId
	return
    }
}

proc seq_redirect {fileId args} {
    set noa [llength $args]
    if {($noa != 0) && ($noa != 3)} {
	error {wrong # args: should be "seq_redirect <fileId> ?<fileName> <checkPeriod> <maxSize>"}
    }

    if {$noa == 0} {
	# means reset of redirection
	if {[info exists ::internal::redirect($fileId,dupFileId)]} {
	    # close the logFile
	    close [set ::internal::redirect($fileId,logFileId)]
	    # duplicate a copy of the original fileId into the orig fileId
	    dup [set ::internal::redirect($fileId,dupFileId)] $fileId
	    # close this duplicated fileId
	    close [set ::internal::redirect($fileId,dupFileId)]
	    # clean-up the array
	    unset ::internal::redirect($fileId,dupFileId)
	    unset ::internal::redirect($fileId,fileName)
	    unset ::internal::redirect($fileId,checkPeriod)
	    unset ::internal::redirect($fileId,maxSize)
	    unset ::internal::redirect($fileId,logFileId)
	    return
	} else {
	    error "wrong fileId: \"$fileId\" has not been redirected"
	}
    }
	    
    # Getting here means we want to set-up this redirection
    lassign $args fileName checkPeriod maxSize
    # keep some of these parameters for later use
    set ::internal::redirect($fileId,dupFileId) [dup $fileId]
    set ::internal::redirect($fileId,fileName) $fileName
    set ::internal::redirect($fileId,checkPeriod) $checkPeriod
    set ::internal::redirect($fileId,maxSize) $maxSize

    # Set-up the logfile
    ::internal::redirectLogFile $fileId
    
    # Start the log-file checking (which re-activates itself)
    ::internal::redirectCheckFileSize $fileId

    return
}

