#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclBufToPan.tcl,v 1.17 2002/04/04 11:04:41 vltsccm Exp $"
#
# slxtclBufToPan.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/05/96  created
#
#************************************************************************
#   NAME
#   slxtclBufToPan - Load contents of a setup buffer in a GUI panel
# 
#   SYNOPSIS
#   slxtclBufToPan <buf>
# 
#   DESCRIPTION
#   Tcl procedure, used within GUI panels built with the VLT panel editor, 
#   to assign to variables associated to widgets the values of setup
#   keywords contained in the setup buffer <buf>.
#   The format of this buffer is the same as returned by oslx after 
#   the command OSETUP -getSetupBuf
#
#   This procedure is intended to be used in cases where a list of keywords
#   and values has been extracted from a setup file, loaded in a oslx setup
#   buffer, and then manipulated using functionality provided by oslx. 
#   For those cases where the list has to be retrieved from a setup file as it
#   is (no manipulation needed), it is recommended to use slxtclLoadSetup.
#
#   The value of all variables associated to a widget matching a keyword
#   in the setup buffer are replaced with the value of that keyword (see
#   slxtclLoadSetup for examples and more information).
#
#   The list of variables whose value has been updated is returned to the
#   calling process.  
#
#   It belongs to the library libslxtcl.tcl; therefore, in order to be able
#   to use it, one has to register the panel edited with the Panel Editor
#   to this library.
#
#   RETURN VALUES
#   list of panel variables whose value has been updated
#
#   EXAMPLES
#   # 1
#   # retrieve the contents of setup file myRef in a oslx buffer
#   seq_oslxCmd OSETUP -load slxtcl myRef
#   # add two keywords
#   set newKeys "DET.WIN1.BINX 8 DET.WIN1.BINY 8" 
#   seq_oslxCmd OSETUP -storekeyword slxtcl $newKeys
#   # retrieve the contents of the oslx buffer
#   set buf [seq_oslxCmd OSETUP -getSetupBuf slxtcl]
#   # load the contents of the buffer in the panel
#   set varPan [slxtclBufToPan $buf]
#   # print the list of variables whose value has been updated
#   foreach name $varPan { puts $name }
#
#   SEE ALSO
#   slxtclLoadSetup
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#
proc slxtclBufToPan { oslxBuf } {   

   # Declare global variables

   global cv 
   global gvar slxFits slxFitsValue
   global slxFitsVarMod

   set slxFits {}
   set slxFitsValue {}
   #
   # get setup keywords from buffer
   #
   set keyL [split $oslxBuf ","]; # split string in a list
   set keyL [lreplace $keyL 0 0]; # remove first element (number of keywords)
   foreach key $keyL {;           # for each pair name + value ...
     set keyNV [split $key];      # split name and value
     lappend slxFits [lindex $keyNV 0];  # add name to list of names
     lappend slxFitsValue [lindex $keyNV 1]; # add value to list of values
   }

   #
   # extract all setup variables from panel
   #
   set varName [slxtclSetupList]

   #
   # assign value of setup keywords to panel variables
   #
   set leng [llength $slxFits]
#foreach name $slxFits { puts $name }
#foreach name $slxFitsValue { puts $name }
#foreach name $slxFitsVarMod { puts $name }
   set varPan {}
   set count 0
   while {$count < $leng} {
       set name [lrange $slxFits $count $count]
       set index [lsearch $slxFitsVarMod "*$name"]
       if {$index >= 0} {;     # setup keyword found in panel gvar
         set name [lrange $varName $index $index]
         set value [lrange $slxFitsValue $count $count]
         set value [string trim $value "\{\}\""]
         set gvar($name) $value
         lappend varPan $name
       }
       incr count
   };  # end while

   #
   # Refresh input widgets
   #
   panUpdateCanvas $cv

   return $varPan ; # return the list of modified variables
}


#
# ___oOo___


