#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclFilterSetup.tcl,v 1.17 2002/04/04 11:04:42 vltsccm Exp $"
#
# slxtclFilterSetup.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  11/12/95  created
# alongino  13/12/95  multiple class instances added
# alongino  14/05/96  prefix changed slxTcl --> slxtcl
#
#
#************************************************************************
#   NAME
#   slxtclFilterSetup - Filter setup keywords from panel variables
# 
#   SYNOPSIS
#   slxtclFilterSetup <mode> <pattern> [<from>] [<to>]
# 
#   DESCRIPTION
#   Tcl procedure used to define a filter for the setup variables defined
#   in a panel.
# 
#   Typically called from within a panel to exclude from the setup keywords 
#   one or more variables defined in it, or modify the associated keyword. 
#   For example, a panel defining the complete setup for an instrument having
#   two detectors will normally instanciate (import) twice a detector setup 
#   panel class: in this case one has to modify the keywords associated
#   at least to one of the two detectors, in order to be able to distinguish
#   them from those of the other detector. Furthermore, for those observing 
#   modes requiring one detector only, all variables associated to the other 
#   detector have to be excluded from the setup file associated to that 
#   observing mode.
# 
#   The parameter <mode> determines the mode how the filter is used:
#     a : add an entry in the list of filters
#     r : remove an entry in the list of filters
#     c : add an entry in the list of keywords to be modified
#     u : remove an entry in the list of keywords to be modified
# 
#   The parameter <pattern> indicates the pattern against which all
#   variables associated to widgets must be checked. Wildcard options, as
#   accepted by Tcl for glob-style pattern matching, can be used. 
#   The special value "all" combined to the value "r" or "u" for <mode> 
#   removes all existing filter entries.
# 
#   The parameter <from> indicates the sub-string in the keyword to be replaced
#   (meaningful only if <mode> is "c")
# 
#   The parameter <to> indicates the sub-string in the keyword replacing <from>
#   (meaningful only if <mode> is "c")
# 
#   Note that the filtering applies to all operations done from the moment
#   when the filtering is requested by calling this function (<mode> "a" or 
#   "c") until when the filtering is explicitely removed (<mode> "r" or "u"). 
#   Do not forget to remove filters (e.g. "r" "all") whenever you do not want 
#   them to take effect in the next operations.
#
#   It belongs to the library libslxtcl.tcl; therefore, in order to be able
#   to use it, one has to register the panel edited with the Panel Editor
#   to this library.
#
#   FILES
#   none
#
#   ENVIRONMENT
#
#   GLOBALS
#
#   RETURN VALUES
#   None
#
#   CAUTIONS
#
#   EXAMPLES
#   # 1
#   # Exclude for the creation of the setup file all variables whose
#   # associated setup keyword begins with DET.EXP.
#   slxtclFilterSetup a *DET.EXP.*
#   slxtclSaveSetup $file
#   
#   # 2
#   # remove all filter entries defined before (all keywords will be written 
#   # in the setup file)
#   slxtclFilterSetup r all
#   slxtclSaveSetup $file
#
#   # 3
#   # remove all filter entries defined before and define a new filter
#   # excluding all INS keywords
#   slxtclFilterSetup r all
#   slxtclFilterSetup a *INS*
#   slxtclSaveSetup $file
#
#   # 4
#   # For all those variables beginning with "blueArm", the sub-string
#   # ".DET" has to be replaced with "0.DET1"
#   # For example the keyword associated to the variable 
#   # blueArm_setup.4.DET.EXP.TYPE will be DET1.EXP.TYPE and the index 40
#   # instead of 4
#   slxtclFilterSetup c blueArm* .DET 0.DET1
#   slxtclSaveSetup $file
#
#   # 5
#   # leave unchanged the keywords associated to all variables whose name 
#   # begins with "blueArm"
#   slxtclFilterSetup u blueArm*
#   slxtclSaveSetup $file
#
#   SEE ALSO
#   slxtclSaveSetup, slxtclPanToBuf
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc slxtclFilterSetup { mode filter {from ""} {to ""}} {

 # Declare global variables

   global slxFitsFilter
   global slxFitsFilterMod slxFitsFrom slxFitsTo

   set modify 0
   set myList slxFitsFilter
   if {($mode == "c") || ($mode == "u")} {; # modify keyword
       set modify 1
       set myList slxFitsFilterMod
   }

   # if filter list does not exist, create and initialize it
   if {![info exists $myList]} {
       set $myList {}
       if {$modify} {
           set slxFitsFrom {}
           set slxFitsTo {}
       }
   }

   if {$mode == "a"} {; # add one filter entry
       set index [lsearch -exact $slxFitsFilter $filter]
       if {$index == -1} {; # new filter
           lappend slxFitsFilter $filter
       }
   } elseif {$mode == "r"} {; # remove one filter entry
       set index [lsearch -exact $slxFitsFilter $filter]
       if {$filter == "all"} {; # remove all filters
           set slxFitsFilter {}       
       } elseif {$index >=0 } {; # old filter; remove it
           set slxFitsFilter [lreplace $slxFitsFilter $index $index]
       }
   } elseif {$mode == "c"} {; # add one entry in list for replacement
       set index [lsearch -exact $slxFitsFilterMod $filter]
       if {$index == -1} {; # new filter
           lappend slxFitsFilterMod $filter
           lappend slxFitsFrom $from
           lappend slxFitsTo $to
       }
   } elseif {$mode == "u"} {; # remove one filter entry
       set index [lsearch -exact $slxFitsFilterMod $filter]
       if {$filter == "all"} {; # remove all filters
           set slxFitsFilterMod {}       
       } elseif {$index >=0 } {; # old filter; remove it
           set slxFitsFilterMod [lreplace $slxFitsFilter $index $index]
           set slxFitsFrom [lreplace $slxFitsFrom $index $index]
           set slxFitsTo [lreplace $slxFitsTo $index $index]
       }
   }
#if {$modify} {; # add one filter entry
#  set fitsFilter [join $slxFitsFilterMod "\n"]
#  set fitsFrom [join $slxFitsFrom "\n"]
#  set fitsTo [join $slxFitsTo "\n"]
#  puts $fitsFilter 
#  puts $fitsFrom 
#  puts $fitsTo
#} else {
#  set fitsFilter [join $slxFitsFilter "\n"]
#  puts $fitsFilter
#}
return
}


#
# ___oOo___


