#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclLoadSetup.tcl,v 1.17 2002/04/04 11:04:42 vltsccm Exp $"
#
# slxtclLoadSetup.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  30/07/95  created
# alongino  25/09/95  added call to panUpdateCanvas for slide widgets
# alongino  13/05/95  moved to slxtcl module
#                     setup files handling done through oslxServer
# alongino  14/05/96  prefix changed slxTcl --> slxtcl
#                     usage of oslxServer incorporated
# alongino  09/07/96  added check on slxtclReadSetup possible failure
#
#
#************************************************************************
#   NAME
#   slxtclLoadSetup - Load contents of a setup file in a GUI panel
# 
#   SYNOPSIS
#   slxtclLoadSetup <file> [<inst>]
# 
#   DESCRIPTION
#   Tcl procedure, used within GUI panels built with the VLT panel editor, 
#   to assign to variables associated to widgets the values of setup
#   keywords contained in the setup file <file>.
# 
#   The value of all variables associated to a widget matching a keyword
#   in the setup file (except prefix setup.<i>. See slxtclSaveSetup) are 
#   replaced with the value of that keyword read from the setup file.
#
#   Example: if the setup file contains the following line:
#   
#   DET.WIN1.BINX        8;         # Binning factor in X
#   
#   and in the panel a variable associated to a widget is called
#   setup.1.DET.WIN1.BINX, then its value is set to 8
#  
#   Keywords not matching any variable name within the panel are ignored.   
#
#   The list of variables whose value has been updated is returned to the
#   calling process.  
#
#   This procedure uses funtionality provided by oslx to search for
#   the setup file specified by <file>, to read and format its contents. 
#   It assumes that the data dictionary containing the description of the 
#   keywords handled has already been loaded (see slxtclDictionary).
#   The parameter <inst> (default slxtcl) specifies the instance of setup 
#   buffer oslx has to use.
#
#   It belongs to the library libslxtcl.tcl; therefore, in order to be able
#   to use it, one has to register the panel edited with the Panel Editor
#   to this library.
#
#   FILES
#   <file>    input setup file
#
#   ENVIRONMENT
#   INS_ROOT      see [2]
#   INS_USER      see [2]
#
#   RETURN VALUES
#   list of panel variables whose value has been updated
#   "FAILURE" if an error occurred.
#
#   CAUTIONS
#   1) This procedure does NOT check if the appropriate data dictionary has 
#      been loaded through slxtclDictionary. The application has to make 
#      sure that this is true.
#
#   EXAMPLES
#   # 1
#   # read the contents of setup file myRef and load keyword values in the  
#   # panel using the setup buffer slxtcl of oslx
#   set dictionaries {TCS ICS CCDDCS}
#   slxtclDictionary $dictionaries
#   slxtclLoadSetup myRef
#
#   # 2
#   # read the contents of setup file myRef and load keyword values in the  
#   # panel using the setup buffer myInst of oslx
#   slxtclDictionary $dictionaries
#   slxtclLoadSetup myRef myInst
#
#   # 3
#   # read the contents of setup file myRef, load keyword values in the  
#   # panel using the setup buffer slxtcl of oslx and print the list of 
#   # variables whose value has been updated
#   slxtclDictionary $dictionaries
#   set varPan [slxtclLoadSetup myRef]
#   foreach name $varPan { puts $name }
#
#   SEE ALSO
#   slxtclBufToPan
#   slxtclSaveSetup, slxtclDictionary
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#

proc slxtclLoadSetup { file {inst slxtcl} } {

   set varPan {}
   #
   # Store contents of setup file in oslx setup buffer
   #
   set oslxBuf [slxtclReadSetup $file $inst]

   #
   # Update contents of widgets according to contents of oslx setup buffer
   #
   if {$oslxBuf != "FAILURE"} {
     set varPan [slxtclBufToPan $oslxBuf]
   }

#foreach name $varPan { puts $name }
   return $varPan ; # return the list of modified variables
}

#
# ___oOo___


