#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclSaveSetup.tcl,v 1.17 2002/04/04 11:04:43 vltsccm Exp $"
#
# slxtclSaveSetup.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  30/07/95  created
# alongino  14/05/96  prefix changed slxTcl --> slxtcl
#                     usage of oslxServer incorporated
# alongino  09/07/96  added check on slxtclWriteSetup possible failure
#                     result returned
# alongino  02/05/97  interface to oslx changed
#
#
#************************************************************************
#   NAME
#   slxtclSaveSetup - Save contents of GUI panel in a setup file
# 
#   SYNOPSIS
#   slxtclSaveSetup <file> [<type>] [<inst>]
# 
#   DESCRIPTION
#   Tcl procedure, used within GUI panels built with the VLT panel editor, 
#   to save values of variables associated to widgets in the setup file <file>.
#
#   When editing the configuration of a widget with the panel editor, the 
#   field "Variable" must contain a name with the following syntax:
#
#   setup.<i>.<short FITS>
#
#   whereby 
#     <i>          is the order of appearance in the setup file to be generated
#     <short FITS> is the keyword name in short FITS (see [1])
#
#   Examples: 
#             setup.0.DET.EXP.TYPE
#             setup.1.DET.WIN1.BINX
#
#   Widgets whose associated variable does not follow the syntax described
#   above are just ignored by this procedure. This allows to mix within a
#   panel information needed to be saved in a setup file with other just
#   needed for other purposes.
#   
#   In some cases one might temporarily want to ignore also some variables
#   following the syntax described above. In this case, it is possible to
#   define filters (see slxtclFilterSetup).
#
#   In more complex cases, the same widget class is instanciated twice within
#   the same panel. In this case, one may want to change the name of the
#   keywords associated to one or more instances. Also for this case, see
#   slxtclFilterSetup
#
#   This procedure uses funtionality provided by oslx to search for
#   keywords and formats in a data dictionary and save formatted keywords
#   and values in the setup file specified by <file>; the type of setup file
#   (see [2]) is specified by <type> (default "Reference Setup"). 
#   It assumes that the data dictionary containing the description os the 
#   keywords handled has already been loaded (see slxtclDictionary).
#   The parameter <inst> (default slxtcl) specifies the instance of setup 
#   buffer that oslx has to use.
#
#   It belongs to the library libslxtcl.tcl; therefore, in order to be able
#   to use it, one has to register the panel edited with the Panel Editor
#   to this library.
#
#   FILES
#   <file>          setup file generated
#
#   ENVIRONMENT
#   INS_ROOT      see [2]
#   INS_USER      see [2]
#
#   RETURN VALUES
#   "FAILURE" if an error occurred.
#   "OK" if successful
#
#   CAUTIONS
#   1) An already existing setup file is overwritten
#   2) This procedure does NOT check if the appropriate data dictionary has 
#      been loaded through slxtclDictionary. The application has to make 
#      sure that this is true.
#
#   EXAMPLES
#   # 1
#   # save keywords and values from a panel in the reference setup file myRef
#   # using the setup buffer slxtcl of oslx
#   set dictionaries {TCS ICS CCDDCS}
#   slxtclDictionary $dictionaries
#   slxtclSaveSetup myRef
#
#   # 2
#   # save keywords and values from a panel in the reference setup file myRef
#   # using the setup buffer myInst of oslx
#   slxtclDictionary $dictionaries myInst
#   slxtclSaveSetup myRef "Reference Setup" myInst
#
#   # 3
#   # save only DETector keywords and values from a panel in the detector 
#   # setup file myDet using the setup buffer slxtcl of oslx
#   slxtclSaveSetup myDet "Detector Setup"
#
#   # 4
#   # save only DETector keywords and values from a panel in the reference 
#   # setup file myRef using the setup buffer slxtcl of oslx
#   slxtclFilterSetup a *GEN*
#   slxtclFilterSetup a *INS*
#   slxtclFilterSetup a *TEL*
#   slxtclSaveSetup myRef "Reference Setup"
#
#   SEE ALSO
#   slxtclFilterSetup, slxtclPanToBuf
#   slxtclLoadSetup, slxtclDictionary
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#
proc slxtclSaveSetup { file {type "Reference Setup"} {inst slxtcl} } {

   #
   # Build oslx setup buffer from contents of widgets
   #
   set oslxBuf [slxtclPanToBuf]

   #
   # Write oslx setup buffer into setup file
   #
   set status [slxtclWriteSetup $oslxBuf $file $type $inst]

   #
   # Return result
   #
   if {$status != "FAILURE"} {set status "OK"}
   return $status
}


#
# ___oOo___


