#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxtclSetupList.tcl,v 1.17 2002/04/04 11:04:43 vltsccm Exp $"
#
# slxtclSetupList.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  08/12/95  created
# alongino  11/12/95  filtering added
# alongino  13/12/95  multiple class instances added
# alongino  15/12/95  multiple appearence of setup string in variable name
# alongino  14/05/96  prefix changed slxTcl --> slxtcl
#
#************************************************************************
#   NAME
#   slxtclSetupList - Get from a GUI panels the list of variables for setup
# 
#   SYNOPSIS
#   slxtclSetupList [f/nf]
# 
#   DESCRIPTION
#   Tcl procedure, called by slxtclPanToBuf and slxtclBufToPan, 
#   to get the list of all variables in a GUI panel built with the panel 
#   editor, which follow the syntax for setup variables (see slxtclSaveSetup)
#   and can be associated to one line in a setup file.
#   The list of variables is returned to the calling process.
#   If optional "f" is specified, variables to be filtered out are excluded
#   from the returned list. Default is "nf" (no filter).
#
#   RETURN VALUES
#   list of variables for setup operations
#
#   EXAMPLES
#   # 1
#   # Extract lists of all variables for setup operations
#   set varName [slxtclGetSetupList]
#
#   # 2
#   # Extract lists of variables for setup operations filtered out
#   set varName [slxtclGetSetupList f]
#
#   SEE ALSO
#   slxtclPanToBuf, slxtclBufToPan
#   [1] VLT-MAN-ESO-17240-0726 VLT INS Common Software slx  - User Manual 
#   [2] VLT-MAN-ESO-17240-0853 VLT INS Common Software oslx - User Manual
#
#------------------------------------------------------------------------
#
proc slxtclSetupList { {useFilter "nf"} } {

 # Declare global variables

   global gvar 
   global slxFitsFilter
   global slxFitsFilterMod slxFitsFrom slxFitsTo
   global slxFitsIndex slxFitsTmp
   global slxFitsVarMod

   set prefix setup
   set format "$prefix.%d.%s"
   set varName {}
   set slxFitsVarMod {}
   set slxFitsIndex {}
   set slxFitsTmp {}
   if {![info exists slxFitsFilter]} {
       set useFilter nf
   }
   foreach name [array names gvar] {
#puts "$name $gvar($name)" 
       #
       # check in the list of replacements if this variable matches
       #
       set nameMod $name
       if {[info exists slxFitsFilterMod]} {
           set leng [llength $slxFitsFilterMod]
           set count 0 
           while {$count < $leng} {
               set filter [lrange $slxFitsFilterMod $count $count]
	       set replace [string match $filter $name]
               if {$replace} {
                   set from [lrange $slxFitsFrom $count $count]
                   set to [lrange $slxFitsTo $count $count]
                   regsub $from $name $to nameMod
                   break
               } else {
                   incr count
               }
           }
       }

       #
       # extract all setup variables
       # setup variable must have name setup.<index>.<name>
       # variables for imported classes are <widget name>_<variable>
       # AL951215 Note that when instanciating classes, the name of the 
       # instance may contain the same prefix. Therefore one has to read 
       # backwards
       #set prefLoc [string first $prefix $nameMod]
       set prefLoc [string last $prefix $nameMod]
       if {$prefLoc >= 0} {
           set locName [string range $nameMod $prefLoc end]
           set i [scan $locName $format index short]
#puts "$name $nameMod $locName $i"
           if {$i == 2} {
               #
               # currently the panel editor keeps internally both variable
               # name in the class definition and the corresponding in the
               # instantiation. The former is not used and must be thrown
               # away. Example: in class "emperor" a widget is associated to
               # variable "power". When creating instance "nero", the variable
               # nero_power is created; "power" is also imported in the
               # panel, but not used.
               #
               set i [lsearch $slxFitsTmp $short]
               if {$i >=0 } {;    # same keyword already in another variable
	           if {$prefLoc > 0} {; # instance of class
                     set varName [lreplace $varName $i $i $name]
                     set slxFitsVarMod [lreplace $slxFitsVarMod $i $i $nameMod]
                     set slxFitsIndex [lreplace $slxFitsIndex $i $i $index]
                   }
               } else {
                   #
                   # Add variable to the list
                   #
                   lappend varName $name
                   lappend slxFitsVarMod $nameMod
                   lappend slxFitsTmp $short
                   lappend slxFitsIndex $index
               }
           }
       }
   }
   #
   # Remove from the list variables to be filtered-out
   #
   if {$useFilter == "f"} {
       set leng [llength $varName]
       set fLeng [llength $slxFitsFilter]
       set count 0 
       while {$count < $leng} {
           set name [lrange $varName $count $count]
           set fCount 0 
           while {$fCount < $fLeng} {
               set fName [lrange $slxFitsFilter $fCount $fCount]
               if {[string match $fName $name]} {; # filtered out
                   set varName [lreplace $varName $count $count]
                   set slxFitsVarMod [lreplace $slxFitsVarMod $count $count]
                   set slxFitsTmp [lreplace $slxFitsTmp $count $count]
                   set slxFitsIndex [lreplace $slxFitsIndex $count $count]
                   incr count -1
                   incr leng -1
                   break
	       }
               incr fCount
           }
           incr count
       }
   }
#set string [join $varName "\n"]
#puts $string
#set string [join $slxFitsVarMod "\n"]
#puts $string
#set string [join $slxFitsTmp "\n"]
#puts $string
#set string [join $slxFitsIndex "\n"]
#puts $string
   return $varName
}


#
# ___oOo___
