#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: findVLTFile.tcl,v 3.36 2003/01/20 10:33:41 vltsccm Exp $" 
#
# findVLTFile.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# G.Filippi 23/06/94  created
# M.COMIN   14/05/95  Add second parameter
# M.COMIN   16/05/95  Improved version, without exec ...
# M.COMIN   29/11/95  Add search to VLTDATA
# mcomin    11/01/96  Update man page

#************************************************************************
#   NAME
#     findVLTFile - locate a file in MODROOT, INTROOT, VLTDATA or VLTROOT
#   SYNOPSIS
#     findVLTFile  <file> ?confirm?
#   DESCRIPTION
#
#     This procedure searches a file in the VLT file structure.
#
#     file    <IN> -   Name of the file in the format <relPath>/<fileName>.
#
#                      <relPath> is a relative path to the file <fileName>,
#                      for example "app-defaults/XerrEditor"
#
#     confirm <IN> -   <yes/no>  : flag specifying if the a dialog should 
#                      appear to the user in case the file does not exists.
#                      Default value "yes" 
#
#     The serach path is the following :
#
#     - Current Directory : using the basename only -e.g.- XerrEditor
#     - Current Directory : using the basename + relative path.
#                           -e.g.-  app-defaults/XerrEditor
#     - Parent  Directory   -e.g.-  ../app-defaults/XerrEditor
#     - Integration Area    -e.g.-  $INTROOT/app-defaults/XerrEditor
#     - VLTDATA Area        -e.g.-  $VLTDATA/app-defaults/XerrEditor
#     - VLTROOT Area        -e.g.-  $VLTROOT/app-defaults/XerrEditor
#
#     The full pathname of the first occurrence is reported as return value.
#
#   FILES
#
#   ENVIRONMENT
#     VLTROOT    Current VLT root area
#     VLTDATA    Current VLT Data area
#     INTROOT    Current integration area
#
#   RETURN VALUES
#
#     The full pathname of the first occurrence of <file> 
#
#   CAUTIONS
#
#   EXAMPLES
#
#   This procedure should be used instead of the file name:
#
#      button .... -bitmap [findVLTFile  bitmaps/xyz.xbm] ....
#      source [findVLTFile  bin/bindings.tcl]
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

proc findVLTFile { file { confirm "yes" } } {
    global env
     
    #
    # Take the only file name and check in the current directory
    #
    set myFile  [file tail $file] 
    set myDir   [pwd]

    if {[file exist $myFile]} { return  ${myDir}/${myFile} }

    #
    # Does the file exist in the current sub-directory?
    #
    
    set fileName ${myDir}/$file
    if {[file exist $fileName]} { return  $fileName }
  
    #
    # Does the file exist in the current module?
    # (the current module is assumed to be the parent directory 
    #  of the current pwd)
    
    set fileName ../$file
    if {[file exist $fileName]} { return  $fileName }
    #
    # Does the file exist in the current integration area, if any?
    #
    if {[info exists env(INTROOT)]} {
        set fileName $env(INTROOT)/$file
        if {[file exist $fileName]} { return  $fileName }
    }
    #
    # Does the file exist in the current VLT data area, if any?
    #
    if {[info exists env(VLTDATA)]} {
        set fileName $env(VLTDATA)/$file
        if {[file exist $fileName]} { return  $fileName }
    }
    #
    # Does the file exist in the current VLT root area, if any?
    #
    if {[info exists env(VLTROOT)]} {
        set fileName $env(VLTROOT)/$file
        if {[file exist $fileName]} { return  $fileName }
    }

    #
    # File does not exist.
    #
    if { $confirm == "yes" } {
       set tmp "File  $myFile  NOT FOUND"
       tk_dialog .dialog "Dialog Box" $tmp info 0 OK
    }

    return ""
}

# ___oOo___ 

