#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifArrowsButton.tcl,v 3.36 2003/01/20 10:34:03 vltsccm Exp $" 
#
# uifFrame.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    19/12/97   created 
#
#************************************************************************
#   NAME
#     UifArrowsButton - Widget used to increment/decrement values by
#                       clicking on up/down arrows
#   SYNOPSIS
#     UifArrowsButton  <widget-name>
#   DESCRIPTION
#
#     UifArrowsButton is a widget formed by two buttons showing two
#     arrows (up/down arrows).  The widget can be linked to a variable 
#     and used to increment/decrement the value in steps.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     upArrow   - button with the 'up' arrow
#     downArrow - button with the 'down' arrow
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Inherits options from UifWidget classe and supports 
#     the followings :
#
#     variable  <string>   Name of the variable to control
#     incrStep  <integer>  Value used to increase the variable : default +1
#     decrStep  <integer>  Value used to decrease the variable : default -1
#     script    <string>   Name of the procedure to invoke when either
#                          up/down buttons are clicked.
#                          The value of the variable is modified first and then
#                          the procedure is executed.
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifArrowsButton .w  -variable gvar(myVariable) -incrStep +2
#                         -decrStep -3 -script processStep
#
#     Re-configure the widget
#        .w  configure -decrStep -5
#
#     Other operations
#        set script [.w cget -script]    Get a widget's resource
#
#   SEE ALSO
#     UifWidget
#
#   BUGS
#
#------------------------------------------------------------------------
#

itk::usual UifArrowsButton { }

class UifArrowsButton {
    inherit  UifInteractiveWidget

    constructor {args} {}

    itk_option define -variable variable Variable    ""
    itk_option define -incrStep  incrStep  IncrStep  1  
    itk_option define -decrStep  decrStep  DecrStep  -1
    itk_option define -script    script    Script    ""

    public method step { what }
}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifArrowsButton::constructor {args} {

   set b1 @[findVLTFile bitmaps/uifUpArrow.xbm]  
   set b2 @[findVLTFile bitmaps/uifDownArrow.xbm]

   itk_component add upArrow {
      button $itk_interior.upArrow -bd 2 -highlightthickness 0
   } { 
      keep -background -foreground -state
      keep -activeforeground -activebackground -disabledforeground
   }

   itk_component add downArrow {
      button $itk_interior.downArrow -bd 2 -highlightthickness 0
   } { 
      keep -background -foreground -state 
      keep -activeforeground -activebackground -disabledforeground
   }

   pack $itk_component(upArrow) $itk_component(downArrow) -pady 1 -padx 2

   eval itk_initialize
   set initialized 1

   # Set default configuration

   $itk_component(upArrow)   configure -command "$this step up" -bitmap $b1
   $itk_component(downArrow) configure -command "$this step down" -bitmap $b2
   eval configure $args
}

#---------------------------------------------------------------
# OPTION : script - Tcl procedure invoked when the button is pushed
#                   The associated variable is increased/decreased before.
#---------------------------------------------------------------

configbody UifArrowsButton::script {
   
   if {! $initialized } { return }

   if {[cequal $itk_option(-script) ""]} {
      $itk_component(upArrow)   configure -command "$this step up"
      $itk_component(downArrow) configure -command "$this step down"
   } else {
      $itk_component(upArrow)   configure -command "$this step up ; $itk_option(-script)"
      $itk_component(downArrow) configure -command "$this step down ; $itk_option(-script)"
   }
}

#---------------------------------------------------------------
# METHOD : step - 
#---------------------------------------------------------------

body UifArrowsButton::step { what } {
   
    if { ![cequal $itk_option(-variable) ""] &&
          [uplevel #0 info exists $itk_option(-variable)] } {

      if {[cequal $what up]} {
         uplevel #0 incr $itk_option(-variable) $itk_option(-incrStep)
      } else {
         uplevel #0 incr $itk_option(-variable) $itk_option(-decrStep)
      }
   }
}
