#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifBarChart.tcl,v 3.36 2003/01/20 10:33:43 vltsccm Exp $" 
#
# uifBarChart.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.Comin    07/11/94   created
# M.COMIN    27/02/95   Add public variable 'barColor'
# M.COMIN    06/03/95   Fixed bug with default min,max values
# M.COMIN    15/03/95   Man page updated
# M.COMIN    20/07/95   Added #0 to upvar in CmdTrace (bug fixing)
# mcomin     05/10/95   Add public variables for the event mechanism
# mcomin     27/11/95   Fixed bug with tickSize : it must not be declared
#                       global variable.
#
# mcomin     6/03/96    Porting to Tk4.0 and itcl2.0
#            13/05/96   Update man page
#            06/03/97   Porting to tk 4.2
#            26/04/98   Increase size of bar : in paranal it was not visible.
#                       Add default values for options
#            10/08/98   Size is now expressed in mm instead of pixels
#            18/06/99   Option -mapped changed into -hide (tcl 8)
#                       Adjust widget size
#
#************************************************************************
#   NAME
#     UifBarChart - Class for VLT barChart widget
#   SYNOPSIS
#     UifBarChart  <widget-name>
#   DESCRIPTION
#     UifBarChart defines the class for the VLT BarChart widget. 
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#     max         - Label for the max value
#     min         - Label for the min value
#     uifBarChart - Bar Chart widget
#   METHODS
#     The class inherits the methods of class UifWidget.
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     axis     <1/0>         Enable show axis
#     barColor <color name>  Bar color
#     length   <number>      Widget length
#     minVal   <value>       Minimum value of the bar : can be real
#     maxVal   <value>       Maximum value of the bar : can be real
#     orient   <h/v>         BarChart Orientation : horizontal or vertical
#     ticks    <1/0>         Enable show ticks
#     variable <var. name>   Name of the variable associated to the widget. 
#   SPECIFIC BINDINGS
#
#   CAUTION
#
#   EXAMPLES
#     Create an instance
#
#     UifBarChart .w  -barColor Blue -ticks 1 -axis 1 -dbMode dbPoll\
#                      -label barChart -length 100 -maxVal 150-minVal -20 \
#                      -orient h -ticks 1 -variable gvar(myVar)
#
#     Re-configure the widget
#        .w configure -label "new Label" -barColor Red -minVal  -20.5
#
#     Other operations
#        set label   [.w cget -label]      Get a widget's resource
#        set minVal  [.w cget -minVal]
#   SEE ALSO
#     UifWidget UifOutputWidget
#   BUGS
#
#------------------------------------------------------------------------
#

option add *UifBarChart.label  ""  widgetDefault

itk::usual UifBarChart { }

class UifBarChart {
    inherit UifOutputWidget

    constructor {args} {}
    destructor {}

    itk_option define -variable variable Variable ""
    itk_option define -orient   orient   Orient   h
    itk_option define -length   length   Length   100
    itk_option define -minVal   minVal   MinVal   0
    itk_option define -maxVal   maxVal   MaxVal   100
    itk_option define -axis     axis     Axis     1
    itk_option define -ticks    ticks    Ticks    1
    itk_option define -barColor barColor BarColor ""
      
    public method setValue {value} 
    public method configure { {option ""} args }

    private method packUifBarChart {}

    protected variable myName       "myBar"
    protected variable tickSize     0
    protected variable oldVar       ""
    protected common id 1   ;# Temporary patch ...

}; # end of class

#------------------------------------------------------
# Destructor
#------------------------------------------------------

body UifBarChart::destructor {} {
   # Temporary patch ...
   catch { destroy $itk_component(uifBarChart) } 

   if {! [cequal $oldVar ""]} { varTraceOff }  
}

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifBarChart::constructor { args } {

   itk_component add f1 {
     frame $itk_interior.f1
   }

   pack $itk_component(f1) -fill both
   
   itk_component add min {
     label $itk_component(f1).min
   } {
     keep -text
   }

   itk_component add max {
     label $itk_component(f1).max
   } {
     keep -text
   }

   itk_component add uifBarChart {
      ::blt::barchart  $itk_component(f1).uifBarChart${id} \
                       -plotpadx 1 -plotpady 1 -title ""
   } {
     keep   -invertxy -height -width -background
     rename -title -label label Label
   }

   incr id
   eval itk_initialize -label \"\"
   set initialized 1

   $itk_component(uifBarChart) element create  myBar -xdata 0 -ydata 0 \
                              -foreground Blue -background White

   $itk_component(uifBarChart) legend configure  -hide yes ;# No legend
   $itk_component(uifBarChart) xaxis  configure  -hide yes ;# No X axis
   $itk_component(uifBarChart) yaxis  configure  -title ""
 
   #  Explicitly handle config's that may have been ignored earlier
 
   eval configure $args
}


#----------------------------------------------------------------------
# METHOD : configure
# The method configure has been re-defined because of the need to
# execute the method setSize and setLength at the end of the whole 
# configuration process in order not to repeat the execution several times
#
# Parameter 'ret' returns the current configuration when 'configure' is
# called without arguments
#
#----------------------------------------------------------------------

body UifBarChart::configure { {option ""} args } {
   set ret [eval itk::Archetype::configure $option $args]
   packUifBarChart
   return $ret
}


#---------------------------------------------------------------
# OPTION : -barColor
#---------------------------------------------------------------

configbody  UifBarChart::barColor { 
   if {$initialized } {
     $itk_component(uifBarChart) element configure $myName -fg $itk_option(-barColor)
   }
}


#---------------------------------------------------------------
# OPTION : -minVal
#---------------------------------------------------------------

configbody  UifBarChart::minVal {
   $itk_component(uifBarChart) yaxis configure -min $itk_option(-minVal)
}


#---------------------------------------------------------------
# OPTION : -maxVal
#---------------------------------------------------------------

configbody  UifBarChart::maxVal {
   $itk_component(uifBarChart) yaxis configure -max $itk_option(-maxVal);
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody  UifBarChart::variable {
   if {$initialized } {
      setTraceVar   $itk_option(-variable)
      addToDataList $itk_option(-variable)
      setValue [uplevel #0 set $itk_option(-variable)]
   }
}

#---------------------------------------------------------------
# OPTION : -axis
#---------------------------------------------------------------

configbody UifBarChart::axis {
   if {$itk_option(-axis) == 0} {
      $itk_component(uifBarChart) yaxis configure -hide yes
   } else {
      $itk_component(uifBarChart) yaxis configure -hide no
   }
}

#---------------------------------------------------------------
# OPTION : -ticks
#---------------------------------------------------------------

configbody UifBarChart::ticks {        
   $itk_component(uifBarChart) yaxis configure -showticks $itk_option(-ticks) 
}

#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------
           
body UifBarChart::setValue { value } {
   if {[cequal $value ""] } { return }         
   $itk_component(uifBarChart) element configure $myName -ydata $value 
} 

#---------------------------------------------------------------
# METHOD : packUifBarChart
#---------------------------------------------------------------
                 
body UifBarChart::packUifBarChart {} {
   
   pack forget $itk_component(min) $itk_component(max) \
               $itk_component(uifBarChart)

#   set sf   [expr [winfo pixels . 1m]/ 4.0]
#   set sf   1.0

   set size 25

#   set aaa "sf [expr [winfo pixels . 1m]/ 4.0] tick $itk_option(-ticks)  axis $itk_option(-axis) label $itk_option(-label)"

   if { [cequal $itk_option(-orient) v] } {
           
      # Vertical Configuration 

       if { $itk_option(-ticks) && $itk_option(-axis) } {
	   incr size 40
       }
       
      $itk_component(uifBarChart) configure -width $size \
	      -invertxy 0 -height $itk_option(-length)
                     
      if { $itk_option(-ticks) } {
         pack  $itk_component(uifBarChart) -side top  -anchor e
      } else {            
        pack $itk_component(max) $itk_component(uifBarChart) \
              $itk_component(min) -side top -fill x    
      }
          
   } else {                  
   
      # Horizontal Configuration

       if { $itk_option(-ticks) && $itk_option(-axis) } {
	    incr size 25
       }

       if { ! [cequal $itk_option(-label) ""]} {
	    incr size 25
       }

      $itk_component(uifBarChart) configure -height $size -invertxy 1 \
                                  -width $itk_option(-length)
        
      if { $itk_option(-ticks) } {
         pack $itk_component(uifBarChart) -side left  
      } else {
         pack $itk_component(min) $itk_component(uifBarChart) \
              $itk_component(max) -side left -fill y 
      }
   }
 

#      puts "$itk_option(-orient)  $aaa        size ${size} $ss"

   if { ! $itk_option(-ticks) } {   
      $itk_component(max) configure -text $itk_option(-maxVal)        
      $itk_component(min) configure -text $itk_option(-minVal)
   }
}

#___oOo___
