#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifEntry.tcl,v 3.36 2003/01/20 10:33:46 vltsccm Exp $" 
#
# uifEntry.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# M.Comin    07/11/94   Add methods and documentation
# M.COMIN    18/02/94   Taken away variable 'oldvar'
# M.COMIN    15/03/95   Man page updated and added HOME,END bindings
# M.COMIN    31/07/95   Use args instead of config
# -------------------
# mcomin     09/03/96   Ported to Tk4.0 and itcl 2.0
#            13/05/96   Update man page
#            24/06/96   Add get method
#            17/12/96   The widget contains a label to be packed before
#                       the real entry.
#            12/12/97   Add option -units
#            26/04/98   Add default values for options
#************************************************************************
#   NAME
#     UifEntry - Class for VLT entry widget
#   SYNOPSIS
#     UifEntry  <widget-name>
#   DESCRIPTION
#
#     UifEntry defines the class for the VLT entry widget. The entry widget
#     is formed by a single line and allows the user to type in data.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifEntry - Entry widget
#     label    - Optional label at the left widget side
#   METHODS
#
#     Inherits the methods of class UifInteractiveWidget and support :
#
#     enable    
#        Enable the widget editing functions. 
#        Overloaded from UifInteractiveWidget class.
#
#     disable    
#        Disabel the widget : it will appear with 'flat' relief.
#        Overloaded from UifInteractiveWidget class.
#
#     checkType   (Not yet implemented ) 
#        Returns 1 if the input is of the proper type otherwise 0 
#        valid types are : ASCII or NUMERICAL
#
#     checkRange
#        Returns 1 if the input is on range otherwise 0.
#
#     get <from> <to>
#         Returns the current widget value 
#      
#     setRange    (Not yet implemented ) 
#        Defines range
#  
#     insert <pos> <text>
#        Insert <text> in position <pos>
#
#     delete <first> <last>
#        Delete characters from position <first> to position <last>
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifIntercativeWidget classe and supports 
#     the followings :
#
#     asciiFlag  <1/0>         Not Implemented
#     label      <string>      String used to label the widget
#     labelFont  <string>      New font for the label
#     labelWidth <string>      Label width : can be used to align several
#                              widgets with different labels in columns
#     maxVal     <value>       Not Implemented
#     minVal     <value>       Not Implemented
#     units      <string>      String attached to the right side
#                              indicating the nature of the value
#     variable   <var. name>   Variable associated to the widget
#     width      <integer>     Set the length of the entry field
#   SPECIFIC BINDINGS
#
#     <Delete>       Deletes character left to the insertion cursor
#     <BackSpace>    Deletes character left to the insertion cursor
#     <End>          Sets insertion cursor at the end of the field.
#                    It displays the last part of the typed-in data. 
#     <Home>         Sets insertion cursor at the beginning of the field.
#                    It displays the beginning of the typed-in data.
#     <Arrow-Left>   Moves the insertion cursor leftwards
#     <Arrow-Right>  Moves the insertion cursor rightwards
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifEntry .w  -maxVal 60 -minVal -20 -width 10 -variable gvar(myVar) 
#
#     Re-configure the widget
#        .w configure -width  15 -maxVal 10
#
#     Other operations
#        set min   [.w cget -min]        Get a widget's resource
#        set myVar [.w cget -variable]
#        .w  disable ;
#   SEE ALSO
#    UifWidget  UifInteractiveWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

option add *UifEntry.label      ""   widgetDefault
option add *UifEntry.labelFont  "-Adobe-helvetica-medium-r-normal--14*"  widgetDefault
option add *UifEntry.labelWidth 0    widgetDefault
option add *UifEntry.width      10   widgetDefault

itk::usual UifEntry { 
    keep -background
}

class UifEntry {
    inherit UifInteractiveWidget

    constructor {args} {}

    itk_option define -minVal   minVal   MinVal    0
    itk_option define -maxVal   maxVal   MaxVal    100
    itk_option define -label    label    Label     ""
    itk_option define -units    units    Units     ""

    # The following options are kept just for backward compatibility

    itk_option define -activeFont          activeFont     ActiveFont    ""
    itk_option define -activeBackground    actBackground  ActBackground ""
    itk_option define -selectedBackground  selBackground  SelBackground ""
    itk_option define -confirm             confirm        Confirm       ""

    public  method erase      {} { $itk_component(uifEntry) delete 0 end }
    public  method enable     {}
    public  method disable    {}
    public  method checkRange {}
    
    public  method insert {pos   text }
    public  method delete {first last }
    public  method get    { }

    private method setThisVar
    
    private variable oldVariable ""
}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifEntry::constructor {args} {

   itk_component add uifEntry {
      entry $itk_interior.uifEntry -bd 2 -relief sunken
   } {
      rename -textvariable -variable variable Variable
      keep -width -background -foreground -selectforeground \
           -state -selectbackground -font
   }

   itk_component add label {
      label $itk_interior.label -anchor w -relief flat
   } {
     rename -width -labelWidth labelWidth LabelWidth
     rename -font  -labelFont labelFont LabelFont
     rename -foreground -labelColor labelColor LabelColor
     keep -background
   }

   itk_component add units {
      label $itk_interior.units -anchor w -relief flat
   } {
     keep -text
     keep -background
    rename -font  -labelFont labelFont LabelFont
   }

   pack $itk_component(label) $itk_component(uifEntry) -side left -fill x

   itk_initialize ; set initialized 1
   eval configure $args
}

#---------------------------------------------------------------
# METHOD : insert
#---------------------------------------------------------------

body UifEntry::insert { pos text } {
   $itk_component(uifEntry) insert $pos $text
}
 
#---------------------------------------------------------------
# METHOD : get
#---------------------------------------------------------------

body UifEntry::get {} {
   return [$itk_component(uifEntry) get]
}

#---------------------------------------------------------------
# METHOD : delete
#---------------------------------------------------------------

body UifEntry::delete { first last } {
   $itk_component(uifEntry) delete $first $last
}
 
#---------------------------------------------------------------
# METHOD : disable
#---------------------------------------------------------------

body UifEntry::disable {} {
   $itk_component(uifEntry) configure -state disabled -relief groove
}
    
#---------------------------------------------------------------
# METHOD : enable
#---------------------------------------------------------------

body UifEntry::enable {} {
   $itk_component(uifEntry) configure -state normal -relief sunken
}

#---------------------------------------------------------------
# METHOD : checkRange
#---------------------------------------------------------------    

body UifEntry::checkRange {} {
   set var $itk_option(-variable)
   set myVal [uplevel #0 set $var]

   if { $myVal < $itk_option(-minVal) || $myVal > $itk_option(-maxVal) } { 
      return 0 
   }
   return 1    
}


#---------------------------------------------------------------
# METHOD : setThisVar 
#---------------------------------------------------------------    

body UifEntry::setThisVar  {} {
   global ${this}_variable
   set var $itk_option(-variable)
   set ${this}_variable [uplevel #0 set $var]
}

#---------------------------------------------------------------
# OPTION : -units
#---------------------------------------------------------------

configbody UifEntry::units {
    if {[cequal $itk_option(-units) "" ]} {
       pack forget $itk_component(units)
    } else {
       $itk_component(units) configure -text "($itk_option(-units))"
       pack $itk_component(units) -side left -padx 3 \
            -after $itk_component(uifEntry)
    }
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------

configbody UifEntry::label {
    if {[cequal $itk_option(-label) ""] } {
       pack forget $itk_component(label)
    } else {
       $itk_component(label) configure -text "$itk_option(-label) : "
       pack $itk_component(label) -side left -before $itk_component(uifEntry)
    }
}

#---------------------------------------------------------------
# OPTION : -minVal
#---------------------------------------------------------------
 
configbody UifEntry::minVal {
   if { [cequal [string index $itk_option(-minVal) 0] "@"] } {
      ::set itk_option(-minVal) [dbRead $itk_option(-minVal)]
   }
}

 #---------------------------------------------------------------
# OPTION : -maxVal
#---------------------------------------------------------------
    
configbody UifEntry::maxVal {
   if { [cequal [string index $itk_option(-maxVal) 0] "@"] } {
      ::set itk_option(-maxVal) [dbRead $itk_option(-maxVal)] 
   }
}


#___oOo___
