#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifFileBrowser.tcl,v 3.36 2003/01/20 10:33:57 vltsccm Exp $"
#
# who       when        what
# --------  --------    ----------------------------------------------
# mcomin    04/06/97    created
#           17/08/98    Add TCLTK_ROOT to the list of standard dirs.
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#
#************************************************************************
#   NAME
#     UifFileBrowser - VLT File Browser
#   SYNOPSIS
#     UifFileBrowser  <widget-name>
#   DESCRIPTION
#
#     UifFileBrowser implements a file browser for the VLT file structure.
#     The user create an instance of file browser by :
#
#         UifFileBrowser  <widget-name> <config-parameters>
#
#     The user can set the 'Search Path' to the directory MODULE, INTROOT,
#     VLTROOT, VLTDATA and Other.
#     
#     If the 'Search Path' is 'Other' the user can specify an arbitrary 
#     directory as 'Root Directory' : pressing RETURN sets the file browser
#     to the specified directory if it exists, otherwise it has no effect.
#
#     The returned filename can be 'Absolute' or 'Relative' to the current 
#     'search directory'. 
#
#   COMPONENTS
#
#   METHODS
#
#     The class supports the following methods :
#
#     filter <mask>
#       The File Browser is updated with the list of files found 
#       in the current directory whose names match with the <mask>. 
#
#       For example :      <widgetName> filter "*.tcl"
#
#       will display all files with '.tcl' extension.      
#
#     getFile 
#       Stops the Tcl interpreter until the user makes a selection and 
#       returns a list of two elements : the selected filename and a list
#       of information on the selected file.
#
#       The second element contains the following information :
#
#       - type     : Returned values are file, directory, link or socket
#       - size     : File size in bytes
#       - exist    : Boolean flag to indicate if file exists
#       - writable : Boolean flag to indicate if file is writable 
#                    for the user
#
#     CD <directory>
#       The File Browser is updated with the list of files found 
#       in the new directory and according to the current filter mask. 
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Supports the following options :
#
#     iconify  <1/0>      Controls the treatment of the 'toplevel'
#                         window after a selection :
#                         '1' , the 'toplevel' window is iconified.
#                         '0' , the 'toplevel' window is destroyed.
#
#     mask     <string>   Defines the extension of the files to be 
#                         displayed in the box.
#     subDir   <string>   Relative directory path. This is appended to the
#                         selected root directory when MODULE,INTROOT or 
#                         VLTROOT are choosen.
#     title    <string>   Specifies a title for the 'toplevel' window.
#
#   SPECIFIC BINDINGS
#
#   <Return>   in the 'Root Directory' entry field sets the file browser
#              to the specified directory if it exists, otherwise it has 
#              no effect.
#   CAUTIONS
#
#   RETURN VALUES 
#     The selected filename is available by using the getFile method.
#     If the user did not select any file the retuned value is "".
#
#   EXAMPLES
#
#     Create instance of UifFileSelectionBox : the mask is set to look
#     for Tcl files and after the selection the panel is only iconified.
#
#     UifFileBrowser .x -iconify 1 -title "File Browser" -mask "*.tcl"
#
#     .x CD $VLTROOT/lib  ;# Set the file bowser to specific directory
#
#     Retrieve the file selected by the user
#
#     lassign [.x getFile] filePath fileInfo
#
#     if {$filePath == ""} {
#        .. no file selected
#     } else {
#        .. process the file
#     }
#
#   SEE ALSO
#     UifFileSelBox
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual UifFileBrowser {}

class UifFileBrowser {
    inherit itk::Toplevel

    constructor {args} {}
 
    itk_option define -subDir subDir SubDir ""
    itk_option define -newDir newDir NewDir .

    public method getFile {}
    public method CD      {dir}  { $itk_component(fileBox) CD $dir }
    public method filter  {what} { $itk_component(fileBox) filter $what }

    # Internal methods, they are declared as public because used in bind
    public method setDir  {}
    public method setPath {}

    private method checkRootDir { newDir } {}

    private variable initialized 0
    private variable wthis ""
    private variable INTROOT "undefined"
    private variable VLTROOT "undefined"
    private variable VLTDATA "undefined"
    private variable TCLTK_ROOT "undefined"
    private variable MODULE  "."
    private variable HOME    "."
    private variable newDirPath "."
}
 
#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifFileBrowser::constructor { args } {
   global env tcl_version

   set wthis [namespace tail $this]

   itk_component add custom {
      frame  $itk_interior.custom -bd 3 -relief ridge
   }

   #
   #  Definition of names for frames
   #
   
   set f11 [frame  $itk_component(custom).f11]
   set f12 [frame  $itk_component(custom).f12]
   set f2 [frame  $itk_component(custom).f2]
   set f3 [frame  $itk_component(custom).f3]
   set f4 [frame  $itk_component(custom).f4]

   pack  $f11 $f12 $f2 $f4 $f3 -fill x -padx 3
   
   # Add radioButtons to set the root directory

   label $f11.lib -text "Search Path" -width 13 -anchor w
   pack $f11.lib -side left

   radiobutton $f11.mod  -variable ${wthis}_libRoot -value MODULE\
           -anchor w  -text "Module"  -command "$this setDir"  -width 12
   radiobutton $f11.int  -variable ${wthis}_libRoot -value INTROOT\
            -anchor w -text "INTROOT" -command "$this setDir" -width 12
   radiobutton $f11.vlt  -variable ${wthis}_libRoot -value VLTROOT\
            -anchor w  -text "VLTROOT" -command "$this setDir" -width 12

   radiobutton $f12.data -variable ${wthis}_libRoot -value VLTDATA\
                 -anchor w  -text "VLTDATA" -command "$this setDir" -width 12
   radiobutton $f12.tcltk -variable ${wthis}_libRoot -value TCLTK_ROOT\
             -anchor w -text "TCLTK_ROOT" -command "$this setDir" -width 12
   radiobutton $f12.local -variable ${wthis}_libRoot -value local \
             -anchor w -text "Other" -command "$this setDir" -width 12

   pack $f11.vlt $f11.int $f11.mod      -side right -padx 5 -pady 3
   pack $f12.local $f12.tcltk $f12.data -side right -padx 5 -pady 3

   itk_component add rootDir {
       UifEntry  $f2.rootDir -label "Root Directory" -width 50 -labelWidth 15
   } {
   }

   itk_component add subDir {
       UifEntry  $f4.subDir -label "Sub Directory" -width 50 -labelWidth 15
   } {
   }

   # Add switch to select between absolute/relative return path

   label $f3.lbl1 -text "Returned Path " -width 13 -anchor w

   radiobutton $f3.rel  -variable ${wthis}_path -value Relative\
                        -text "Relative" -command "$this setPath"
   radiobutton $f3.abs  -variable ${wthis}_path -value Absolute\
                        -text "Absolute" -command "$this setPath"

   pack $f3.lbl1 -side left
   pack $f3.abs $f3.rel -side left -padx 3 -pady 3

   # Add File Selection Box component

   itk_component add fileBox {
       UifFileSelBox  $itk_interior.box -topLevel $wthis -dirGeometry 30x10 \
                      -fileGeometry 25x10 
   } {
     keep -iconify -mask -title
   }  

   pack $itk_component(rootDir) -side left -pady 5
   pack $itk_component(subDir)  -side left -pady 5
   pack $itk_component(custom) -pady 5  -fill both
   pack $itk_component(fileBox) -fill both -expand 1

   # Set default values and initialize some private variables
 
   uplevel #0 set ${wthis}_path  Absolute

   set MODULE [file dirname [pwd]]
   set HOME   [pwd]

   eval itk_initialize $args

   bind [$itk_component(rootDir) component uifEntry] <Return> "$this setDir"

   wm withdraw   $itk_component(hull)
   wm group      $itk_component(hull) .
   wm resizable  $itk_component(hull) 1 1
   wm minsize    $itk_component(hull) 565 520

   # Re-define INTROOT/VLTROOT with complete path to avoid problems 
   # with remotely mounted disks

   if {[info exists env(INTROOT)] } {
      cd $env(INTROOT)
      set INTROOT [pwd]
   }     

   if {[info exists env(VLTROOT)] } {
      cd $env(VLTROOT)
      set VLTROOT [pwd]
   } 
    
   if {[info exists env(VLTDATA)] } {
      cd $env(VLTDATA)
      set VLTDATA [pwd]
   }

   if {[info exists env(TCLTK_ROOT)] } {
      cd $env(TCLTK_ROOT)
      set TCLTK_ROOT  [pwd]
   }


#   checkRootDir $HOME
   cd $HOME

   eval configure $args

   $itk_component(subDir) delete 0 end
   $itk_component(subDir) insert 0 $itk_option(-subDir)

   set initialized 1

   configure -newDir  $itk_option(-newDir)

   wm deiconify  $itk_component(hull)
}

#---------------------------------------------------------------
# OPTION : -newDir
#---------------------------------------------------------------  
    
configbody UifFileBrowser::newDir {
   if {$initialized} {
      checkRootDir $itk_option(-newDir)
      setDir
   } 
}
#---------------------------------------------------------------
# METHOD : checkRootDir
#---------------------------------------------------------------

body UifFileBrowser::checkRootDir { newDir } {

   if {[file exists $newDir]} {
      set tmpHome [pwd]
      cd $newDir
      set newDirPath [pwd]
      # Check which if current directory is INTROOT/VLTROOT or VLTDATA

      if {[string match ${INTROOT}* $newDirPath ]} {
         uplevel #0 set ${wthis}_libRoot INTROOT
      } elseif {[string match ${VLTROOT}* $newDirPath ]} {
         uplevel #0 set ${wthis}_libRoot VLTROOT
      } elseif {[string match ${VLTDATA}* $newDirPath ]} {
         uplevel #0 set ${wthis}_libRoot VLTDATA
      } elseif {[string match ${tmpHome}* $newDirPath ] } {
         uplevel #0 set ${wthis}_libRoot MODULE
      } elseif {[string match ${TCLTK_ROOT}* $newDirPath ] } {
         uplevel #0 set ${wthis}_libRoot TCLTK_ROOT
      } else {
         uplevel #0 set ${wthis}_libRoot local
      }

      cd $tmpHome
   } else {
      puts "Cannot change to directory   $newDir"
   }
}

#---------------------------------------------------------------
# METHOD : getFile
# Waits until the user has selected a file
#---------------------------------------------------------------

body UifFileBrowser::getFile {} {
   wm deiconify $itk_component(hull)
   raise $itk_component(hull)

   uplevel #0 vwait $itk_component(fileBox)_fsbReturn

   set fsbReturn  [uplevel #0 set $itk_component(fileBox)_fsbReturn]
   set fsbInfo    [uplevel #0 set $itk_component(fileBox)_fsbInfo]

   update idletasks ;# Needed for the file selection box disappear
   return [list $fsbReturn $fsbInfo]
}

#---------------------------------------------------------------
# METHOD : setPath
#---------------------------------------------------------------

body UifFileBrowser::setPath {} {

   if {[uplevel #0 set ${wthis}_path] == "Relative"} {

      set rootDir [uplevel #0 set ${wthis}_libRoot]

      switch $rootDir {
	 MODULE    { set tmp $MODULE }
         INTROOT   { set tmp $INTROOT }
         VLTROOT   { set tmp $VLTROOT }
         VLTDATA   { set tmp $VLTDATA }
         TCLTK_ROOT { set tmp $TCLTK_ROOT}
         local     { set tmp [$itk_component(rootDir) get]}
         default   { set tmp [$itk_component(fileBox) getDir] }
      }

      if {[set subDir [$itk_component(subDir) get]] != "" } { 
         if {[set len [string first $subDir $tmp]] != -1 } {
	     set tmp [string range $tmp 0 [expr {$len -1} ]]
         }
      }

      $itk_component(fileBox) configure -path $tmp
   } else {
      $itk_component(fileBox) configure -path ""
   }
}

#---------------------------------------------------------------
# METHOD : setDir
#---------------------------------------------------------------

body UifFileBrowser::setDir {} {
   global env

   set rootDir [uplevel #0 set ${wthis}_libRoot]

   if {[cequal $rootDir local]} {
      $itk_component(rootDir) enable
   } else {
      $itk_component(rootDir) disable
   }

#   puts "itk_option(-newDir) = $itk_option(-newDir)  HOME = $HOME"
#   puts "newDirPath = $newDirPath  MODULE = $MODULE"

   switch $rootDir {
      MODULE  {
	  if {[file exists $MODULE] == 0} {
             uplevel #0 set ${wthis}_libRoot $HOME
             setDir; return
          }
	  set tmp $MODULE
      }   
     
      VLTDATA { 
	  if {[file exists $VLTDATA] == 0} {
             uplevel #0 set ${wthis}_libRoot MODULE
             setDir; return
          }
	  set tmp $VLTDATA
      }

      INTROOT { 
	  if {[file exists $INTROOT] == 0} {
             uplevel #0 set ${wthis}_libRoot MODULE
             setDir; return
          }
	  set tmp $INTROOT
      }
          
      VLTROOT {  
	  if {[file exists $VLTROOT] == 0} {
             uplevel #0 set ${wthis}_libRoot MODULE
             setDir; return
          }
	  set tmp $VLTROOT
      }


      TCLTK_ROOT {  
	  if {[file exists $TCLTK_ROOT] == 0} {
             uplevel #0 set ${wthis}_libRoot MODULE
             setDir; return
          }
	  set tmp $TCLTK_ROOT
      }

      default {

          set tmp [$itk_component(rootDir) get]

          if {[cequal $tmp ""]} {
             if {[cequal $itk_option(-newDir) "."]} {
                 set tmp [pwd]
             } else {
                 set tmp $itk_option(-newDir)
             }
             $itk_component(rootDir) insert 0 $tmp
	  }
      }
   } ;# end of switch

   # Trick to get the 'true' name of the root directory

   if {[file exists $tmp]} { cd $tmp }

   if {![cequal $rootDir local]} {
      if {[set subDir [$itk_component(subDir) get]] != ""} {
         # Append sub directory to module directory

         if {[cequal $rootDir MODULE]} { set tmp $MODULE }

         if {[file exists ${tmp}/${subDir}]} {
	    append tmp "/${subDir}"
         }
      }
   }
 
   $itk_component(fileBox) CD $tmp
   setPath ;# Update File Selection Box according to path mode
   cd $HOME
}

#
# ___oOo___



