#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifFileSelBox.tcl,v 3.36 2003/01/20 10:33:46 vltsccm Exp $" 
#
# uifFileSelBox.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.Comin    07/11/94   created
# -------------------
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            14/05/96   Update man page
#            04/06/97   Add options -path, -mask
#                       Man page upgrade 
#            23/03/98   Files shown with left-alignement.
#
#************************************************************************
#   NAME
#     UifFileSelBox - Class for VLT File Selection Box
#   SYNOPSIS
#     UifFileSelBox  <widget-name>
#   DESCRIPTION
#     UifFileSelBox defines the class for the VLT File Selection Box widget.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     filter    - Entry widget : 'filter' specification
#     selection - Entry widget : 'selection' specification
#     listDir   - List Box : list of sub-directories
#     listFile  - List Box : list of files in the selected sub-directory
#   METHODS
#
#     The class supports the following methods :
#
#     filter <mask>
#       The File Selection Box is updated with the list of files found 
#       in the current directory whose names match with the <mask>. 
#
#       For example :      <widgetName> filter "*.tcl"
#
#       will display all files with '.tcl' extension.      
#
#     CD <directory>
#       The File Selection Box is updated with the list of files found 
#       in the new directory and according to the current filter mask. 
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     dirGeometry  <w>x<h>   Width/Height of the box listing the directories
#     fileGeometry <w>x<h>   Width/Height of the box listing the files
#
#     iconify      <1/0>     Controls the treatment of the 'toplevel'
#                            window after a selection :
#                            '1' , the 'toplevel' window is iconified.
#                            '0' , the 'toplevel' window is destroyed.
#     mask         <string>  Defines the extension of the files to be 
#                            displayed in the box.
#     path         <dir>     If this is defined, the returned file name
#                            is relative to the <path> directory
#     title        <string>  Specifies a title for the 'toplevel' window.
#
#     topLevel     <widget name>  Name of the top level window.
#
#   SPECIFIC BINDINGS
#
#     See documentation about OSF/MOTIF File Selection Box.
#
#   CAUTIONS
#
#   RETURN VALUES 
#
#     The returned value is stored into two global variables :
#
#     <widgetName>_fsbReturn  contains the selected filename.
#
#     <widgetName>_fsbInfo    contains a the following list of information
#                             about the file :
#
#         - type     : Returned values are file, directory, link or socket
#         - size     : File size in bytes
#         - exist    : Boolean flag to indicate if file exists
#         - writable : Boolean flag to indicate if file is writable 
#                      for the user
#
#     If the user did not select any file the both returned global variables
#     are set to "".
#
#
#   EXAMPLES
#
#     Define toplevel window
#
#     set wtop .browser
#     toplevel $wtop 
#
#     Create instance of UifFileSelBox : the mask is set to look
#     for Tcl files and after the selection the panel is only iconified.
#
#     set fsb $top.fsb
#
#     UifFileSelBox $fsb -fileGeometry 30x10 -dirGeometry 15x10 \
#                        -mask "*.tcl" -topLevel $wtop -iconify 1
#
#     Configure the file selection box to return filenames relative
#     to a given directory -e.g.- VLTROOT
#
#        $fsb configure -path $VLTROOT
#
#        $fsb filter "*.c"  Refresh list of files according to the new mask.
#        $fsb CD     [pwd]  Change directory
#
#     Wait for the user selection : in this case the Tcl interpreter must
#     wait until the user has selected a file or pressed 'Cancel'.
#
#        uplevel #0 vwait ${fsb}_fsbReturn
#
#        set filePath [uplevel #0 set ${fsb}_fsbReturn]
#
#   SEE ALSO
#
#    UifWidget
#
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifFileSelBox { }

class UifFileSelBox {
    inherit UifWidget
    
    constructor {args} {}

    itk_option define -topLevel     topLevel     TopLevel      "."
    itk_option define -dirGeometry  dirGeometry  DirGeometry   "30x10"
    itk_option define -fileGeometry fileGeometry FileGeometry  "15x10"

    itk_option define -iconify  iconify  Iconify  0
    itk_option define -mask     mask     Mask     "*"
    itk_option define -path     path     Path     ""

    itk_option define -title    title    Title    "File Selection Box"

    public method CD     {newDir}
    public method filter { {mask {}} }

    # Used only internally

    public method cancel    {}
    public method ok        {}
    public method bind1     {}
    public method takeSel   {}
    public method setFilter {}
    public method getDir    {} { return $currDir }

    protected variable initialized 0
    private variable currDir "."

}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifFileSelBox::constructor {args} {

   itk_component add f1 {
      frame  $itk_interior.f1 
   }

   itk_component add f2 {
      frame  $itk_interior.f2 
   }
       
   itk_component add f3 {
      frame  $itk_interior.f3 
   }

   itk_component add f4 {
      frame  $itk_interior.f4 
   }

   pack $itk_component(f1) -fill x -pady 5
   pack $itk_component(f2) -fill both -expand 1 -pady 5
   pack $itk_component(f3) $itk_component(f4) -fill x -pady 5

   set tmpFont 9x15

   #
   # Frame 1 contains the "Filter" part
   #

   itk_component add lblFilter {
      label  $itk_component(f1).lblFilter -text "Filter :" -font $tmpFont
   }

   itk_component add filter {
      entry  $itk_component(f1).filter -relief sunken
   }
   
   pack $itk_component(lblFilter)  -anchor w -padx 5
   pack $itk_component(filter)     -anchor w -fill x -padx 5

   #
   # Frame 2 contains the list boxes
   #
  
   itk_component add listDir {
      UifListBox $itk_component(f2).listDir
   } {
      keep -label -geometry -font
   }

   itk_component add listFile {
      UifListBox $itk_component(f2).listFile
   } {
      keep -label -geometry -font
   }

   pack $itk_component(listDir) $itk_component(listFile) -side left \
                                -padx 5 -fill both -padx 5 -expand 1

   #
   # Frame 3 contains the "selection" part
   #

   itk_component add lblSel {
      label  $itk_component(f3).lblSel -text "Selection" -font $tmpFont
   }

   itk_component add selection {
      entry  $itk_component(f3).selection -relief sunken -font $tmpFont
   }

   pack $itk_component(lblSel)    -anchor w -padx 5
   pack $itk_component(selection) -anchor w -fill x -padx 5

   #
   # Frame 3 contains the button to interact with the widget
   #

   itk_component add butOK {
      button $itk_component(f4).ok  -text "OK"  -width 7 -command "$wthis ok"
   }

   itk_component add butFILTER {
      button $itk_component(f4).filter  -text "Filter"  -width 7 \
                                        -command "$wthis filter"
   }

   itk_component add butCANCEL {
      button $itk_component(f4).cancel  -text "Cancel"  -width 7 \
                                        -command "$wthis cancel"
   }

   itk_component add butHELP {
      button $itk_component(f4).help -text "Help" -width 7 -command ""
   }

   pack  $itk_component(butOK) $itk_component(butFILTER)  \
         $itk_component(butCANCEL)  $itk_component(butHELP) \
         -side left  -expand 1  -pady 5

   eval itk_initialize 

   set initialized 1

   $itk_component(listDir) configure -label "Directories" \
                                     -geometry 30x10 -font $tmpFont

   $itk_component(listFile) configure -label "Files" \
                                      -geometry 20x10 -font $tmpFont
   #
   # Set Specific  Bindings
   #  

   $itk_component(listFile) setBind <Double-Button-1> \
                            "$wthis takeSel; $wthis ok"
   $itk_component(listFile) setBind <1>               "$wthis takeSel"
   $itk_component(listDir)  setBind <Double-Button-1> "$wthis bind1"

   bind $itk_component(filter)     <Return> "$wthis setFilter"
   bind $itk_component(selection)  <Return> "$wthis ok"

   set currDir [pwd]
   eval configure $args
   filter
}

body UifFileSelBox::setFilter {} {
   configure -mask [$itk_component(filter) get]
   filter
}

body UifFileSelBox::bind1 {} {
   CD [$itk_component(listDir) getSelection]
}

#---------------------------------------------------------------
# METHOD : CD 
# Change Directory : The File Selection Box is updated with the list
#                    of files found in the new directory.
#---------------------------------------------------------------

body UifFileSelBox::CD {newDir} {
   if {[cequal $newDir ""]} { return }

   if {! [file exists $newDir] } {
      puts "UifFileSelBox::CD > Cannot change to directory $newDir"
      return 
   }

   set tmpHome [pwd]
   cd $newDir
   set wildCard [pwd]/*
   set tmp [glob -nocomplain $wildCard] ;# Insert /.. at the top of the list

   $itk_component(listDir) deleteItem 0 end 
   $itk_component(listDir) addItem    end  "[pwd]/.."
   set currDir [pwd]

   foreach t [lsort -ascii $tmp] {
      if {[file isdirectory $t] && [file readable $t]} { 
           $itk_component(listDir) addItem end "$t/" 
      }
   }

   $itk_component(listDir)  moveto h 1.0  
   filter
   cd $tmpHome
}

#---------------------------------------------------------------
# METHOD : filter
# Update the list of files  according to the  new filter
#---------------------------------------------------------------

body UifFileSelBox::filter { {what {}} } {
   set tmpHome [pwd]
   $itk_component(listFile)  deleteItem 0 end  ;# Clear current list of files
   $itk_component(selection) delete     0 end  ;# Clear Selection input field

   if {![cequal $what ""]} { configure -mask $what }

   if {! [file exists $currDir] } {
      puts "UifFileSelBox::filter > Cannot change to directory $currDir"
   } else {

      cd $currDir
      foreach t  [lsort -ascii [glob -nocomplain $itk_option(-mask)]] {
         if {[file isfile $t]} {
             $itk_component(listFile) addItem end "$t"
         }
      }
   }
 
   $itk_component(selection) insert 0 $currDir
   cd $tmpHome  
}


#---------------------------------------------------------------
# METHOD : cancel
#---------------------------------------------------------------

body UifFileSelBox::cancel {} {
   uplevel #0 set ${wthis}_fsbReturn \"\"
   uplevel #0 set ${wthis}_fsbInfo   \"\"

   if {$itk_option(-iconify)} {
      wm iconify $itk_option(-topLevel)
      wm withdraw $itk_option(-topLevel)
   } else {
       after idle destroy $itk_option(-topLevel) 
   }
}

#---------------------------------------------------------------
# METHOD : ok
#---------------------------------------------------------------

body UifFileSelBox::ok {} {

   if {[cequal $itk_option(-path) ""]} {
       set fsbReturn  [$itk_component(selection) get]
   } else {
       set tmp [$itk_component(selection) get]
       if {[string match $itk_option(-path)* $tmp]} {
          set len [string length $itk_option(-path)]
          set tmp [string range $tmp $len end]
          set fsbReturn [string trimleft $tmp "/"]
       } else {
          set fsbReturn $tmp
       }
   }

   # This is due to the fact that double click triggers also
   # another single click .. which must be processed before exiting
   
   if {$itk_option(-iconify) } {
      wm iconify $itk_option(-topLevel) 
   } else {
      after idle destroy $itk_option(-topLevel) 
   }

   # Record file infos

   if {[file exists $fsbReturn]} {
      set info [list [file type $fsbReturn] [file size $fsbReturn] \
               [file exists $fsbReturn] [file writable $fsbReturn] ]
   } else {

      # File does not exists .. so fill with default values
      set info [list file 0 0 1]
   }

   uplevel #0 set ${wthis}_fsbReturn $fsbReturn
   uplevel #0 set ${wthis}_fsbInfo   \"$info\"  
}

#---------------------------------------------------------------
# METHOD : takeSel
#---------------------------------------------------------------

body UifFileSelBox::takeSel {}  {    

   set file [$itk_component(listFile) getSelection ]
   
   $itk_component(selection) delete  0  end;  # Clear selection input field
   $itk_component(selection) insert  0  "${currDir}/$file"
}

#---------------------------------------------------------------
# OPTION : -dirGeometry 
#---------------------------------------------------------------  
    
configbody UifFileSelBox::dirGeometry {
  $itk_component(listDir) configure -geometry $itk_option(-dirGeometry)
}

#---------------------------------------------------------------
# OPTION : -fileGeometry 
#---------------------------------------------------------------  
    
configbody UifFileSelBox::fileGeometry {
  $itk_component(listFile) configure -geometry $itk_option(-fileGeometry)
}

#---------------------------------------------------------------
# OPTION : -mask 
#---------------------------------------------------------------  
    
configbody UifFileSelBox::mask {
  if {$initialized} {
     $itk_component(filter)  delete 0 end
     $itk_component(filter)  insert 0 $itk_option(-mask)
  }
}

#---------------------------------------------------------------
# OPTION : -path
#---------------------------------------------------------------  
    
configbody UifFileSelBox::path {
#   puts "UifFileSelBox::path \"$itk_option(-path)\""

   if {! [cequal $itk_option(-path) ""]} {
      if {[file exists $itk_option(-path)] } {
         set home [pwd]
         cd $itk_option(-path)
         set itk_option(-path) [pwd]
         cd $home
      }
   }
}

#---------------------------------------------------------------
# OPTION : -title
#---------------------------------------------------------------  
    
configbody UifFileSelBox::title {
   if {$initialized} {
       wm title $itk_option(-topLevel) $itk_option(-title)
   }
}

#___oOo___
