#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifListBox.tcl,v 3.36 2003/01/20 10:33:47 vltsccm Exp $" 
#
# uifListBox.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.Comin    07/11/94   created
# M.COMIN    15/03/95   Man page updated 
# M.COMIN    20/03/95   Add bg,fg,label public variables
# M.COMIN    21/03/95   Add method : addItemList
# M.COMIN    07/04/95   Add switch Y scroll ON/OFF
# M.COMIN    20/04/95   Add method getList
# M.COMIN    31/07/95   Use args instead of config
# M.COMIN    10/08/95   Add check on current selection index
# M.COMIN    18/08/95   Add check on getSelection and getItem
# M.COMIN    02/10/95   Relief changed to sunken
#
# mcomin      6/03/96   Porting to Tk4.0 and itcl2.0
#            07/05/96   Fixed bug in setBind method
#            13/05/96   Fixed bug in setVariable method
#            15/05/96   Update man page
#
#            05/02/97   Several upgrades
#            16/02/97   Add multiple selection, method setSelection upgraded
#            06/03/97   Porting to TK 4.2
#            02/06/97   Add method moveto
#            24/06/97   Activate default binding by setting focus to uifListBox
#            11/03/98   Changed method getList (improved version)
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#
#************************************************************************
#   NAME
#     UifListBox - Class for VLT listBox widget
#   SYNOPSIS
#     UifListBox  <widget-name>
#   DESCRIPTION
#
#     UifListBox defines the class for the VLT List Box widget. 
#     The widget also provides a vertical and horizontal scroll bars
#     which can be enabled/disabled through the associated widget options.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     scrollh    - Horizontal scroll bar
#     scrollv    - Vertical   scroll bar
#     uifListBox - Listbox widget
#   METHODS
#
#    General remarks :
#
#    - Positions in the list start at element 0.
#    - keyword "end" identifies the last position of the listBox
#
#     addItem <pos> <text>
#        Adds the string specified in <text> at position <pos>
#
#     addItemList <pos> <list>
#        Adds the specified list of items from  position <pos>
#
#     clearSelection
#        Clears all selections
#
#     deleteItem <first> <last>
#        Removes from the list elements from <first> to <last>.
#        If <last> is omitted, only the element <first> is removed.
#
#     getIndex
#        Returns the index of the currently selected item.
#
#     getList <first> <last>
#        Returns a list of elements, from <first> to <last>.
#
#        If <last> is omitted, it returns the element from the <first> on.
#        If this is called without parameters, it will return the whole
#        list elements.
#
#     getSize
#        Returns the total number of elements in the list.
#
#     getItem <pos>
#        Returns list elements in position <pos>
#        If <pos> is a wrong number, it return an empty string ("").
#
#     getSelection   
#        Returns the currently selected item.
#        If no element are selected or the listBox is empty it returns
#        an empty string ("")
#
#     moveto <h/v> <pos>
#        Shows the <pos> percentage of the listbox along h=horizontal or 
#        v=vertical direction :  <pos> must be in the range of 0 and 1, 
#        being 0 the top/left-side and 1 the bottom/right-side.
#
#     setBind <key> <action>
#        Allows the user to define specific binding.
#
#     setSelection <pos>
#        Sets the selection to element <pos> and returns the value of the 
#        selected element(s) : selected elements are highlighted.
#
#        If the selection mode is not 'single' more than one element can 
#        be selected : if <POS> = ALL all elements are selected.
#
#     updateList <array> <formatting_procedure>
#        Initializes the content of the listBox with the content of the <array>
#        Each element of the <array> becomes an entry of the listBox and
#        it is displayed according to the format returned by 
#        <formatting_procedure> (see example)
#
#     updateSortList <order> <type> <array> <formatting_procedure>
#        Perform the same function described for 'updateList', but the <array>
#        is first sorted according to the following flags :
#
#        <order>    Specifies the ordering as : "-decreasing" or"-increasing"
#
#        <type>     The type of data to be sorted : "-ascii", "-integer" or
#                   "-real"
#
#        If <order> <type> are set to "", the following default values
#        are used :  "-increasing", "-ascii"
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifInteractiveWidget class and supports :
#
#     font      <font name>
#     geometry  <width>x<height> 
#     itemList  <list>            List of elements.
#     label     <string>          Give a title to the listbox
#     scrollx   <1/0>             Enables X scrolling : default is 1.
#     scrolly   <1/0>             Enables Y scrolling : default is 1.
#     selectmode                  Valid values : single/browse/multiple
#     variable  <var. Name>       Name of the variable where to store
#                                 the selection.
#   SPECIFIC BINDINGS
#
#     <Button-1>     Selects an element of the list.
#                    The selected element is stored into the variable specified
#                    by widget option 'variable'.
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifListBox .w  -label myListBox -variable gvar(myItem) \
#                    -geometry 10x5 -scrollx 1
#
#     .w setBind <B1-ButtonRelease>  "myProcedure"    # Set Bindings 
# 
#     Add and Delete Elements
#    
#        set myList { item1 item2 item3 }
#        .w addItemList end $myList
#
#        Another alternative
#
#        foreach item $itemList {
#          .w addItem end $myList   
#        }
#
#     The following table summarizes the widget behaviour when the use
#     uses the 'setSelection' method to choose elements, for example, when
#     the widget is configured as follows :
#
#          UifListBox .l -itemList { a b "c d" e f g } -variable tmp
#
#         selectmode    |   statement             |   tmp contains
#     ______________________________________________________________________
#           single      | .w setSelection 2       | "c d"
#                       | .w setSelection 5       |  g
#                       | .w setSelection 2 3 5   |  g (the last element)
#                       | .w setSelection ALL     |  g ( tmp does not change)
#                       |                         | 
#       browse/multiple | .w setSelection 2       |  {c d}
#                       | .w setSelection 2 3 5   |  {c d} e g
#                       | .w setSelection ALL     |  a b {c d} e f g
#
#     Examples using arrays : In this example each element of the array
#     contains the information on a country : continent, area.
#
#     set array(japan)  "Asia  144"
#     set array(china)  "Asia  1247"
#     set array(Brazil) "{South America} 3286"
#     set array(France) "Europe 211"
#     
#     proc myFormat { text } {        
#        set country [lindex $text 0]
#        set pop     [lindex $text 1]
#        set newText [format "%-20s  %5d" $country $pop]
#        return $newText
#     }   
#
#     List items are displayed with no format
#       .w updateList array ; # List items are displayed with no format
#       .w deleteItem 0 end ; # Clear the list Box
#
#     List elements are displayed according to the format
#       .w updateList array myFormat

#     List elements are sorted and displayed according to the format
#       .w updateSortList "-decreasing" "-ascii" array myFormat
#   SEE ALSO
#     UifIntercativeWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

option add *UifListBox.relief sunken widgetDefault
option add *UifListBox.borderwidth 2 widgetDefault
option add *UifListBox.selectmode single widgetDefault

itk::usual UifListBox { }

class UifListBox {
    inherit UifInteractiveWidget

    constructor {args} {}

    itk_option define -variable  variable  Variable   ""
    itk_option define -scrollx   scrollx   Scrollx    1
    itk_option define -scrolly   scrolly   Scrolly    1
    itk_option define -geometry  geometry  Geometry   "5x5"
    itk_option define -label     label     Label      ""
    itk_option define -itemList  itemList  ItemList   {}    

    public method getSelection   {}
    public method getItem        {pos}
    public method deleteItem     {first {last {}} } 
    public method getList        {{first 0} {last end}}
    public method addItemList    {pos listOfItems}
    public method updateList     {array args }
    public method updateSortList { order type  array args } 
    public method setVariable    {var}
    public method setBind        {key action}
    public method setSelection   {pos}
    public method clearSelection {}
    public method setCursor      {pos}
    public method moveto         {what pos}

    public method getSize  {} {return [$itk_component(uifListBox) size] }
    public method getIndex {} {return [$itk_component(uifListBox) curselection ]}
    public method addItem {pos text} {$itk_component(uifListBox) insert $pos $text}  
    
    private method writeElements {nameList array}
    private method checkPos      {pos}

    private variable formatProc ""
    private variable packLabel  0
    private variable bindFlag   0

}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifListBox::constructor {args} {

   itk_component add f1 {
      frame  $itk_interior.f1 
   }

   itk_component add f2 {
      frame  $itk_interior.f2 
   }

   itk_component add label {
      label  $itk_interior.label
   } {
      keep -font
   }

   pack $itk_component(f1) -fill both -expand 1
   pack $itk_component(f2) -fill x    

   itk_component add uifListBox {
      listbox $itk_component(f1).uifListBox -selectmode single \
              -font "-Adobe-courier-medium-r-normal--14*"
   } { 
      keep -relief -font -yscrollcommand -xscrollcommand -borderwidth \
           -width -height -background -foreground -selectbackground \
           -selectforeground -selectmode
   }
   
   itk_component add scrollv {
      scrollbar $itk_component(f1).scrollv -orient v -width 10 -relief sunken\
                -command [code $itk_component(uifListBox) yview] 
   } { 
      keep -borderwidth -activebackground
   }

   itk_component add scrollh {
      scrollbar $itk_component(f2).scrollh -orient h -width 10 -relief sunken\
                -command [code $itk_component(uifListBox) xview]
   } { 
      keep -borderwidth -activebackground
   }

   pack $itk_component(uifListBox)  -side left -fill both -expand 1 
   pack $itk_component(scrollv)     -side left -fill y  
   pack $itk_component(scrollh)     -fill x
   
   eval itk_initialize 
   set initialized 1

   # Set the focus on the listbox in order to acivate the default bindings
   bind $itk_component(uifListBox) <1> "focus $itk_component(uifListBox)"

   $itk_component(uifListBox) configure \
                        -yscrollcommand [code $itk_component(scrollv) set] \
	                -xscrollcommand [code $itk_component(scrollh) set]

   eval configure -selectmode single $args
}    

#---------------------------------------------------------------
# METHOD : moveto - Set the h/v scrollbar to the percentage 
#                   indicated by what
#---------------------------------------------------------------

body UifListBox::moveto {what pos} {
   switch $what {
      h       {$itk_component(uifListBox) xview moveto $pos }
      v       {$itk_component(uifListBox) yview moveto $pos }
      default { puts "Invalid 'what' for method UifListBox::moveto" }
   }
}

#---------------------------------------------------------------
# METHOD : setBind 
#---------------------------------------------------------------

body UifListBox::setBind {key action} {
   
#    puts "DBG> OLD  bindtags listBox [bindtags $itk_component(uifListBox)]"
    bind ${this}${key} $key "$action" 

    # Add new bindtag ... if it does not already appear on the list

    set tmp [ bindtags $itk_component(uifListBox)]

    if {[lsearch $tmp ${this}${key}] == -1 } {
       bindtags $itk_component(uifListBox) \
	     [linsert [bindtags $itk_component(uifListBox)] end ${this}${key}]
    }
#    puts "DBG> NEW  bindtags listBox [bindtags $itk_component(uifListBox)]"
}
  
  
#---------------------------------------------------------------
# METHOD : getSelection
#---------------------------------------------------------------

#body UifListBox::getSelection {} {
#   set tmp ""
#   if { [set index [$list curselection]] != "" } {
#      foreach item $index {
#         lappend tmp [$list get $item]
#      }
#   }
#   return $tmp
#}

     
body UifListBox::getSelection {} {
   set index [$itk_component(uifListBox) curselection]

   if {[cequal $index ""] } { return "" }

   if {[cequal [$this cget -selectmode] single]} {
       return [$itk_component(uifListBox) get $index]
   } else {
       foreach item $index {
          lappend tmp [$itk_component(uifListBox) get $item]
       }
       return $tmp
   }
}


#---------------------------------------------------------------
# METHOD : checkPos - If the position is not valid it returns 
#                     an empty string                
#---------------------------------------------------------------
           
body UifListBox::checkPos {pos} {
   if {[cequal $pos end] || $pos == 0 } { return $pos }
   if { ( $pos >= 0 && $pos < [$itk_component(uifListBox) size]) } { 
      return $pos 
   }
   return ""
}

#---------------------------------------------------------------
# METHOD : getItem
#---------------------------------------------------------------
           
body UifListBox::getItem {pos} {
   if {[cequal [checkPos $pos] ""]} { 
      return ""
   } else {
      return [$itk_component(uifListBox) get $pos] 
   } 
}

#---------------------------------------------------------------
# METHOD : getList
#---------------------------------------------------------------

body UifListBox::getList { {first 0} {last end} } { 
   return [$itk_component(uifListBox) get $first $last]  
}    
   
#---------------------------------------------------------------
# METHOD : deleteItem
#---------------------------------------------------------------
       
body UifListBox::deleteItem {first {last {}} } {
   if {![cequal $last ""]} {
      $itk_component(uifListBox) delete $first $last
   } else {
      $itk_component(uifListBox) delete $first 
   }        
}  
  
#---------------------------------------------------------------
# METHOD : clearSelection
#---------------------------------------------------------------
           
body UifListBox::clearSelection {} {
   $itk_component(uifListBox) selection clear 0 end
}       


#---------------------------------------------------------------
# METHOD : setSelection
#---------------------------------------------------------------
           
body UifListBox::setSelection { args } {

   if {[cequal $args ALL]} {

      # Select ALL elements only if '-selectmode' is not se to 'single'
      if {[cequal [$this cget -selectmode] single]} { return }
      $itk_component(uifListBox) selection set 0 end

   } else {

      # If the listBox has set the '-selectmode' to 'single', at most
      # one element can be active at the time.

      foreach i $args {
         if {[checkPos $i] != ""} { 
            if {[cequal [$this cget -selectmode] single]} {
               clearSelection  ;# clear previous elements
            }

            $itk_component(uifListBox) selection set $i
         }
      }
      $itk_component(uifListBox) see $i
   }
   
   if {![cequal $itk_option(-variable) ""]} {
      setVariable $itk_option(-variable)
   }
}    
       

# This is here for backward compatibility ..
body UifListBox::setCursor { pos } {
   setSelection $pos
}
 
#---------------------------------------------------------------
# METHOD : addItemList
#---------------------------------------------------------------
 
body UifListBox::addItemList { pos listOfItems } {

   if {[cequal [checkPos $pos] ""]} { return }

   if {[cequal $pos end] } {
      foreach item $listOfItems {
         $itk_component(uifListBox) insert end $item
      }
   } else {
      set len [llength $listOfItems]
       for { set i [expr {$len-1}]} {$i >=0} {incr i -1}  {
          $itk_component(uifListBox) insert $pos [lindex $listOfItems $i] 
      }  
   }                
} 
 
#---------------------------------------------------------------
# METHOD : updateList
#---------------------------------------------------------------

body UifListBox::updateList {array args} {
   upvar $array arrayName
   
   if { [llength $args] } { 
      set formatProc [lindex $args 0] 
   } else {
      set formatProc ""
   }
   writeElements [array names arrayName]  arrayName          
}   

#---------------------------------------------------------------
# METHOD : updateSortList
#---------------------------------------------------------------

body UifListBox::updateSortList {order type array args} {
   upvar $array arrayName

   if { [llength $args] } { 
      set formatProc [lindex $args 0] 
   } else {
      set formatProc ""
   }

   if {[cequal $order ""] } { set order "-increasing" }
   if {[cequal $type  ""] } { set type  "-ascii" }

   writeElements [lsort $order $type [array names arrayName]] arrayName 
}

#---------------------------------------------------------------
# METHOD : setVariable

# The statement list; is used to avoid complaints when the value
# contains special characters such as : "" and $.

#---------------------------------------------------------------
           
body UifListBox::setVariable { var } {
   uplevel #0 set $var [list [$this getSelection]]
} 
 
#---------------------------------------------------------------
# METHOD : writeElements
#---------------------------------------------------------------

body UifListBox::writeElements { nameList array } {
   upvar $array arrayName
   $itk_component(uifListBox) delete 0 end

   foreach name $nameList {
      if { $formatProc != "" } {   
         set newText    [ eval { $formatProc $arrayName($name) }]
         $itk_component(uifListBox)  insert end  $newText        
      } else {
         $itk_component(uifListBox)  insert end  $arrayName($name)
      }
   }
}

#---------------------------------------------------------------
# OPTION : -scrollx
#---------------------------------------------------------------  

configbody UifListBox::scrollx {    
   if { $itk_option(-scrollx) } {
      pack $itk_component(f2) -fill x
   } else {
      pack forget $itk_component(f2)   
   }
}

#---------------------------------------------------------------
# OPTION : -scrolly
#---------------------------------------------------------------  

configbody UifListBox::scrolly {
   if {$itk_option(-scrolly)} {
      pack $itk_component(scrollv) -side left -fill y
   } else {
      pack forget $itk_component(scrollv)
   }
}
   
#---------------------------------------------------------------
# OPTION : -geometry
#---------------------------------------------------------------  

configbody UifListBox::geometry {
   if {[scan $itk_option(-geometry) "%dx%d" w h] > 0} {
      $itk_component(uifListBox) configure -width $w -height $h
   }
}
 
#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------  

configbody UifListBox::variable {
   if { [cequal $itk_option(-variable) ""] } {
      if {$bindFlag} {
          setBind <B1-ButtonRelease> "" 
	  bind $itk_component(uifListBox) <Return> {}
      }
   } else {

       # Added to set the variable when using default bindings to select
       # a list element

       bind $itk_component(uifListBox) <Return> \
            "$wthis setVariable $itk_option(-variable)"

       setBind <B1-ButtonRelease> "$wthis setVariable $itk_option(-variable)" 
       set bindFlag 1
   }
}     

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------  

configbody UifListBox::label {

   if {[cequal $itk_option(-label) ""] } {
      if {$packLabel} { 
         pack forget $itk_component(label)
         set packLabel 0
      }
   } else {
      $itk_component(label) configure -text $itk_option(-label)

      if { $packLabel == 0 } {
         pack $itk_component(label) -before $itk_component(f1) -fill x
         set packLabel 1
      }
   }
} 
 
#---------------------------------------------------------------
# OPTION : -itemList
#---------------------------------------------------------------  

configbody UifListBox::itemList {
   $itk_component(uifListBox) delete 0 end

   foreach item $itk_option(-itemList) {
       $itk_component(uifListBox) insert end $item 
   }
}


#___oOo___



