#
# E.S.O. - VLT project
#
# "@(#) $Id: uifMenuButton.tcl,v 3.36 2003/01/20 10:33:47 vltsccm Exp $" 
#
#
# uifMenuButton.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    07/04/95   created 
# M.COMIN    03/05/95   Add getSize method
# M.COMIN    16/10/95   Add global enable/disable mechanism
#
# mcomin     6/03/96    Porting to Tk4.0 and itcl2.0
#            15/05/96   Update man page
#
#************************************************************************
#   NAME
#     UifMenuButton - Class for Menu Buttons
#   SYNOPSIS
#     UifMenuButton <widget-name>
#   DESCRIPTION
#     UifMenuButton defines the class for the VLT menu buttons.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifMenuButton - Menu button widget
#   METHODS
#
#     The methods use the following parameters :
#
#     <label>     String to label the entry
#     <uline>     Character to be used as 'accelerator key' :
#                   - 1  identifies the first character
#                   - 0  No characters are underlined
#     <command>   TCL command to be executed
#     <var>       Variable to associate to the Check/Radio Button
#     <value>     Value assigned to <var>       
#
#     The class supports the following methods :
#
#     getSize
#        Returns the total number of elements in the Menu.
#
#     addSubMenu     <name>  <label> <uline>
#
#     addCommand     <label> <uline> <command>
#        Add an entry allowing the user to invoke a TCL command and behaves 
#        like a button widget.
#
#     addRadioButton <label> <uline> <command> <var> <value>
#        Add an entry configured as RadioButton. 
#        Related entries share the same variable <var>
#
#     addCheckButton <label> <uline> <command> <var>
#        Add an entry configured as CheckButton. 
#        The button assigns values 0/1 to the associated variable <var>
#
#     addSeparator 
#        Add an entry configured as separator.
#        This is used as graphic decoration to group entries together.
#
#     The next set of methods implement functions on a generic entry.
#
#     General Remark :
#     - Entries start with index 0
#     - Keyword "last" always identifies the last entry of the menu
#
#     deleteEntry  <from> <to>
#        Delete menu items from position <from> to position <to> 
#      
#     enableEntry  <pos>
#        Enable selection of entry in position <pos>
#
#     disableEntry <pos> 
#        Disable selection of entry in position <pos>
#
#     getMenuName 
#        Get full name of the menu associated to the widget.
#  
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifWidget class and supports :
#
#     label     <string>    String used to label the button.
#     width     <number>    Widget length
#     tearoff   <1/0>       Enable/Disable the tearoff capability.
#                           When enabled the menu associated to the menubutton
#                           remains 'opened' until the user selects an item.
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifMenuButton { }

class UifMenuButton {
    inherit  UifWidget

    constructor {args} {}

    itk_option define -label   label   Label   ""
    itk_option define -tearoff tearoff Tearoff 0
    
    public method addSubMenu { name label { uline {0}} }
    public method addCommand { label uline command }
    public method addRadioButton { label uline command {var {a}} {value {0}} }
    public method addCheckButton { label uline command {var {}} }
    public method addSeparator {}

    #*# method: getSize

    public method getSize { } {
       return [$menu index last]
    }

    #*# method: deleteEntry

    public method deleteEntry { from {to {last}} } { 
       $menu delete $from $to 
    }

    #*# method: enableEntry

    public method enableEntry { pos } { 
       $menu entryconfigure $pos -state normal
    }

    #*# method: disableEntry

    public method disableEntry { pos } { 
       $menu entryconfigure $pos -state disabled
    }

    #*# method: disable

    public method disable {} { 
       $me configure -state disabled
    }

    #*# method: enable

    public method enable {} { 
       $me configure -state normal 
    }

    #*# method: getMenuName

    public method getMenuName { } { 
       return $menu 
    }

    protected variable menu         "" 
    protected variable me           ""
    protected variable listOfMenu   {}
    protected variable maxEntry     0    

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifMenuButton::constructor { args } {

   itk_component add uifMenuButton {
       set me $itk_interior.uifMenuButton
       menubutton $me -menu $me.menu -bd 2 -text ""
   } {
       keep -width -text -relief -foreground
   }

   set menu [menu $me.menu] ; $menu configure -tearoff 0
   
   pack $itk_component(uifMenuButton)

   eval itk_initialize 
   
   configure -relief raised
     
   set initialized 1
   if {[llength $args] != 0} { eval configure $args }
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------

configbody UifMenuButton::label {
   $itk_component(uifMenuButton) configure -text $itk_option(-label)
}

#---------------------------------------------------------------
# OPTION : -tearoff
#---------------------------------------------------------------

configbody UifMenuButton::tearoff {
   $menu configure -tearoff $itk_option(-tearoff)
}

#--------------------------------------------------------------
# METHOD : addSubMenu 
#--------------------------------------------------------------

body UifMenuButton::addSubMenu { name label { uline {0}} } {

   if { $name == "" } {
      regexp -nocase {([a-z]*[0-9]*)} $label name
      set name    [string tolower $name]
   }
       
   set myMenu  "$me.$name"
   lappend listOfMenu $myMenu

   set param   "-text \"$label\" " 
  
   if {$uline != 0} { incr uline -1; append param "-underline $uline" }  
     
   eval menubutton $myMenu -menu $myMenu.menu  $param
                                   
   pack $menuBar.$name 
   incr maxEntry                            
                                                                         
   return [menu $myMenu.menu]
}

#--------------------------------------------------------------
# METHOD : addCommand 
#--------------------------------------------------------------

body UifMenuButton::addCommand { label uline cmd } {

   set param "add command -label \"$label\" "
  
   if { $uline != 0 } {incr uline -1; append param "-underline $uline" }  
          
   eval $menu  $param -command {$cmd} 
}

#--------------------------------------------------------------
# METHOD : addRadioButton 
#--------------------------------------------------------------

body UifMenuButton::addRadioButton { label uline cmd {var {a}} {value {0}} } {

   set param   "add radiobutton -label \"$label\" "

   if {$var   != ""}  { append param " -variable $var "}
   if { $uline != 0 } {incr uline -1; append param "-underline $uline" }  
          
   eval $menu  $param -value $value -command {$cmd} 
}

#--------------------------------------------------------------
# METHOD : addCheckButton
#--------------------------------------------------------------

body UifMenuButton::addCheckButton { label uline cmd  {var {}} } {

   set param   "add checkbutton -label \"$label\" "
    
   if { $var != ""}   { append param " -variable $var "} 
   if { $uline != 0 } {incr uline -1; append param "-underline $uline" }  
          
   eval $menu $param -command {$cmd}
}

#--------------------------------------------------------------
# METHOD : addSeparator
#--------------------------------------------------------------

body  UifMenuButton::addSeparator { }  {
     $menu add  separator; incr maxEntry 
}

#___oOo___





