#
# E.S.O. - VLT project
#
# "@(#) $Id: uifOptionButton.tcl,v 3.36 2003/01/20 10:33:53 vltsccm Exp $" 
#
#
# uifOptionButton.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    10/04/95   created 
# M.COMIN    15/05/95   Fixed bug with infinite loop in method command
#                       Fixed bug with label made of several words
# M.COMIN    16/08/95   Fixed bug with default value
# M.COMIN    15/09/95   Modified as follows  ...  cmd "$command [list $lbl]"
# M.COMIN    16/10/95   Add global enable/disable mechanism
#                       Add internal method setvalue
# M.COMIN    15/11/95   Fix bug with label
#
# mcomin     6/03/96    Porting to Tk4.0 and itcl2.0
#            07/05/96   Re-implementation using the -indicatoron configuration
#                       option of menubuttons.
#                       Inherit now from uifInteractiveWidget
#            15/05/96   Update man page
#            15/01/97   Fixed bug when configuring options -command, 
#                       -variable, -itemList in different combinations
#
#************************************************************************
#   NAME
#     UifOptionButton - Class implementing the option button MOTIF widget.
#   SYNOPSIS
#     UifOptionButton <widget-name>
#   DESCRIPTION
#     UifOptionButton defines the class for the implementation of the 
#     MOTIF option button. The label shows the last selected option.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifOptionButton - Option button widget
#   METHODS
#
#     Inherits the methods of class UifInteractiveWidget and support the 
#     following :
#
#     General Remark :
#     - Entries start with index 0
#     - Keyword "last" always identifies the last entry of the menu
#
#     addItem  <label> ?proc?
#        Add an entry allowing the user to invoke a TCL command.
#     
#        - <label>     String to label the entry
#        - ?proc?      Optional parameter specifyimg a TCL procedure 
#                      to be executed.
#
#                      This method allows the user to call different
#                      procedures for each selected item.
#
#     deleteEntry  <from> <to>
#        Delete menu items from position <from> to position <to> 
#      
#     enableEntry  <pos>
#        Enable selection of entry in position <pos>
#
#     disableEntry <pos> 
#        Disable selection of entry in position <pos>
#
#     getMenuName 
#        Get full name of the menu associated to the widget.
#
#     getSize                  
#        Returns the total number of options
#  
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifWidget class and supports :
#
#     command     <proc. name>  Name of a procedure to invoke when the user
#                               selects an option. The value of the selected
#                               option is appended to the list of the other
#                               arguments given to <proc. name>.
#
#                               BEWARE : The procedure <proc. name> must
#                               have at least one argument.
#
#     default     <value>       Default value when the widget is created
#     font        <font name>   Text Font 
#     itemList    <list>        List of options.
#     itemValue   <list>        List of values associated to each option.
#                               If this is empty the position in the list 
#                               of the selected option is taken as value.
#                               This is choosen when option -selectMode is set 
#                               to 'value'.
#     length      <number>      Specifies the widget length.
#     selectMode  label/value   Specifies how to set the variable associated
#                               to the option button.
#     variable    <va. name>    Name of the variable associated to the widget
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifOptionButton .w -itemList {item1 item3 item5} -default item5
#                        -command puts  -variable gvar(a) -length 10
#
#     Re-configure the widget
#       .w configure -command  myProcedure
#       .w configure -variable gvar(b) -itemList { a b c }
#
#     Other operations
#       set myVar      [.out cget -variable]    Get a widget's resource
#       set entryList  [.out cget -itemList]
#
#    Example how to selecet a device to be initialized
#       .w addItem DEV1 "initDEV1"
#       .w addItem DEV2 "initDEV2"
#
#    Example of procedure : the procedure writes on the screen what type
#    of number the user has selected.
# 
#       .w configure -command test -itemList {-3 5 8 0 -35 }
#
#       proc test { val } {
#          if {$val == 0 } {
#             puts "Null Value"
#          } elseif {$val < 0 } {
#             puts "Negative Value"
#          } else {
#             puts "Negative Value"
#          }
#       }
#   SEE ALSO
#     UifIntercativeWidget  UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifOptionButton { }

class UifOptionButton {
    inherit UifInteractiveWidget

    constructor {args} {}

    destructor {
       if {![cequal $oldVar ""] } {
          uplevel #0 trace vdelete [list $oldVar] w  \"$wthis varTrace\"
       }
    }

    itk_option define -variable variable Variable  ""
    itk_option define -command  command  Command   ""
    itk_option define -itemList itemList ItemList  ""
    itk_option define -itemValue itemValue ItemValue ""
    itk_option define -selectMode selectMode SelectMode label

    public method configure { {option ""} args }
    public method addItem  { label cmd }
    public method setValue { label }
    public method varTrace {name elem op}

    #*# method: getSize

    public method getSize { } { 
       return [$menu index last]
    }

    #*# method: deleteEntry

    public method deleteEntry { from {to {last}} } { 
       $menu delete $from $to 
    }

    #*# method: enableEntry

    public method enableEntry { pos } { 
       $menu entryconfigure $pos -state normal
    }

    #*# method: disableEntry

    public method disableEntry { pos } { 
       $menu entryconfigure $pos -state disabled
    }

    #*# method: getMenuName

    public method getMenuName {} { return $menu }

    private method configMenu {}

    protected variable menu     ""
    protected variable frame_fg ""
    protected variable interactiveFlag 0
    protected variable oldVar   ""

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifOptionButton::constructor { args } {

   itk_component add uifOptionButton {
       menubutton $itk_interior.uifOptionButton \
                  -menu $itk_interior.uifOptionButton.menu \
                  -relief raised -text "" -padx 8 \
                  -indicatoron 1 -borderwidth 2  \
                  -textvariable ${wthis}_optVar     
   } {
       rename -width -length length Length

       keep -text -font -state -foreground -background -activeforeground \
            -activebackground
   }

   pack $itk_component(uifOptionButton) -side left -fill both

   set menu  [menu $itk_interior.uifOptionButton.menu -tearoff 0 ]

   eval itk_initialize

   if {[set idx [lsearch $args "-selectMode"]] != -1} {
       configure -selectMode [lindex $args [expr {$idx +1}]]
       set args [lreplace $args $idx [expr {$idx +1}]]
   }

   if {[set idx [lsearch $args "-itemList"]] != -1} {
      configure -itemList [lindex $args [expr {$idx +1}]]
      set args [lreplace $args $idx [expr {$idx +1}]]
   }

   set initialized 1
   eval configure $args
}

#----------------------------------------------------------------------
# METHOD : configure
# The method configure has been re-defined because of the need to execute
# the method configMenu at the end of the whole  configuration process in 
# order not to repeat the execution several times
#
# Parameter 'ret' returns the current configuration when 'configure' is
# called without arguments
#
#----------------------------------------------------------------------

body UifOptionButton::configure { {option ""} args } {
#   puts "\nUifOptionButton::configure   init = $initialized  $option $args"
   set ret [eval itk::Archetype::configure $option $args]
   if {$initialized} { configMenu }
   return $ret
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody UifOptionButton::variable {

   if {![cequal $itk_option(-variable) ""]} {
      # Declare callback function to keep widget updated
      #   puts "DBG > trace variable $var w \"$wthis varTrace \" "

      uplevel #0 trace variable [list $itk_option(-variable)] w \"$wthis varTrace\"
   }

   if {![cequal $oldVar ""] && ![cequal $itk_option(-variable) $oldVar]} {
       uplevel #0 trace vdelete [list $oldVar] w  \"$wthis varTrace\"
   }

   if {![cequal [set oldVar [list $itk_option(-variable)]] ""]} {
       if {[uplevel #0 info exists [list $oldVar]]} {
          setValue [uplevel #0 set [list $oldVar]]
       } else {
          setValue $itk_option(-default)
       }
   }
}

#---------------------------------------------------------------
# OPTION : -command
#---------------------------------------------------------------
#
#configbody UifOptionButton::command {
#   if {$initialized } { 
#      configMenu 
#   }
#}

#---------------------------------------------------------------
# OPTION : -itemList
#---------------------------------------------------------------

configbody UifOptionButton::itemList {
    $menu delete 0 end
    foreach item $itk_option(-itemList) {
       $menu add command -label "$item"
    }

#    if {$initialized } { configMenu }
}

#---------------------------------------------------------------
# METHOD : varTrace
#---------------------------------------------------------------

body UifOptionButton::varTrace { name elem  op } {

   set interactiveFlag 0
   if {$interactiveFlag == 1} { return }

   if { [cequal $elem ""] } {
      set value [uplevel #0 set [list ${name}]]
   } else {
      set value [uplevel #0 set [list ${name}]($elem)]
   }

   if {[cequal $itk_option(-selectMode) value]} {
      if { [lempty $itk_option(-itemValue)] } {
         set value [lindex $itk_option(-itemList) $value]
      } else {
         if {[set idx [lsearch $itk_option(-itemValue) $value]] == -1} {
             set value "Wrong Value"
         } else {
             set value [lindex $itk_option(-itemList) $idx]
         }
      }
   }

   uplevel #0 set ${wthis}_optVar \"$value\"
}


#---------------------------------------------------------------
# METHOD : setValue
# Checks if the new  value is an element of the list.
# The new values is changed according to the -selectMode.
#---------------------------------------------------------------

body UifOptionButton::setValue { label } {
    set interactiveFlag 1
    if {[cequal $itk_option(-variable) ""] } { return }

    if { [set idx [lsearch $itk_option(-itemList) $label]] == -1 } { return }

    uplevel #0 set ${wthis}_optVar \"$label\"

    if {[cequal $itk_option(-selectMode) value]} {
        if {[lempty $itk_option(-itemValue)] } {
            uplevel #0 set [list $itk_option(-variable)] $idx
        } else {
            set tmp [lindex $itk_option(-itemValue) $idx]
            uplevel #0 set [list $itk_option(-variable)] \"$tmp\"
        }
    } else {
        uplevel #0 set [list $itk_option(-variable)] \"$label\"
    }
}

#---------------------------------------------------------------
# METHOD : addItem
#---------------------------------------------------------------

body UifOptionButton::addItem { label cmd } {
   $menu add command -label $label
   if {![cequal $cmd ""]} { append cmd "; "}
   append cmd "$this setValue \"$label\""
   eval $menu entryconfigure last -command {$cmd}
}

#---------------------------------------------------------------
# METHOD : configMenu
#---------------------------------------------------------------

body UifOptionButton::configMenu {} {
   if {[set maxId  [$menu index last]] == "none"} { return }

   for {set i 0} { $i <= $maxId } { incr i } {
       set lbl [lindex [$menu entryconfigure $i -label] 4]
       set cmd ""
       set tmp [string trim $lbl "\{\}"]

       if { ! [cequal $itk_option(-variable) ""] } {
          append cmd "$this setValue \"$tmp\" ; "
       }

       if {! [cequal $itk_option(-command) ""]} {

          if {[cequal $itk_option(-selectMode) value]} {
             if {[lempty $itk_option(-itemValue)] } {
                set tmpVal $i
             } else {
                set tmpVal [lindex $itk_option(-itemValue) $i]
             }
          } else {
             set tmpVal $tmp
          }

         append cmd "$itk_option(-command) \"$tmpVal\""
       }

       eval $menu entryconfigure $i -command {$cmd}
   }
}

#___oOo___



