#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifOutput.tcl,v 3.36 2003/01/20 10:33:48 vltsccm Exp $" 
#
# uifOutput.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# mcomin     07/11/94   Add methods and documentation
# mcomin     14/02/95   Improved class
# mcomin     15/02/95   Add public variable format
# mcomin     27/04/95   Took away publ. var. length
# mcomin     31/07/95   Use args instead of config
# mcomin     05/10/95   Add public variables for the event mechanism
# -------------------
# mcomin     09/03/96   Ported to Tk4.0 and itcl 2.0
#            13/05/96   Update man page
#            14/01/97   Add -units option
#            12/12/97   Add options -label, -labelWidth, -labelFont
#
#************************************************************************
#   NAME
#     UifOutput - Class for VLT output widget
#   SYNOPSIS
#     UifOutput  <widget-name>
#   DESCRIPTION
#     UifOutput defines the class for the VLT output widget which allows the 
#     user to display values.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifOutput - Label widget
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifOutputWidget class and supports :
#
#     anchor     <value>    Determine where the value is displayed within
#                           the widget -.e.g.- centered, left or right 
#                           justified, all tcl/tk values for 'anchor' are valid.
#     color      <string>   Text Color
#     font       <string>   Text Font 
#     frame      <1/0>      Switch on/off the frame around the widget.
#     label      <string>   String used to label the widget
#     labelFont  <string>   New font for the label
#     labelWidth <integer>  Label width : can be used to align several
#                           widgets with different labels in columns
#     units      <string>   Text appended at the right side describing the
#                           nature of the displayed value.
#     variable   <string>   Name of the variable associated to the widget 
#     width      <integer>  Widget length
#   CAUTIONS
#
#   EXAMPLES
#
#    Create an instance
#
#    UifOutput .w -variable gvar(:PARAMS:SCALARS.scalar_string20) \
#                 -color Red -frame 1 -width 21 -font CourR14
#
#    Re-configure the widget
#       .w configure -frame 0  -font helvB24 -color green
#
#     Other operations
#       set myVar    [.w cget -variable]   Get a widget's resource
#       set myColor  [.w cget -color]
#   SEE ALSO
#     UifOutputWidget UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifOutput { }

class UifOutput {
    inherit UifOutputWidget

    constructor {args} {}    

    itk_option define -frame    frame    Frame     1
    itk_option define -variable variable Variable  ""
    itk_option define -units    units    Units     ""
    itk_option define -label    label    Label     ""

}; #end of class definition


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifOutput::constructor {args} {
   itk_component add uifOutput {
      label $itk_interior.uifOutput -bd 2 
   } { 
      keep -width -font -textvariable -anchor
      keep -anchor -background -foreground -justify
      rename -foreground -color color Color
   }

   itk_component add units {
      label $itk_interior.units -anchor w -relief flat
   } {
     keep  -text -background
     rename -font  -labelFont labelFont LabelFont
   }

   itk_component add label {
      label $itk_interior.label -anchor w -relief flat
   } {
     keep -background
     rename -width -labelWidth labelWidth LabelWidth
     rename -font  -labelFont labelFont LabelFont
   }

   pack $itk_component(uifOutput) $itk_component(units) -side left -fill x
 
   itk_initialize
   eval configure $args
}


#---------------------------------------------------------------
# OPTION : -frame
#---------------------------------------------------------------

configbody UifOutput::frame {
   if { $itk_option(-frame) == 1 } {
      $itk_component(uifOutput) configure -relief ridge
   } else {
      $itk_component(uifOutput) configure -relief flat
   }
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------

configbody UifOutput::label {
    if {[cequal $itk_option(-label) ""] } {
       pack forget $itk_component(label)
    } else {
       $itk_component(label) configure -text $itk_option(-label)
       pack $itk_component(label) -side left -before $itk_component(uifOutput)
    }
}

#---------------------------------------------------------------
# OPTION : -units
#---------------------------------------------------------------

configbody UifOutput::units {
    if {[cequal $itk_option(-units) "" ]} {
       pack forget $itk_component(units)
    } else {
       $itk_component(units) configure -text "($itk_option(-units))"
       pack $itk_component(units) -side left -padx 3 \
            -after $itk_component(uifOutput)
    }
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody UifOutput::variable {
   addToDataList $itk_option(-variable) 
   $itk_component(uifOutput) configure -textvariable $itk_option(-variable)
} 
