#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifOutputArray.tcl,v 3.36 2003/01/20 10:34:02 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    19/12/97  created
#           21/04/98  Label configured with proper anchor parameter
#                     according to the orientation
#           23/04/98  Catch error of formatting : in such a case just
#                     output the original value
#

#************************************************************************
#   NAME
#      UifOutputArray - Displays the contents of arrays or tables rows/columns
#   SYNOPSIS
#      UifOutputArray <widget-name>
#   DESCRIPTION
#      This widgets is able to display the contents of arrays or table 
#      rows/coulumns. The widget is formed by a label and a sequence of 
#      UifOutput elements each showing a single element of the array or table.
#
#      The elements can be configured to be horizontally or vertically alligned.
#
#   COMPONENTS
#      The itk 'component' method returns the component's full name.
#
#      topFrame - Frame containing the label and the arrayBox
#      label    - Widget's label.
#      arrayBox - Frame containing the UifOutput widgets.
#   METHODS
#
#   getCellVariable <pos>
#      Return the full name of the UifOutput widget for a given position.
#      <pos> ranges between 0 and (size - 1).
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#      Supports the following options :
#
#      anchor     <string>  Determine where the value should be displayed
#                           within the cell -.e.g.- centered, left or right 
#                           justified, all tcl/tk values for 'anchor' are valid.
#      color      <string>  Text Color
#      font       <string>  Text Font
#      format     <string>  Format to used to display the value.
#                           (printf like syntax).
#      frame      <1/0>     Switch on/off the frame around each cell. 
#      label      <string>  String used to label the widget
#      labelFont  <string>  New font for the label
#      labelWidth <string>  Label width : can be used to align several
#                           widgets with different labels, in particular 
#                           when the cells layout is horizontal.
#      orient     <h/v>     Layout configuration : horizontal/vertical
#      padding    <integer> Determines the distance between cells.
#      size       <integer> Number of cells.
#      variable   <string>  Name of the variable associated to the widget
#      width      <integer> Cell length
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#    Create an instance
#    UifOutputArray .w -anchor w -dbMode dbPoll -frame 1 \
#                    -label {Output Array} -orient h -padding 12 \
#                    -variable gvar(VECTORS.vector_double(0:2)) \
#                    -size 3 -width 10 -format %4.1f 
#
#    Re-configure the widget
#       .w configure -frame 0  -font helvB24 -color green
#
#    Other operations
#       set myVar    [.w cget -variable]   Get a widget's resource
#       set myColor  [.w cget -color]
#   SEE ALSO
#     UifOutputWidget UifWidget UifOutputString UifOutput UifOutputLed
#   BUGS     
#
#------------------------------------------------------------------------
#
itk::usual UifOutputArray { }

class UifOutputArray {
    inherit UifOutputWidget

    constructor {args} {}
    destructor {}

    itk_option define -anchor    anchor    Anchor    center    
    itk_option define -color     color     Color     Black
    itk_option define -hideCells hideCells HideCells {}
    itk_option define -width  width    Width   5
    itk_option define -font  font   Font -Adobe-courier-medium-r-normal--14*
    itk_option define -format format Format ""    
    itk_option define -frame  frame    Frame   1
    itk_option define -label  label    Label   ""
    itk_option define -variable variable Variable ""
    itk_option define -orient   orient   Orient   v
    itk_option define -size     size     Size     1
    itk_option define -padding  padding  Padding  1

      
    public method setValue {args} 
    public method configure { {option ""} args }
    public method getCellVariable { pos }

    private method packUifOutputArray {}
    private method setPackConfiguration { value }
    private method setWidgetConfiguration {}
    private method parseRtapIndex { what }

    protected variable oldVar     ""
    protected variable newPacking 1
    protected variable packSide   top
    protected variable padx       1
    protected variable pady       1
    protected variable rec0       ""
    protected variable rec1       ""   
    protected variable field0     ""
    protected variable field1     ""
    protected variable whatFlag   ""
    protected variable elemName   "" ;# Just the name of the attribute
    protected variable arrayName  ""
    protected variable attrName   ""
    protected variable attrRange  ""

}; # end of class

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifOutputArray::constructor { args } {
   itk_component add topFrame {
     frame $itk_interior.topFrame
   }
   pack $itk_component(topFrame) -fill both

   itk_component add arrayBox {
     frame $itk_component(topFrame).arrayBox
   }

   itk_component add label {
     label $itk_component(topFrame).label -text ""
   } {
     rename -width -labelWidth labelWidth LabelWidth
     rename -font  -labelFont labelFont LabelFont
   }

   eval itk_initialize
   set initialized 1

   #  Explicitly handle config's that may have been ignored earlier
   
   eval configure $args
}

#--------------------------------------------------------------
# Destructor
#--------------------------------------------------------------
body UifOutputArray::destructor {} {
   if {! [cequal $oldVar ""]} { varTraceOff }  
}

#--------------------------------------------------------------
# METHOD : configure
# The method configure has been re-defined because of the need to
# execute the method setSize and setLength at the end of the whole 
# configuration process in order not to repeat the execution several times
#
# Parameter 'ret' returns the current configuration when 'configure' is
# called without arguments
#
#--------------------------------------------------------------

body UifOutputArray::configure { {option ""} args } {
   set ret [eval itk::Archetype::configure $option $args]

   setPackConfiguration $itk_option(-orient)
   setWidgetConfiguration
   packUifOutputArray
   return $ret
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody UifOutputArray::variable {

    if {!$initialized } { return }

#    puts "UifOutputArray::variable  $itk_option(-variable)"

    # Extract element of TABLE/VECTOR to be read.

    set whatFlag ""; set attrRange "" ; set attrName ""
    set rec0 "" ;  set rec1 "" ; set field0 ""; set field1 ""

    # Check if this is an array or not

    if {[regexp -nocase {([a-z0-9]+)\((.*)\)} $itk_option(-variable) all arrayName elemName] == 0} {
       set arrayName ""
       set elemName  $itk_option(-variable)
    } else {

       # Name of the variable can be :
       # array(xxx) or array(xxx(yyy))

       set ret [regexp -nocase {([\.\:a-z0-9]+)\((.*)\)} $elemName all attrName attrRange] 

       if {$ret == 0} { set attrName $elemName ; set attrRange "" }
    }
      
    if {[regexp {([\:0-9]*)([,]*)([\:0-9]*)} $attrRange all recs tmp fields] == 0} {
       set recs   ""
       set fields ""
    }

    if {![cequal $recs ""]}   {lassign [parseRtapIndex $recs] rec0 rec1}
    if {![cequal $fields ""]} {lassign [parseRtapIndex $fields] field0 field1}

#    puts "UifOutputArray::variable  whatFlag = $whatFlag attrName = $attrName  recs $rec0 : $rec1 , $field0 : $field1"

    setTraceVar   $itk_option(-variable)
    addToDataList $itk_option(-variable)
}

#---------------------------------------------------------------
# OPTION : -orient
#---------------------------------------------------------------
configbody UifOutputArray::orient {
   set newPacking 1
}

#---------------------------------------------------------------
# OPTION : -padding
#---------------------------------------------------------------
configbody UifOutputArray::padding {
   set newPacking 1
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------
configbody UifOutputArray::label {
   if {!$initialized} { return }
   set newPacking 1
   $itk_component(label) configure -text $itk_option(-label)
}


#---------------------------------------------------------------
# OPTION : -size
#---------------------------------------------------------------
configbody UifOutputArray::size {
   if {$itk_option(-size) == 0 } { return }

   # Delete exiting components
   foreach w [winfo children $itk_component(arrayBox)] {
     destroy $w
   }

   for {set i 0} {$i < $itk_option(-size) } {incr i} {
       label $itk_component(arrayBox).cell${i} -width 5 -bd 2
       pack $itk_component(arrayBox).cell${i} -side $packSide
   }
}

#---------------------------------------------------------------
# METHOD : parseRtapIndex
#---------------------------------------------------------------          
body UifOutputArray::parseRtapIndex { what } {
   regexp {([0-9]+)([:]*)([0-9]*)} $what all i1 tmp i2

   if {[cequal $i2 ""]} {
      return "$i1 $i1"
   } else {
      return "$i1 $i2"
   }
} 
#---------------------------------------------------------------
# METHOD : getCellVariable
#---------------------------------------------------------------        
body UifOutputArray::getCellVariable { pos } {
   if {$pos >=0 && $pos < $itk_option(-size)} {
      return [$itk_component(arrayBox).cell${pos} cget -textvariable]
   } else {
      return ""
   }
} 

#---------------------------------------------------------------
# METHOD : setValue
#          This method sets the current value for all th widget cells.
#          The parameter passed to the method must be a list of values.
#---------------------------------------------------------------
body UifOutputArray::setValue {listOfValues} {

   if {$initialized == 0} { return }

   set args [eval list $listOfValues]
   if {[cequal $args ""] } { return }

   if {[set len [llength $args]] > $itk_option(-size)} { 
      set len $itk_option(-size)
   }

   for {set i 0} {$i < $len} {incr i} {
       set var [$itk_component(arrayBox).cell${i} cget -textvariable]
       if {![cequal $var ""]} {
           if {[cequal $itk_option(-format) ""]} {
              set value [lindex $args $i]
	   } else {

              # Catch if format conversion cannot be done

              set origVal [lindex $args $i]
              if {[catch {set value [format "$itk_option(-format)" $origVal]} ]} {
                 set value $origVal
              }
           }
	   uplevel #0 set $var \"${value}\"
       }
   }
} 

#---------------------------------------------------------------
# METHOD : setPackConfiguration
#---------------------------------------------------------------  
body UifOutputArray::setPackConfiguration { value } {
   if {[cequal $value v]} {
      set packSide top
      set padx 1
      set pady $itk_option(-padding)
   } else {
      set packSide left
      set pady 1
      set padx $itk_option(-padding)
   }
}

#---------------------------------------------------------------
# METHOD : setWidgetConfiguration
#          This mehod is used to configure the uifOutput widgets.
#---------------------------------------------------------------
body UifOutputArray::setWidgetConfiguration {} {

#   puts "setWidgetConfiguration ($arrayName) ($elemName) ($attrName) ($attrRange)"

   set childList [winfo children $itk_component(arrayBox)]

   foreach w $childList {
       $w configure -relief ridge -width $itk_option(-width) \
                    -font $itk_option(-font) -anchor $itk_option(-anchor)\
                    -foreground $itk_option(-color)

       if {$itk_option(-frame) == 0} { $w configure -relief flat }
   }

   # Find what kind of addressing we are using.

   if {[cequal $attrRange ""]} {
      set whatFlag tclVar
   } else {

      if {[cequal $field0 ""] && [cequal $field1 ""]} {
         set whatFlag vector
      } elseif { ![cequal $field0 ""] && [cequal $field0 $field1] } {
         set whatFlag tableRecord
      } else {
         set whatFlag tableField
      }
   }

   set i 0
   foreach w $childList {
      
      switch $whatFlag {
        vector      {set var "${arrayName}\(${attrName}\([expr {$rec0 + $i}]\)\)"}
        tableRecord {set var "${arrayName}\(${attrName}\([expr {$rec0 + $i}],${field0}\)\)"}
        tableField  {set var "${arrayName}\(${attrName}\(${rec0},[expr {${field0} + $i}]\)\)"}
        default     {
	            if {[cequal $arrayName ""]} {
                       set var "${elemName}_cell\($i\)"
                    } else {
			set var "${arrayName}\(${elemName}_cell\($i\)\)"
                    }
        }
      } ;# end of switch ...

#      puts "$w configure -textvariable  $var"
      $w configure -textvariable $var
      incr i
   }
}

#---------------------------------------------------------------
# METHOD : packUifOutputArray
#---------------------------------------------------------------
body UifOutputArray::packUifOutputArray {} {

   if {$newPacking == 0 } { return }

   set newPacking 0 ;# reset packing flag

   # un-pack widgets
   foreach w [winfo children $itk_component(arrayBox)] {
     pack forget $w
   }

   pack forget $itk_component(label) $itk_component(arrayBox)

   # Pack widgets with the new configuration
   foreach w [winfo children $itk_component(arrayBox)] {
     pack $w -side $packSide -padx $padx -pady $pady
   }

#   puts "UifOutputArray::packUifOutputArray $padx $pady"
   if {![cequal $itk_option(-label) ""]} {
      pack $itk_component(label) -side $packSide -padx $padx -pady $pady
   }
   
   if {[cequal $packSide left]} {
      $itk_component(label) configure -anchor w
   } else {
      $itk_component(label) configure -anchor center
   }

   pack $itk_component(arrayBox) -side $packSide -padx $padx -pady $pady
}





#
# ___oOo___
