#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifOutputLed.tcl,v 3.36 2003/01/20 10:34:03 vltsccm Exp $" 
#
# uifOutputLed.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    17/12/97   created 
#
#************************************************************************
#   NAME
#     UifOutputLed - Displays the ON/OFF status by means of a LED widget
#   SYNOPSIS
#     UifOutputLed  <widget-name>
#   DESCRIPTION
#     The widget has two states where the value 0 is associated
#     with the OFF level and any other value switches the Led to ON.
#
#     The widget reacts to the value of the associated variable according 
#     configuration parameters "flashing". See description later.
#
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifOutputLed - LED widget.
#
#   METHODS
#
#     flash
#       This method makes turns the LED ON for a time according to the
#       -flashTime option and after that turns off the LED
#
#     turnOff
#       Turns the LED off with the color according to the current values of
#       the '-colorOff' option.       
#
#     turnOn
#       Turns the LED on with the color according to the current values of
#       the '-colorOn' option.
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Inherits the following options from UifLed :
#
#     colorOn   <string>       Color to display when the LED is ON.
#     colorOff  <string>       Color to display when the LED is ON.
#     ledColor  <string>       Current led color
#
#     and supports the following options :
#
#     flashTime  <milli-sec>   Flashing time for the flash method
#                              This determines how long the led remains
#                              in the ON/OFF states.
#     flashing   <2/1/0>       This flag drives the behaviour of the led :
#                              0 - Led is OFF if the variable is ZERO 
#                                  otherwise is ON.
#                              1 - The led reacts to the event whenever there is 
#                                  a 'write' access to the associated variable. 
#                                  In this case the led flashes no matter the
#                                  value of the variable. 
#                                  That is it goes ON for flashTime msecs and 
#                                  then OFF again.
#
#                              2 - Led is OFF if the variable is ZERO 
#                                  and as soon as the variable is NOT ZERO
#                                  it 'blinks' : that is it goes ON and OFF, 
#                                  each state lasting flashTime msecs.
#                                  
#     label      <string>      String used to label the widget
#     labelFont  <string>      New font for the label
#     labelWidth <string>      Label width : can be used to align several
#     variable   <var. name>   Variable associated to the widget               
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#    Create an instance
#    UifOutputLed .w -colorOff Gray90 -colorOn Red \
#                   -flashTime 500 -flashing 0 -label {Output Led} \
#                   -variable gvar(SCALARS.scalar_logical)
#
#    Re-configure the widget
#       .w configure -flashTime 1000  -font helvB24 -colorOff green
#
#    Other operations
#       set myVar    [.w cget -variable]   Get a widget's resource
#       set myColor  [.w cget -color]
#
#       .w flash     Make the LED flashing
#
#   SEE ALSO
#     UifWidget UifLed UifOutputWidget UifOutput UifOutputString UifOutputArray
#
#   BUGS
#
#------------------------------------------------------------------------
#

itk::usual UifOutputLed { 
  keep -background
}

class UifOutputLed {
    inherit UifOutputWidget

    constructor {args} {}

    destructor {
       if {! [cequal $afterId ""]} { after cancel $afterId }
    }

    itk_option define -flashTime flashTime FlashTime 500
    itk_option define -variable  variable Variable  ""
    itk_option define -flashing flashing  Flashing  0
    itk_option define -label    label     Label     ""

    public method flash {}

    # Keeps the led blinking as long as the variable is not zero

    public method blink { state }

    # Keeps  This method is used by the 'flash' method and hence
    # it can be called with a given delay, so better check if the widget
    # still exists.

    public method turnOn {} {  
       if {[winfo exists  $itk_component(uifOutputLed)]} {
          configure -ledColor $itk_option(-colorOn)
       }
       update idletasks
    }

    # Turns the LED off

    public method turnOff {} {
       if {[winfo exists  $itk_component(uifOutputLed)]} {
          configure -ledColor $itk_option(-colorOff)
       }
       update idletasks
    }

    public method setValue { value } 

    protected variable afterId   ""
    protected variable blinkON    0
}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifOutputLed::constructor {args} {

   itk_component add uifOutputLed {
      UifLed $itk_interior.uifOutputLed
   } {
     keep -background
     keep -colorOn -colorOff -ledColor
   }

   itk_component add label {
      label $itk_interior.label -anchor w -relief flat
   } {
     keep -background
     rename -width -labelWidth labelWidth LabelWidth
     rename -font  -labelFont labelFont LabelFont
     rename -foreground -labelColor labelColor LabelColor
   }

   pack $itk_component(label) $itk_component(uifOutputLed) -side left -fill x

   eval itk_initialize
   set initialized 1
   eval configure $args
}

#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------

body UifOutputLed::setValue { refValue } {

#    puts "DBG> UifOutputLed::setValue  $refValue"

    if { $itk_option(-flashing) == 1 } {
	flash;
    } else {

	if { $refValue == 0} {
	    turnOff
	    set blinkON 0 
	} else {
	    if { $itk_option(-flashing) == 2 } {
		if {$blinkON == 0} {
		    set blinkON 1 
		    blink ON ;# Start blinking
		}
	    } else { 
		turnOn;
	    }
	}
    }
}


#---------------------------------------------------------------
# OPTION : -flashTime
#---------------------------------------------------------------

configbody  UifOutputLed::flashTime {

    if {$itk_option(-flashTime) == 0} {
	set itk_option(-flashTime) 1000
    }
}


#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody  UifOutputLed::variable {

   if {$initialized == 0 } { return }

   addToDataList $itk_option(-variable)
   setTraceVar   [list $itk_option(-variable)]

   if { ! [cequal $itk_option(-variable) ""]} {
      setValue [uplevel #0 set [list $itk_option(-variable)]]
   }
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------

configbody UifOutputLed::label {
    if {[cequal $itk_option(-label) ""] } {
       pack forget $itk_component(label)
    } else {
       $itk_component(label) configure -text $itk_option(-label)
       pack $itk_component(label) -side left -before $itk_component(uifOutputLed)
    }
}

#---------------------------------------------------------------
# METHOD : flash
#---------------------------------------------------------------

body UifOutputLed::flash {} {
    turnOff    ;# Just to reset on condition if two consecutive '1' have been received
    turnOn;
    set afterId [after $itk_option(-flashTime) $this turnOff]
}


#---------------------------------------------------------------
# METHOD : blink
#---------------------------------------------------------------

body UifOutputLed::blink { state } {

    set val [uplevel #0 set [list $itk_option(-variable)]]

    if { $itk_option(-flashing) == 2 && $val != 0 } {

	if { [cequal $state ON] } {
            turnOn
            after $itk_option(-flashTime) $this blink OFF
	} else { 
            turnOff
            after $itk_option(-flashTime) $this blink ON
	}
    } else {
	set blinkON 0
	turnOff
    }

    update
    return
}
