#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifOutputList.tcl,v 3.36 2003/01/20 10:34:05 vltsccm Exp $" 
#
# uifOutput.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# rschmutzer 28/09/94   created 
# mcomin     07/11/94   Add methods and documentation
# mcomin     14/02/95   Improved class
# mcomin     15/02/95   Add public variable format
# mcomin     27/04/95   Took away publ. var. length
# mcomin     31/07/95   Use args instead of config
# mcomin     05/10/95   Add public variables for the event mechanism
# -------------------
# mcomin     09/03/96   Ported to Tk4.0 and itcl 2.0
#            13/05/96   Update man page
#            14/01/97   Add -units option
#            12/12/97   Add options -label, -labelWidth, -labelFont
#
#************************************************************************
#   NAME
#     UifOutputList - Class for VLT output widget
#   SYNOPSIS
#     UifOutputList  <widget-name>
#   DESCRIPTION
#     UifOutputList defines the class for the VLT output widget which allows the 
#     user to display values. The user can also define a set of keywords for which
#     the value is displayed with a different color.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifOutput - Label widget
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifOutputWidget class and supports :
#
#     anchor     <value>    Determine where the value is displayed within
#                           the widget -.e.g.- centered, left or right 
#                           justified, all tcl/tk values for 'anchor' are valid.
#     color      <string>   Text Color
#     font       <string>   Text Font 
#     frame      <1/0>      Switch on/off the frame around the widget.
#     label      <string>   String used to label the widget
#     labelFont  <string>   New font for the label
#     labelWidth <integer>  Label width : can be used to align several
#                           widgets with different labels in columns
#     listKey    <string>   List of string keywords for which the output
#                           value changes the color.
#     listColor  <string>   List of colors associated to the different
#                           keywords. If there are more keywords than colors,
#                           the last keywords will be displayed in the widget's
#                           default color.
#     units      <string>   Text appended at the right side describing the
#                           nature of the displayed value.
#     variable   <string>   Name of the variable associated to the widget 
#     width      <integer>  Widget length
#   CAUTIONS
#
#   EXAMPLES
#
#    Create an instance
#
#    UifOutputList  .w -variable gvar(deviceStatus) -color Blue -frame 1\
#               -width 21 -font CourR14 -listKey {normal warning error} \
#               -listColor { green orange red }
#
#    Re-configure the widget
#       .w configure -frame 0  -font helvB24 -color green
#
#     Other operations
#       set myVar    [.w cget -variable]   Get a widget's resource
#       set myColor  [.w cget -color]
#   SEE ALSO
#     UifOutputWidget UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifOutputList  { }

class UifOutputList {
    inherit UifOutput

    constructor {args} {}

    itk_option define -listColor listColor  ListColor  {}
    itk_option define -listKey   listKey    ListKey    {}
    itk_option define -variable  variable Variable     ""

    public method setValue { value } 

    private variable numKeywords ""
}; #end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifOutputList::constructor {args} {
   eval itk_initialize
   set initialized 1
   eval configure $args
}

#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------

body UifOutputList::setValue { refValue } {

#    puts "DBG> UifOutputList::setValue  $refValue"

    set keyFound 0
    set idx 0

    while {$idx < $numKeywords && $keyFound == 0 } {
	set  key [lindex $itk_option(-listKey) $idx]

        if {[cequal $key $refValue]} {
	    set fc [lindex $itk_option(-listColor) $idx]
	    if {[cequal $fc ""]} { 
		set fc $itk_option(-color) 
	    }
	    set keyFound 1
	}

        incr idx
    }

    if { $keyFound == 0 } {
	set fc $itk_option(-color)
    }

    $itk_component(uifOutput) configure -foreground $fc
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody  UifOutputList::variable {

   if {$initialized == 0 } { return }

   addToDataList $itk_option(-variable)
   setTraceVar   $itk_option(-variable)

   if { ! [cequal $itk_option(-variable) ""]} {
      setValue [uplevel #0 set $itk_option(-variable)]
   }
}


#---------------------------------------------------------------
# OPTION : 
#---------------------------------------------------------------

configbody  UifOutputList::listKey {
   set numKeywords [llength $itk_option(-listKey)]
}
