#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifOutputString.tcl,v 3.36 2003/01/20 10:33:52 vltsccm Exp $" 
#
# uifOutput.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    22/03/95   created 
# M.COMIN    31/07/95   Use args instead of config
# M.COMIN    15/09/95   Trim " " from strings
# mcomin     05/10/95   Add public variables for the event mechanism
#
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            13/05/96   Update man page
#            21/04/98   Add options -label, -labelWidth, -labelFont
#
#************************************************************************
#   NAME
#     UifOutputString - Class for VLT output string widget
#   SYNOPSIS
#     UifOutputString  <widget-name>
#   DESCRIPTION
#
#     UifOutputString defines the class for the VLT output widget.
#     The widget is configured to associate a string to a range of values.
#     It compares the variable current value with those ranges to detect
#     in which range the current value falls into and display the proper 
#     string.
#
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifOutputString - Label widget
#
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifOutputWidget class and supports :
#
#     anchor  left ,center  Determine where how the value should be displayed
#             or right      within the widget.
#     font     <string>    Text Font 
#     frame    <1/0>       Switch on/off the frame around the widget.
#     label      <string>  String used to label the widget
#     labelFont  <string>  New font for the label
#     labelWidth <integer> Label width : can be used to align several
#                          widgets with different labels in columns
#     listColors  <list>   List of colors associated to the specific range.
#     listStrings <list>   List of strings associated to the specific range.
#     listValues  <list>   List of threshold values. The current value
#                          of the <variable> is compared with these values
#                          to determine the range.
#     variable <string>    Name of the variable associated to the widget
#     width    <integer>   Specifies the widget length
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifOutputString  .w -variable gvar(myVar) -width 10 \
#                         -listValues {10 15 50 100} \
#                         -listStrings {string1 string2 string3 string4} \
#                         -listColors  {Black Blue Red Red} \
#                         -font -Adobe-courier-medium-r-normal--14*
#
#     In this example the result would be :
#
#                         |  Output String   Color
#       =========================================== 
#             value <= 10 |  string1         Black
#       10 <  value <= 15 |  string1         Black  
#       15 <  value <= 50 |  string2         Blue
#       50 <  value <=100 |  string3         Red
#       100 < value       |  string4         Red 
#
#    Re-configure the widget
#        .w configure -frame 0 -font CourB24
#
#    Other operations
#        set myVar    [.w cget -variable]   Get a widget's resource
#        set myColor  [.w cget -color]
#   SEE ALSO
#     uifWidget  UifOutputWidget UifOutput UifOutputLed UifOutputArray
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifOutputString { }

class UifOutputString {
    inherit UifOutputWidget

    constructor {args} {}

    itk_option define -variable  variable  Variable  ""  
    itk_option define -frame     frame     Frame     0
    itk_option define -label     label     Label     ""
    itk_option define -listColors  listColors  ListColors  {}
    itk_option define -listStrings listStrings ListStrings {}
    itk_option define -listValues  listValues  ListValues  {}

    public method setValue { refValue }

    # Extract first element of the lists

    private variable minString   ""
    private variable minValue    0
    private variable minColor    Black

    # Extract last element of the lists

    private variable maxString   ""
    private variable maxValue    1
    private variable maxColor    Black

    # Extract length of the lists

    private variable lenValue    0

}; #end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifOutputString::constructor { args } { 

   itk_component add uifOutputString {
       label $itk_interior.uifOutputString -bd 2 -anchor nw
   } {
       rename -foreground -color color Color
       keep -width -font -relief -anchor -background
   }

   itk_component add label {
      label $itk_interior.label -anchor w -relief flat
   } {
     rename -width -labelWidth labelWidth LabelWidth
     rename -font  -labelFont labelFont LabelFont
   }

   pack $itk_component(uifOutputString)  -side left -anchor w -fill x
   itk_initialize

   # Set default configuration 

   $itk_component(uifOutputString) configure -text $minString -width 10
   eval configure -frame 1 $args
}
   
#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------

body UifOutputString::setValue { refValue } {

   if { $refValue <= $minValue} {
      set myColor $minColor 
      set myStr   $minString
   } elseif { $refValue >= $maxValue} {
      set myColor $maxColor 
      set myStr   $maxString
   } else {

      for { set i 0 } { $i < $lenValue } { incr i } {
          set val1 [lindex $itk_option(-listValues) $i];
          set val2 [lindex $itk_option(-listValues) [expr {$i + 1} ]];
      
	  if { $refValue >= $val1 && $refValue < $val2 } {
             set myStr   [lindex $itk_option(-listStrings) $i]
             set myColor [lindex $itk_option(-listColors) $i] 
             break
          }
      }
   }
   configure -color $myColor
   $itk_component(uifOutputString) configure -text $myStr
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------

configbody UifOutputString::label {
    if {[cequal $itk_option(-label) ""] } {
       pack forget $itk_component(label)
    } else {
       $itk_component(label) configure -text $itk_option(-label)
       pack $itk_component(label) -side left -before $itk_component(uifOutputString)
    }
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody  UifOutputString::variable {
   addToDataList [list $itk_option(-variable)]
   setTraceVar   [list $itk_option(-variable)]

   if { ! [cequal $itk_option(-variable) ""]} {
      setValue [uplevel #0 set [list $itk_option(-variable)]]
   }
}

     
#---------------------------------------------------------------
# OPTION : -frame
#---------------------------------------------------------------

configbody  UifOutputString::frame {
   if { $itk_option(-frame) } {
       configure -relief ridge
   } else {
       configure -relief flat
   }
}

#---------------------------------------------------------------
# OPTION : -listColors
#---------------------------------------------------------------   

configbody UifOutputString::listColors {
   if {![lempty $itk_option(-listColors)]} {
      set minColor [lindex $itk_option(-listColors) 0]
      set maxColor [lindex $itk_option(-listColors) end]
   }
}

#---------------------------------------------------------------
# OPTION : -listStrings
#--------------------------------------------------------------- 

configbody UifOutputString::listStrings {
   if {![lempty $itk_option(-listStrings)]} {
      set minString [lindex $itk_option(-listStrings) 0]      
      set maxString [lindex $itk_option(-listStrings) end]
   }
}

#---------------------------------------------------------------
# OPTION : -listValues
#---------------------------------------------------------------

configbody UifOutputString::listValues {
   set lenValue [llength $itk_option(-listValues)]
   if {$lenValue} {
      set minValue [lindex $itk_option(-listValues) 0]
      set maxValue [lindex $itk_option(-listValues) end]
   }
}
 
