#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifScale.tcl,v 3.36 2003/01/20 10:33:50 vltsccm Exp $" 
#
# uifScale.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.Comin    07/11/94   created
# M.COMIN    15/02/95   Add orient public variable
# M.COMIN    30/03/95   Add Man Page
# M.COMIN    31/07/95   Use args instead of config
# M.COMIN    28/09/95   Scale creates the variable ... if it does not exist
# M.COMIN    21/11/95   Add check on value data type
#
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            03/05/96   Add get method
#            14/05/96   Update man page 
#            02/08/2001 Force config. parameter -digit 0 
#
#************************************************************************
#   NAME
#     UifScale - Class for VLT scale widget
#   SYNOPSI
#     UifScale  <widget-name>
#   DESCRIPTION
#     UifScale defines the class for the VLT scale widget. 
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifScale - Scale widget
#   METHODS
#
#     Inherits methods of class UifInteractiveWidget and supports 
#     the following :
#
#     setValue <value>
#       Sets the scale cursor to <value>
#       This can be used to set the scale to some default values.
#
#     get
#       Returns the current scale value.
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifIntercativeWidget classe and supports :
#
#     label    <string>               String used to label the widget
#     length   <integer>              Widget length
#     maxVal   <integer/float>        Maximum value of the scale 
#     minVal   <integer/float>        Minimum vale  of the scale 
#     orient   <horizontal/vertical > Determine the widget orinetation.
#     variable <string>             Variable associated to the widget
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifScale .w -maxVal 29 -minVal -36 -variable gvar(myScale) \
#                 -orient h -length 100 -label Scale
#
#     Re-configure the widget
#       .w configure -label "new Label" -minVal  -20.5 -maxVal 37.4
#
#     Other operations
#       set label   [.w cget -label]       Get a widget's resource
#       set orient  [.w cget -orient]
#   SEE ALSO
#    UifWidget UifInteractiveWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifScale { }

class UifScale {
    inherit UifInteractiveWidget

    constructor {args} {}
    destructor {}

    itk_option define -variable variable Variable ""
    itk_option define -minVal   minVal   MinVal   0
    itk_option define -maxVal   maxVal   MaxVal   100

    public method setValue {value}
    public method get      {} {return [$itk_component(uifScale) get]}

    public method varTraceOff {}
    public method varTrace    {name elem op}

    private variable oldVariable ""
    private variable interactiveFlag 0

}; # end of class


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifScale::constructor {args} {
   itk_component add uifScale {
      scale $itk_interior.uifScale  -digits 0 -highlightthickness 0 -sliderlength 25 -length 100
   } { 
      keep -command -background -state -label -orient -length -activebackground -digits -font
   }

   pack $itk_component(uifScale)
   eval itk_initialize 
   set initialized 1

   eval configure -orient h $args

   # Re-configure option -digit otherwise it is not taken.
   configure -digits [$this cget -digits]

}

#------------------------------------------------------
# Destructor
#------------------------------------------------------

body UifScale::destructor {} {
   if {! [cequal $itk_option(-variable) ""]} {
      varTraceOff 
   }
}

#---------------------------------------------------------------
# METHOD : varTraceOff - Disable callback
#---------------------------------------------------------------

body UifScale::varTraceOff { } {
#   puts "UifScale::varTraceOff  $oldVariable"
   uplevel #0 trace vdelete [list $oldVariable] w  \"$wthis varTrace\"
}

#---------------------------------------------------------------
# METHOD : varTrace
#
# If this interactiveFlag is not zero, the value was changed interactively, 
# therefore DO NOT update the scale ... otherwise there is an endless loop.
#---------------------------------------------------------------

body UifScale::varTrace { name elem  op } {

   if {!$interactiveFlag} {
      if { [cequal $elem ""] } {
         $this setValue [uplevel #0 set [list ${name}]]
      } else {
         $this setValue [uplevel #0 set [list ${name}]($elem)]
      }
   }
}


#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody UifScale::variable {

   if {[cequal $itk_option(-variable) ""]} {
      $itk_component(uifScale) configure -command ""
   } else {
       $itk_component(uifScale) configure -command "uplevel #0 set {[list $itk_option(-variable)]}"
      if {[uplevel #0 info exists [list $itk_option(-variable)]] } { 
         setValue [uplevel #0 set [list $itk_option(-variable)]] 
      }


      if {![cequal $itk_option(-variable) $oldVariable] } {
         uplevel #0 trace variable [list $itk_option(-variable)] w \"$wthis varTrace\"
      }
   }

   if {![cequal $itk_option(-variable) $oldVariable] } {
       if {![cequal $oldVariable ""]}  { varTraceOff }
       set oldVariable [list $itk_option(-variable)]
   }
}


#---------------------------------------------------------------
# OPTION : -minVal
#---------------------------------------------------------------
 
configbody UifScale::minVal {

   set dlen [string len $itk_option(-maxVal)]; incr dlen

   $itk_component(uifScale) configure -digit $dlen -from $itk_option(-minVal) \
                            -tickinterval [expr {$itk_option(-maxVal) - $itk_option(-minVal)} ]
}

#---------------------------------------------------------------
# OPTION : -maxVal
#---------------------------------------------------------------
    
configbody UifScale::maxVal {

   set dlen [string len $itk_option(-maxVal)]; incr dlen

   $itk_component(uifScale) configure -digit $dlen -to $itk_option(-maxVal) \
                            -tickinterval [expr {$itk_option(-maxVal) - $itk_option(-minVal)} ]
}


#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------

body UifScale::setValue {value} {
   set interactiveFlag 1 ;# Disable varTrace callback to avoid loops

   if {[cequal $value ""]} {   
      $itk_component(uifScale) set 0
   } else {
      $itk_component(uifScale) set $value   
   }

   set interactiveFlag 0 ;# Enable varTrace callback
}

#___oOo___
