#***********************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifShortHelp.tcl,v 3.36 2003/01/20 10:33:50 vltsccm Exp $" 
#
# uifShortHelp.tcl
#
# who         when      what
# ----------  --------  ----------------------------------------------
# rschmutzer  28/09/94  converted into a class
#
# mcomin      27/02/96  Ported to TK4.0 and incrTk
#                       Add procedure displayText
#             13/05/96  Set 'usual' to empty
#             27/09/96  Add heart beat
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#
#************************************************************************
#   NAME
#     UifShortHelp - Class for VLT Short Help Text
#   SYNOPSIS
#     UifShortHelp <widget-name>
#   DESCRIPTION
#     UifShortHelp defines the class implementing the VLT short help text.
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     shortHelp - Label displaying the short help text
#     heartBeat - Label containing the heart beat bitmap
#   METHODS
#
#     displayText <text>
#        Displays a<text> in the short help widget. This can be used to
#        convey information about specific conditions or results to the user.
#
#     getHelpVariable
#        Returns the name of the variable associated to the short help widget.
#
#     The is also a procedure used to add an help text to a given widget
#
#     add <shWidget> <widget> <text>
#        Configures the <widget> to display a short help <text> in the
#        short help widget <shWidget>
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     showHeartBeat  0/1   Bollean variable enabling the heart-beat
#   SPECIFIC BINDINGS
#     <Any-Enter>   Displys an help text
#     <Any-Leave>   Clear the short help
#   CAUTIONS
#
#   EXAMPLES
#     Create an instance of short help widget
#
#       set w .top.sh
#       UifShortHelp $w -showHeartBeat 0
#       pack $w -side top -fill x
#
#     Configure a widget <wdg> to display the short help.
#       UifShortHelp $w $wdg "Widget short help"
#
#     Display a user's message in the short help widget
#       $w displayText "Hello World"
#   SEE ALSO
#     UifIntercativeWidget  UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#

itk::usual UifShortHelp { }

class UifShortHelp {
    inherit itk::Widget

    constructor {args} {}   

    destructor {
       foreach id $afterId {
          after cancel $id
       }
    }

    itk_option define -showHeartBeat showHeartBeat ShowHeartBeat 0
    itk_option define -dbBitmap      dbBitmap      DbBitmap      ""

    public method display         { widget }
    public method hide            {}
    public method displayText     {text}
    public method heartBeat       {}
    public method showHeartBeat   { bitmapFile }
    public method disableHeart    {}

    public method getHelpVariable {} {
        return [$itk_component(shortHelp) cget -textvariable]
    }

    #---------------------------------------------------
    # PROC : add
    #
    # Configures the short help text for a given widget
    #
    # Example:  UifShortHelo::add .quit "Quit the application"
    #---------------------------------------------------

    proc add { area widget text } {
	global ::tcl_version
	# Give warning if Short Help Area does not exist   	

        if {[cequal $text ""]} { return }

        if {[cequal $area ""]} {
           if { [info exists ::shArea] == 0 } {
              puts "Short help Area NOT defined \n"
              return
           }
           set area [uplevel #0 set shArea]
        }

        if { [winfo exists $area] == 0 } {
           puts "Short help Area ($area) does NOT exists \n"
           return
        }

        set w [namespace tail $widget]
	set help($w) $text

        bind $w <Any-Enter> "[code $area display %W]"
        bind $w <Any-Leave> "[code $area hide]"
    }

    # Not initialized common variables can be used as arrays !!!

    common help 

    private variable savedLine  ""
    private variable heartBeat1 ""
    private variable heartBeat2 ""
    private variable beat       0
    private variable afterId    {}

} ; # end of class definition

#-----------------------------------------------------------
# Constructor
# constructor ?-text <text>? ?args?
#
# Creates a short help area as a ``label with a border''. You can
# create multiple short help area within the same application, if
# that makes any sense to you.
#
# Args:
#   -text	Initial text to be displayed
#   args    Optional args to the outer frame
#
# Returns:
#   Path name of the inner label.
#
# Example:
#        UifShortHelp .sha -text "This is an example short help area"
#
#----------------------------------------------------------

body UifShortHelp::constructor { args } {
   global ::shortHelp_line

#   puts "DBG > UifShortHelp::constructor .."
   component hull configure -borderwidth 1m -relief raised

   itk_component add f1 {
       frame $itk_interior.f1
   }

   itk_component add f2 {
       frame $itk_interior.f2
   }

   itk_component add shortHelp {
       label $itk_component(f1).shortHelp -relief sunken -anchor w 
   } {
       keep -textvariable 
   }

   itk_component add dbRefresh {
       button $itk_component(f2).dbRefresh -relief raised -anchor w
   } {
       keep -command
   }

   itk_component add heartBeat {
       label $itk_component(f2).heartBeat -foreground Red -anchor w -relief groove
   } {
       keep -bitmap
   }

   pack $itk_component(f2) -side right
   pack $itk_component(f1) -side left -expand yes -fill x

   pack $itk_component(shortHelp) -side left  -expand yes -fill x

   eval itk_initialize

   set shortHelp_line($this) ""
   $itk_component(shortHelp) configure -textvariable shortHelp_line($this)
  
   set heartBeat1 [findVLTFile bitmaps/uifHeartBeat1.xbm]
   set heartBeat2 [findVLTFile bitmaps/uifHeartBeat2.xbm]

   heartBeat

   eval configure $args
   
#   puts "DBG> UifShortHelp End .."   
}

#---------------------------------------------------------------
# METHOD : disableHeart
#---------------------------------------------------------------

body UifShortHelp::disableHeart {} {
    foreach id $afterId {
       after cancel $id
    }
    update
}


#---------------------------------------------------------------
# METHOD : beat
#---------------------------------------------------------------

body UifShortHelp::heartBeat {} {

   configure -bitmap "@${heartBeat2}"
   update

   set id1 [after 200 $this showHeartBeat $heartBeat1]
   set id2 [after 400 $this showHeartBeat $heartBeat2]
   set id3 [after 600 $this showHeartBeat $heartBeat1]

   set afterId "$id1 $id2 $id3"
}

#---------------------------------------------------------------
# METHOD : showHeartBeat 
#---------------------------------------------------------------

body UifShortHelp::showHeartBeat {bitmapFile} {
   if {$itk_option(-showHeartBeat) != 0 } {
      catch {$itk_component(heartBeat) configure -bitmap "@${bitmapFile}" }
   } 
   update
}

#---------------------------------------------------------------
# METHOD : display
#---------------------------------------------------------------

body UifShortHelp::display { widget } {
   global ::shortHelp_line
   catch { set shortHelp_line($this) $help($widget) }
}

#---------------------------------------------------------------
# METHOD : hide
#---------------------------------------------------------------

body UifShortHelp::hide {} {
   global ::shortHelp_line
   set shortHelp_line($this) $savedLine
}


#---------------------------------------------------------------
# METHOD : displayText
#
# Sets the contents of a short help area.
#
# Example:  UifShortHelo::set "Quit the application"
#
#---------------------------------------------------------------


body UifShortHelp::displayText {text} { 
   global ::shortHelp_line 
   catch { set shortHelp_line($this) $text }
}


#---------------------------------------------------------------
# OPTION : -showHeartBeat
#---------------------------------------------------------------

configbody  UifShortHelp::showHeartBeat {

   set w $itk_component(heartBeat)

   if {$itk_option(-showHeartBeat) == 0 } {
       if {[winfo ismapped $w]} { pack forget $w }
       disableHeart
   } else {
       if {[winfo ismapped $w] == 0} { pack $w -padx 2 -side right }
   }
}

#---------------------------------------------------------------
# OPTION : -dbBitmap
#---------------------------------------------------------------

configbody  UifShortHelp::dbBitmap {

   set w $itk_component(dbRefresh)

   if {[cequal $itk_option(-dbBitmap) ""]} {
       if {[winfo ismapped $w]} { pack forget $w }
   } else {
       set fname [findVLTFile bitmaps/$itk_option(-dbBitmap)]
       $w configure -bitmap "@${fname}"
       if {[winfo ismapped $w] == 0} { pack $w -side left}
       
   }
}




#___oOo___



